@@ -65,6 +65,8 @@ namespace ArrayCore {
     _ySampling.setRange(100.0, 1000.0);
 
     setMaximumSlowness(1.0/100.0);
+    // The full plot including aliasing is usually expected.
+    setGridSizeFactor(std::numeric_limits<double>::infinity());
 
     _beamPowerNormalization=Spectrum;
   }
