@@ -59,8 +59,15 @@ namespace ArrayCore {
     void setGridStep(double s) {_gridStep=s;}
     double gridStep() const;
 
+    void setGridStepFactor(double f) {_gridStepFactor=f;}
+    double gridStepFactor() const {return _gridStepFactor;}
+
     void setGridSize(double s) {_gridSize=s;}
     double gridSize() const;
+
+    void setGridSizeFactor(double f) {_gridSizeFactor=f;}
+    double gridSizeFactor() const {return _gridSizeFactor;}
+
     double effectiveGridSize() const;
     double effectiveGridSize(double frequency) const;
 
