@@ -48,7 +48,6 @@ namespace ArrayCore {
     TRACE;
     _binCount=200;
     _minimumSlowness=0.0;
-    _mode=ArrayStations::Vertical;
   }
 
   /*!
@@ -68,7 +67,6 @@ namespace ArrayCore {
   void FKResults::setArray(const ArraySelection * array)
   {
     AbstractArrayResults::setArray(array);
-    _mode=_array->array()->mode();
     // Set a "good" time reference as close as possible to the effective time range
     // Avoid loss of time precision in storage of peaks... finally not so important.
     _peaks.setTimeReference(_array->array()->first()->minTime());
@@ -83,7 +81,7 @@ namespace ArrayCore {
     SamplingParameters slowParam;
     slowParam.setMinimum(_minimumSlowness);
     slowParam.setMaximum(parameters()->maximumSlowness());
-    slowParam.setType(SamplingParameters::Log);
+    slowParam.setScaleType(SamplingParameters::Log);
     slowParam.setCount(_binCount); // Required only for log
     h->setYSampling(slowParam);
     App::log(tr("Slowness histogram\n"
@@ -105,7 +103,7 @@ namespace ArrayCore {
     SamplingParameters slowParam;
     slowParam.setMinimum(_minimumSlowness);
     slowParam.setMaximum(parameters()->maximumSlowness());
-    slowParam.setType(SamplingParameters::Log);
+    slowParam.setScaleType(SamplingParameters::Log);
     h->setYSampling(slowParam);
     return h;
   }
@@ -162,18 +160,18 @@ namespace ArrayCore {
     int n=c.count();
     if(n>1) {
       for(int i=2; i<n; i++) {
-        double nl=fabs((c.at(i-2).y()+
-                        (c.at(i-1).x()-c.at(i-2).x())
-                        /(c.at(i).x()-c.at(i-2).x())
-                        *(c.at(i).y()-c.at(i-2).y())-c.at(i-1).y())
-                        /(0.5*(c.at(i).y()+c.at(i-2).y())));
+        double nl=fabs((c.y(i-2)+
+                        (c.x(i-1)-c.x(i-2))
+                        /(c.x(i)-c.x(i-2))
+                        *(c.y(i)-c.y(i-2))-c.y(i-1))
+                        /(0.5*(c.y(i)+c.y(i-2))));
         if(nl<maxNonLinear) {
-          cr.append(c.at(i-1));
+          cr.append(c.constAt(i-1));
         }
       }
     }
-    App::log(tr("    %1 samples set as invalid\n").arg(c.count()-cr.count()) );
-    App::log(tr("    remaining %1 samples\n").arg(cr.count()) );
+    App::log(tr("    %1 samples set as invalid\n").arg(c.count()-cr.count()));
+    App::log(tr("    remaining %1 samples (%2)\n").arg(cr.count()).arg(round(1000.0*cr.count()/c.count())*0.1));
     return cr;
   }
 
@@ -255,21 +253,10 @@ namespace ArrayCore {
       h->validateCurve(c, _maxMisfit);
     }
     _results->filterArrayLimits(c);
-    _results->_rayleighCurves.clear();
-    _results->_loveCurves.clear();
+    _results->_curves.clear();
     if(!c.isEmpty()) {
       saveCurve(c, _baseName+"-"+QGpCoreWave::Mode::userPolarization(polarization)+".disp");
-      switch (polarization) {
-      case QGpCoreWave::Mode::Vertical:
-      case QGpCoreWave::Mode::Radial:
-      case QGpCoreWave::Mode::Rayleigh:
-        _results->_rayleighCurves.append(c);
-        break;
-      case QGpCoreWave::Mode::Transverse:
-      case QGpCoreWave::Mode::Love:
-        _results->_loveCurves.append(c);
-        break;
-      }
+      _results->_curves.append(c);
     }
   }
 
@@ -330,8 +317,8 @@ namespace ArrayCore {
     for(int i=curves.count()-1; i>=0; i--) {
       Curve<RealStatisticalPoint>& c=curves[i];
       // Filling holes in the curve sampling by re-interpolation
-      double min=c.at(c.minimumX()).x();
-      double max=c.at(c.maximumX()).x();
+      double min=c.x(c.minimumX());
+      double max=c.x(c.maximumX());
       double step=c.minimumStep(LogScale);
       int n=1+qRound(log(max/min)/log(step));
       c.resample(n, min, max, LogScale);
@@ -346,112 +333,64 @@ namespace ArrayCore {
     // When switching between the 2 alternatives do not forget to change fill functions
     Histogram2D * h=initSlownessHistogram();
     //Histogram2D * h=initSlownessWaveNumberHistogram();
-    switch(_mode) {
-    case ArrayStations::Vertical:
-      App::log(tr("%1 for vertical ...\n").arg(a->title()));
-      _peaks.fillSlowness(h, QGpCoreWave::Mode::Vertical);
-      //_peaks.fillSlownessWaveNumber(h, QGpCoreWave::Mode::Vertical);
-      a->run(h, QGpCoreWave::Mode::Vertical);
-      _peaks.commit(h, QGpCoreWave::Mode::Vertical);
-      break;
-
-    case ArrayStations::Horizontal:
-      App::log(tr("%1 for radial ...\n").arg(a->title()) );
-      _peaks.fillSlowness(h, QGpCoreWave::Mode::Radial);
-      //_peaks.fillSlownessWaveNumber(h, QGpCoreWave::Mode::Radial);
-      a->run(h, QGpCoreWave::Mode::Radial);
-      _peaks.commit(h, QGpCoreWave::Mode::Radial);
-      delete h;
-
-      h=initSlownessHistogram();
-      //h=initSlownessWaveNumberHistogram();
-      App::log(tr("%1 for transverse ...\n").arg(a->title()) );
-      _peaks.fillSlowness(h, QGpCoreWave::Mode::Transverse);
-      a->run(h, QGpCoreWave::Mode::Transverse);
-      _peaks.commit(h, QGpCoreWave::Mode::Transverse);
-      break;
-
-    case ArrayStations::ThreeComponents:
-      App::log(tr("%1 for Rayleigh ...\n").arg(a->title()) );
-      _peaks.fillSlowness(h, QGpCoreWave::Mode::Rayleigh);
-      a->run(h, QGpCoreWave::Mode::Rayleigh);
-      _peaks.commit(h, QGpCoreWave::Mode::Rayleigh);
-      delete h;
-
-      h=initEllipticityHistogram();
-      App::log(tr("%1 for ellipticity ...\n").arg(a->title()) );
-      _peaks.fillEllipticity(h);
-      a->run(h);
-      _peaks.commit(h, QGpCoreWave::Mode::Rayleigh);
-      delete h;
-
-      h=initSlownessHistogram();
-      //h=initSlownessWaveNumberHistogram();
-      App::log(tr("%1 for Love ...\n").arg(a->title()) );
-      _peaks.fillSlowness(h, QGpCoreWave::Mode::Love);
-      //_peaks.fillSlownessWaveNumber(h, QGpCoreWave::Mode::Love);
-      a->run(h, QGpCoreWave::Mode::Love);
-      _peaks.commit(h, QGpCoreWave::Mode::Love);
-      break;
-    }
+    App::log(tr("%1...\n").arg(a->title()));
+    _peaks.fillSlowness(h);
+    //_peaks.fillSlownessWaveNumber(h);
+    a->run(h);
+    _peaks.commit(h);
     delete h;
   }
 
   void FKResults::aviosPick(const QString &groupName)
   {
     TRACE;
-    Histogram2D * h;
-    switch(_mode) {
-    case ArrayStations::Vertical: {
-        h=initWaveNumberHistogram();
-        _peaks.fillWaveNumber(h, QGpCoreWave::Mode::Vertical);
-        h->save(groupName+"-beforefilter.max");
-        App::log(tr("Filter hit count=1 for vertical ...\n"));
-        h->filterThreshold(1.0);
-        App::log(tr("Filter 20% from max for vertical ...\n"));
-        h->normalizeAmplitude(YAxis);
-        h->filterThreshold(0.2);
-        h->save(groupName+"-filter.max");
-        App::log(tr("Gaussian mixture for vertical ...\n"));
-        GaussianMixture gm(this, groupName);
-        _rayleighCurves=gm.scan(h, Mode::Rayleigh);
-        _peaks.reset(QGpCoreWave::Mode::Vertical);
-        for(int i=_rayleighCurves.count()-1; i>=0; i--) {
-          h->filterCurveOutsideSigmaRange(_rayleighCurves.at(i), 1.0);
-          _peaks.commit(h, QGpCoreWave::Mode::Vertical);
-          save((groupName+"-rayleigh_%1.max").arg(i, 2, 10, QChar('0')));
-          delete h;
-          h=initSlownessHistogram();
-          _peaks.fillSlowness(h, QGpCoreWave::Mode::Vertical);
-          _rayleighCurves[i]=h->medianCurve();
-          filterArrayLimits(_rayleighCurves[i], 1.0, 0.75);
-          saveCurve(_rayleighCurves[i], (groupName+"-rayleigh-median_%1.disp").arg(i, 2, 10, QChar('0')));
-          delete h;
-          _peaks.reset(QGpCoreWave::Mode::Vertical);
-          h=initWaveNumberHistogram();
-          _peaks.fillWaveNumber(h, QGpCoreWave::Mode::Vertical);
-        }
-        delete h;
+    /*Histogram2D * h;
+    h=initWaveNumberHistogram();
+    _peaks.fillWaveNumber(h);
+    h->save(groupName+"-beforefilter.max");
+    App::log(tr("Filter hit count=1 for vertical ...\n"));
+    h->filterThreshold(1.0);
+    App::log(tr("Filter 20% from max for vertical ...\n"));
+    h->normalizeAmplitude(YAxis);
+    h->filterThreshold(0.2);
+    h->save(groupName+"-filter.max");
+    App::log(tr("Gaussian mixture for vertical ...\n"));
+    GaussianMixture gm(this, groupName);
+    _curves=gm.scan(h, Mode::Rayleigh);
+    _peaks.reset(QGpCoreWave::Mode::Vertical);
+    for(int i=_curves.count()-1; i>=0; i--) {
+      h->filterCurveOutsideSigmaRange(_curves.at(i), 1.0);
+      _peaks.commit(h, QGpCoreWave::Mode::Vertical);
+      save((groupName+"-rayleigh_%1.max").arg(i, 2, 10, QChar('0')), QString());
+      delete h;
+      h=initSlownessHistogram();
+      _peaks.fillSlowness(h, QGpCoreWave::Mode::Vertical);
+      _curves[i]=h->medianCurve();
+      filterArrayLimits(_curves[i], 1.0, 0.75);
+      saveCurve(_curves[i], (groupName+"-rayleigh-median_%1.disp").arg(i, 2, 10, QChar('0')));
+      delete h;
+      _peaks.reset(QGpCoreWave::Mode::Vertical);
+      h=initWaveNumberHistogram();
+      _peaks.fillWaveNumber(h, QGpCoreWave::Mode::Vertical);
+    }
+    delete h;
       }
       break;
-    case ArrayStations::Horizontal:
-      break;
-    case ArrayStations::ThreeComponents:
-      break;
-    }
+    }*/
   }
 
   /*!
-   If \a baseName does not have any extension, it is added automatically
+   If \a baseName does not have any extension, it is added automatically.
+   \a duration of process in milliseconds
   */
-  bool FKResults::save(QString fileName)
+  bool FKResults::save(QString fileName, const QString& log)
   {
     TRACE;
     _peaks.sort();
     if(!fileName.endsWith(".max")) {
       fileName+=".max";
     }
-    if(!_peaks.save(fileName, *parameters(), *_array)) {
+    if(!_peaks.save(fileName, parameters(), _array, &log)) {
       App::log(tr("Error saving results to '%1'.\n").arg(fileName));
       return false;
     }
@@ -480,7 +419,7 @@ namespace ArrayCore {
       s.setRealNumberPrecision(20);
       int n=c.count();
       for(int i=0; i<n; i++) {
-        s << c.at(i).toString(20) << "\n";
+        s << c.constAt(i).toString(20) << "\n";
       }
       return true;
     } else {
@@ -522,7 +461,7 @@ namespace ArrayCore {
     double kmax=kmaxFactor*parameters()->kmax();
     int  n=c.count();
     for(int i=0; i<n; i++) {
-      RealStatisticalPoint &p=c[i];
+      RealStatisticalPoint &p=c.constXAt(i);
       if(p.isValid()) {
         double k=2.0*M_PI*p.x()*p.mean();
         if(k<kmin || k>kmax) {
@@ -545,26 +484,14 @@ namespace ArrayCore {
 
     Restrict search around the median curves with a possible variation of \a fraction.
   */
-  void FKResults::setCurveRange(FKParameters& param, const Curve<RealStatisticalPoint>& curve, double fraction,
-                                QGpCoreWave::Mode::Polarization polarization) const
+  void FKResults::setCurveRange(FKParameters& param, const Curve<RealStatisticalPoint>& curve, double fraction) const
   {
     TRACE;
     App::log(tr("Restricting search around curves (<%1 stddev)...\n").arg(fraction) );
 
     param.clearCurveRefine();
-    switch (polarization) {
-    case QGpCoreWave::Mode::Vertical:
-    case QGpCoreWave::Mode::Radial:
-    case QGpCoreWave::Mode::Rayleigh:
-      param.setRefineGridIndex(0);
-      break;
-    case QGpCoreWave::Mode::Transverse:
-    case QGpCoreWave::Mode::Love:
-      param.setRefineGridIndex(1);
-      break;
-    }
     for(int i=0; i<curve.count(); i++) {
-      const RealStatisticalPoint& p=curve.at(i);
+      const RealStatisticalPoint& p=curve.constAt(i);
       if(p.isValid()) {
         param.addCurveRefine(p.x(), p.mean()-fraction*p.stddev(), p.mean()+fraction*p.stddev());
       }
@@ -583,12 +510,13 @@ namespace ArrayCore {
       const Curve<RealStatisticalPoint>& cc=c;
       // Extrapolate the curve until reaching the kmin and kmax limits
       bool extrapolated=true;
-      while(2.0*M_PI*cc.first().x()*cc.first().y()>kmin && extrapolated) {
+      while(2.0*M_PI*cc.firstX()*cc.firstY()>kmin && extrapolated) {
         // too risky to extrapolate small curves and decreasing slopes, skip extrapolation.
         extrapolated=c.extrapolateFromStart(4, true, LogScale);
       }
       extrapolated=true;
-      while(2.0*M_PI*cc.last().x()*cc.last().y()<kmax && extrapolated) {
+      int n=cc.count()-1;
+      while(2.0*M_PI*cc.x(n)*cc.y(n)<kmax && extrapolated) {
         // too risky to extrapolate small curves and decreasing slopes, skip extrapolation.
         extrapolated=c.extrapolateFromEnd(4, true, LogScale);
       }
@@ -598,15 +526,15 @@ namespace ArrayCore {
         exit(2);
       }
       App::log(1, c.toString()+"\n");
-      f=c.at(c.minimumX()).x();
+      f=c.x(c.minimumX());
       if(f<fMin) {
         fMin=f;
       }
-      f=c.at(c.maximumX()).x();
+      f=c.x(c.maximumX());
       if(f>fMax) {
         fMax=f;
       }
-      s=c.at(c.maximumY()).y();
+      s=c.y(c.maximumY());
       if(s>sMax) {
         sMax=s;
       }
@@ -623,31 +551,20 @@ namespace ArrayCore {
   bool FKResults::setRange(FKParameters& param) const
   {
     TRACE;
-    double fMinR=std::numeric_limits<double>::infinity(), fMaxR=-std::numeric_limits<double>::infinity(), sMaxR=-std::numeric_limits<double>::infinity();
-    double fMinL=std::numeric_limits<double>::infinity(), fMaxL=-std::numeric_limits<double>::infinity(), sMaxL=-std::numeric_limits<double>::infinity();
-    range(_rayleighCurves, fMinR, fMaxR, sMaxR);
-    range(_loveCurves, fMinL, fMaxL, sMaxL);
+    double fMin=std::numeric_limits<double>::infinity(),
+           fMax=-std::numeric_limits<double>::infinity(),
+           sMax=-std::numeric_limits<double>::infinity();
+    range(_curves, fMin, fMax, sMax);
 
-    if((fMinR==std::numeric_limits<double>::infinity() || fMaxR==-std::numeric_limits<double>::infinity() || sMaxR==-std::numeric_limits<double>::infinity()) &&
-       (fMinL==std::numeric_limits<double>::infinity() || fMaxL==-std::numeric_limits<double>::infinity() || sMaxL==-std::numeric_limits<double>::infinity())) {
-      App::log(tr("Rayleigh and Love curves are empty, cannot set frequency range.\n") );
+    if(fMin==std::numeric_limits<double>::infinity() ||
+       fMax==-std::numeric_limits<double>::infinity() ||
+       sMax==-std::numeric_limits<double>::infinity()) {
+      App::log(tr("Curves are empty, cannot set frequency range.\n") );
       return false;
     }
 
-    // We are trying to obtain the largest frequency range that fit with Rayleigh and Love
-    double fMin=fMinR, fMax=fMaxR, sMax=sMaxR;
-    if(fMinL<fMin) {
-      fMin=fMinL;
-    }
-    if(fMaxL>fMax) {
-      fMax=fMaxL;
-    }
-    if(sMaxL>sMax) {
-      sMax=sMaxL;
-    }
-
     SamplingParameters& samplingParam=param.frequencySampling();
-    samplingParam.setType(SamplingParameters::Log);
+    samplingParam.setScaleType(SamplingParameters::Log);
     if(fMin<samplingParam.minimum()) {
       fMin=samplingParam.minimum();
     }
@@ -664,10 +581,8 @@ namespace ArrayCore {
     App::log(tr("Minimum velocity %1 m/s\n")
                      .arg(1.0/param.maximumSlowness()));
 
-    param.setMinimumFrequency(0, fMinR); // Rayleigh
-    param.setMaximumFrequency(0, fMaxR);
-    param.setMinimumFrequency(1, fMinL); // Love
-    param.setMaximumFrequency(1, fMaxL);
+    param.setMinimumFrequency(fMin);
+    param.setMaximumFrequency(fMax);
     return true;
   }
 
@@ -684,28 +599,16 @@ namespace ArrayCore {
     QDir d=fi.absoluteDir();
     QStringList filters, files;
 
-    _rayleighCurves.clear();
+    _curves.clear();
     filters << fi.baseName()+"-Rayleigh-??.disp";
     files=d.entryList(filters);
     for(QStringList::iterator it=files.begin(); it!=files.end(); it++) {
       Curve<RealStatisticalPoint> c;
       if(loadCurve(c, d.absoluteFilePath(*it))) {
-        _rayleighCurves.append(c);
+        _curves.append(c);
       }
     }
-    App::log(tr("%1 curves loaded for Rayleigh\n").arg(_rayleighCurves.count()));
-
-    _loveCurves.clear();
-    filters.clear();
-    filters << fi.baseName()+"-Love-??.disp";
-    files=d.entryList(filters);
-    for(QStringList::iterator it=files.begin(); it!=files.end(); it++) {
-      Curve<RealStatisticalPoint> c;
-      if(loadCurve(c, d.absoluteFilePath(*it))) {
-        _loveCurves.append(c);
-      }
-    }
-    App::log(tr("%1 curves loaded for Love\n").arg(_loveCurves.count()) );
+    App::log(tr("%1 curves loaded\n").arg(_curves.count()));
     return true;
   }
 
