@@ -74,6 +74,20 @@ namespace DinverCore {
     delete _nav;
   }
 
+  QString Generator::type(Type t)
+  {
+    TRACE;
+    switch(t) {
+    case MonteCarlo:
+      return "Monte-Carlo";
+    case Neighborhood:
+      break;
+    case Borders:
+      return "Borders";
+    }
+    return "Neighborhood";
+  }
+
   /*!
   */
   void Generator::run()
@@ -86,6 +100,9 @@ namespace DinverCore {
     case Neighborhood:
       neighborhoodLoop();
       break;
+    case Borders:
+      bordersLoop();
+      break;
     }
   }
 
@@ -95,20 +112,14 @@ namespace DinverCore {
   {
     TRACE;
 
-    forever {
+    while(!_terminated.value()) {
       NewModel * newModel=new NewModel;
 
       monteCarlo(newModel);
-
-      if(_terminated.value()) {
-        delete newModel;
-        break;
-      } else {
-        if(newModel->isValidMisfit()) {
-          _repository->addReport(newModel, _forward);
-        }
-        _repository->addModel(newModel);
+      if(newModel->isValidMisfit()) {
+        _repository->addReport(newModel, _forward);
       }
+      _repository->addModel(newModel);
     }
   }
 
@@ -211,27 +222,10 @@ namespace DinverCore {
           ASSERT(parameterSpace.isOkDebug());
 #endif
         }
-      }
-      newModel->setModelIndex(_repository->allModels()->add(parameterSpace));
-      if(newModel->modelIndex()>-1) {
-        ASSERT(parameterSpace.isOkDebug());
-        bool ok=true;
-        newModel->setMisfit(_forward->misfit(ok));
-        newModel->setValidMisfit(ok);
-        newModel->setValidParent(true);
-        delete [] backupModel;
-        return;
-      } else {
-        // For low dimension parameter spaces (<3), the space can be fully investigated
-        // We put a threshold at 80% coverage to abort the inversion process.
-        double nPossibleModels=parameterSpace.possibleCount();
-        _repository->allModels()->lock();
-        if(_repository->allModels()->count()>0.8*nPossibleModels) {
-          _repository->allModels()->unlock();
-          _repository->stop(); // force termination of inversion
+        if(misfit(newModel, parameterSpace)) {
+          delete [] backupModel;
           return;
         }
-        _repository->allModels()->unlock();
       }
     }
   }
@@ -242,9 +236,12 @@ namespace DinverCore {
   {
     TRACE;
 
-   forever {
+    while(!_terminated.value()) {
       // Selects randomly one of the best cells
-      ParentIndex * parentIndex=_repository->createModel(_randomNumbers->ran2(), _forward->parameterSpace());
+      ParentIndex * parentIndex=_repository->createModel(_randomNumbers->ran2());
+      if(!parentIndex) {
+        break;
+      }
       if(_generatorModelsSerialNumber!=parentIndex->serialNumber()) {
         delete _nav;
         _nav=new VoronoiNavigator(parentIndex->scaledModels());
@@ -254,28 +251,30 @@ namespace DinverCore {
       newModel->setParentIndex(parentIndex);
 
       neighborhood(newModel);
-      if(_terminated.value()) {
-        delete newModel;
-        break;
-      } else {
-        if(newModel->isValidMisfit()) {
-          _repository->addReport(newModel, _forward);
-        }
-        _repository->addModel(newModel);
+      if(newModel->isValidMisfit()) {
+        _repository->addReport(newModel, _forward);
       }
+      _repository->addModel(newModel);
     }
   }
 
+  /*!
+    Does random wolks inside the parent cell of \a newModel.
+  */
   void Generator::neighborhood(NewModel * newModel)
   {
     TRACE;
-
     RealSpace& parameterSpace=_forward->parameterSpace();
+    const ParentIndex * parentIndex=newModel->parentIndex();
+    const ActiveIndex& activeIndex=parentIndex->index();
     int ndVar=parameterSpace.variableParameterCount();
     int nSteps=ndVar*_walkCount;
 
-    _nav->setCurrentPoint(newModel->parentIndex()->activeIndex());
-    _nav->setCurrentAxis(ndVar-1);
+    _repository->modelToRealSpace(activeIndex, parameterSpace);
+    _nav->setCurrentPoint(activeIndex);
+    // Randomize starting axis
+    // TODO: check the difference of classical incremental walk or random
+    _nav->initDistances(ndVar-1);
 
     // Some parametrizations may need some updates before proceeding
     _forward->valueChanged();
@@ -283,34 +282,34 @@ namespace DinverCore {
     // Do random walks, if after 10 tries no new model can be generated (different from
     // all others generated so far), the cell is considered as atomic (only one point inside).
     int iStep, iTries;
-    int xMin, xMax, iMin, iMax;
+    int xMin, xMax;
+    ActiveIndex iMin, iMax;
     for(iTries=0; iTries<10; iTries++) {
       //printf("  Try %3i\n", iTries);
       int atomicAxis=0;
-      //_parentVolume=1.0;
+      // Estimation of the cell volume
+      double parentVolume=1.0;
       for(iStep=0; iStep<nSteps; iStep++) {
         _nav->incrementAxis();
-        //printf("    Step %3i Axis %i\n", iStep, nav.currentAxis());
+        //printf("    Step %3i Axis %i\n", iStep, _nav->currentAxis());
         //_nav->checkAxisDistances();
         Parameter * p=parameterSpace.variableParameter(_nav->currentAxis());
         p->getGridLimits(xMin, xMax);
         //printf("      Parameter min %5i max %5i\n", xMin, xMax);
-        if(xMin>=xMax) {
-          atomicAxis++;
-          if(atomicAxis==ndVar && iStep==(ndVar-1)) {
-            return;
-          }
-        }
+        bool atomicBeforeCellLimits=(xMin>=xMax);
         _nav->cellLimits(xMin, xMax, iMin, iMax);
+        if(atomicBeforeCellLimits) {
+          ASSERT(xMin>=xMax);
+        }
 
         // Update volume estimation
-        //_parentVolume*=abs(xMax-xMin)+1;
+        parentVolume*=abs(xMax-xMin)+1;
 
         // Marks neighboring cells as active
-        if(iMin>-1) {
+        if(iMin.isValid()) {
           newModel->addNavigatorHit(iMin);
         }
-        if(iMax>-1) {
+        if(iMax.isValid()) {
           newModel->addNavigatorHit(iMax);
         }
 
@@ -319,7 +318,7 @@ namespace DinverCore {
         if(xMin>=xMax) {
           atomicAxis++;
           if(atomicAxis==ndVar && iStep==(ndVar-1)) {
-            return;
+            return;  // validParent is false in newModel ==> parent will be set as atomic
           }
         } else {
           if(p->isFussy()) {     // Parameters tagged as fussy need a validation of the random value
@@ -355,12 +354,12 @@ namespace DinverCore {
           }
         }
       }
-      // Average volume on all random walks
-      //_parentVolume/=_nWalks;
 
       // ModelSet has its own mutex... thread safe.
-      newModel->setModelIndex(_repository->allModels()->add(parameterSpace));
-      if(newModel->modelIndex()>-1) {
+      if(misfit(newModel, parameterSpace)) {
+        // Average volume on all random walks, but half of it because of the newly the generated model
+        // Children inherit parent volume
+        newModel->setParentVolume(0.5*parentVolume/_walkCount);
         //printf("  --- ACCEPT new model ---\n" );
         //nav.currentPoint().print();
         // Check that generated model belongs to the right cell
@@ -372,23 +371,157 @@ namespace DinverCore {
           }
           printf("\n" );
         }*/
-        //ASSERT(parameterSpace.isOkDebug());
-        bool ok=true;
-        newModel->setMisfit(_forward->misfit(ok));
-        newModel->setValidMisfit(ok);;
-        newModel->setValidParent(true);;
         return;
-      } else {
-        // For low dimension parameter spaces (<3), the space can be fully investigated
-        // We put a threshold at 80% coverage to abort the inversion process.
-        double nPossibleModels=parameterSpace.possibleCount();
-        _repository->allModels()->lock();
-        if(_repository->allModels()->count()>0.8*nPossibleModels) {
-           _repository->allModels()->unlock();
-          _repository->stop(); // force termination of inversion
-          return;
+      }
+    }
+    /*
+       After several tries, it was not possible to generate a valid model that is
+       not already in the model set. The are good chances that the current parent cell
+       is close to an atomic cell.
+
+       validParent is false in newModel ==> parent will be set as atomic
+       (see ModelRepository::processQueue())
+    */
+    return;
+  }
+
+  /*!
+  */
+  void Generator::bordersLoop()
+  {
+    TRACE;
+    _minimumMisfit=_repository->misfit(_repository->bestModelIndex());
+
+    while(!_terminated.value()) {
+      // Selects randomly one of the best cells
+      ParentIndex * parentIndex=_repository->createModel(_randomNumbers->ran2());
+      if(!parentIndex) {
+        return;
+      }
+      if(_generatorModelsSerialNumber!=_repository->serialNumber()) { // If the same, keep iterating from the last point
+        delete _nav;
+        _nav=new VoronoiNavigator(parentIndex->scaledModels());
+        _generatorModelsSerialNumber=parentIndex->serialNumber();
+      }
+      NewModel * newModel=new NewModel;
+      newModel->setParentIndex(parentIndex);
+
+      borders(newModel);
+      if(newModel->isValidMisfit()) {
+        _repository->addReport(newModel, _forward);
+      }
+      _repository->addModel(newModel);
+    }
+  }
+
+  /*!
+    Does random wolks inside the parent cell of \a newModel.
+  */
+  void Generator::borders(NewModel * newModel)
+  {
+    TRACE;
+    RealSpace& parameterSpace=_forward->parameterSpace();
+    const ParentIndex * parentIndex=newModel->parentIndex();
+    const ActiveIndex& activeIndex=parentIndex->index();
+    int ndVar=parameterSpace.variableParameterCount();
+    int nSteps=ndVar*_walkCount;
+
+    _repository->modelToRealSpace(activeIndex, parameterSpace);
+    _forward->valueChanged();
+    _nav->setCurrentPoint(activeIndex);
+    _nav->initDistances(_randomNumbers->uniform(0, ndVar-1));
+    //_nav->initDistances(ndVar-1);
+
+    // Do random walks, if after 10 tries no new model can be generated (different from
+    // all others generated so far), the cell is considered as atomic (only one point inside).
+    int iStep, iTries;
+    int xMin, xMax;
+    ActiveIndex cellIndex;
+    PdfCurve pdf;
+    for(iTries=0; iTries<10; iTries++) {
+      //printf("  Try %3i\n", iTries);
+      for(iStep=0; iStep<nSteps; iStep++) {
+        _nav->setCurrentAxis(_randomNumbers->uniform(0, ndVar-2));
+        //_nav->incrementAxis();
+        //printf("    Step %3i Axis %i\n", iStep, _nav->currentAxis());
+        _nav->checkAxisDistances();
+        Parameter * p=parameterSpace.variableParameter(_nav->currentAxis());
+        p->getGridLimits(xMin, xMax);
+        //PdfCurve pdfDebug=_nav->intersectionsDebug(xMin, xMax);
+        pdf=_nav->intersections(xMin, xMax);
+        /*int ipdf;
+        if(!pdf.compare(pdfDebug, ipdf)) {
+          _nav->setValue(pdfDebug.at(ipdf).x());
+          _nav->setCurrentCell(pdfDebug.at(ipdf).cell());
+          _nav->checkCurrentPoint(parameterSpace);
+          _nav->setCurrentCell(pdf.at(ipdf).cell());
+          _nav->checkCurrentPoint(parameterSpace);
+        }*/
+        pdf.setAcceptable(*parentIndex->activeModels(), _minimumMisfit);
+        pdf.setBorders();
+        //pdf.setStrictBorders();
+        pdf.hits(newModel);
+
+        if(p->isFussy()) {     // Parameters tagged as fussy need a validation of the random value
+          int nTries;
+          // Allow for a maximum of 100 tries before giving up
+          // This is not really a good solution but in some cases it avoids the process to be blocked
+          for(nTries=0; nTries<100; nTries++) {
+            int val=pdf.generate(_randomNumbers->ran2(), cellIndex);
+            if(!cellIndex.isValid()) {
+              break;
+            }
+            p->setGridValue(val);
+            // Some parametrizations may need some updates before proceeding
+            _forward->valueChanged(p);
+            if(_terminated.value()) {
+              return;
+            }
+            if(_forward->isFussyOk(p)) {
+              //if(nTries>0) {
+              //  App::log(tr("Fussy parameter accepted after %1 tries\n").arg(nTries) );
+              //}
+              break;
+            }
+          }
+          if(nTries==100) {
+            //App::log(tr("Fussy parameter rejected after %1 tries, trying another model.\n").arg(nTries) );
+            return;
+          }
+        } else {
+          int val=pdf.generate(_randomNumbers->ran2(), cellIndex);
+          if(cellIndex.isValid()) {
+            p->setGridValue(val);
+            _forward->valueChanged(p);  // Some parametrizations may need some updates before proceeding
+          }
         }
-        _repository->allModels()->unlock();
+        if(cellIndex.isValid()) {
+          _nav->setValue(p->gridValue());
+          _nav->setCurrentCell(cellIndex);
+          ActiveIndex trueCell;
+          if(!_nav->checkCurrentPoint(parameterSpace)) {
+            pdf.debugPrint(parentIndex->activeModels());
+            ASSERT(false);
+          }
+        } else { // Jumped outside the solution area, but close to border
+                 //    Reset to original starting point
+                 //    Compute misfit and add model
+          _nav->setCurrentPoint(activeIndex);
+          _nav->initDistances(_randomNumbers->uniform(0, ndVar-1));
+          //_nav->initDistances(ndVar-1);
+          break;
+        }
+        ASSERT(parameterSpace.isOkDebug());
+      }
+
+      //printf("Max steps %i\n", iStep);
+      // ModelSet has its own mutex... thread safe.
+      if(misfit(newModel, parameterSpace)) {
+        return;
+      }
+      if(!cellIndex.isValid()) {  // Parameter space cannot be changed before misfit computation
+        _repository->modelToRealSpace(activeIndex, parameterSpace);
+        _forward->valueChanged();
       }
     }
     /*
@@ -399,4 +532,36 @@ namespace DinverCore {
     return;
   }
 
+  /*!
+    If true is returned loop can be stopped: correct misfit found or end of inversion
+  */
+  bool Generator::misfit(NewModel * newModel, RealSpace& parameterSpace)
+  {
+    // Misfit is not directly set in the main repository.
+    // With add, we just test if the model has already been calculated
+    newModel->setIndex(_repository->allModels()->add(parameterSpace));
+    if(newModel->index().isValid()) {
+      bool ok=true;
+      newModel->setMisfit(_forward->misfit(ok));
+      newModel->setValidMisfit(ok);
+      newModel->setValidParent(true);
+      return true;
+    } else {
+      // Generated model already exist
+      // For low dimension parameter spaces (<3), the space can be fully investigated
+      // We put a threshold at 80% coverage to abort the inversion process.
+      double nPossibleModels=parameterSpace.possibleCount();
+      ASSERT(!newModel->isValidMisfit());
+      _repository->allModels()->lock();
+      if(_repository->allModels()->count()>0.8*nPossibleModels) {
+        _repository->allModels()->unlock();
+        _repository->stop(); // force termination of inversion
+        return true;
+      } else {
+        _repository->allModels()->unlock();
+        return false; // Trying another one...
+      }
+    }
+  }
+
 } // namespace DinverCore
