@@ -35,6 +35,8 @@
 
 namespace DinverCore {
 
+  int ModelRepository::seed=0;
+
   /*!
     \class ModelRepository ModelRepository.h
     \brief Brief description of class still missing
@@ -50,7 +52,7 @@ namespace DinverCore {
   {
     TRACE;
 
-    _maximumModelCount=10000;         // Similar to Ns0+Itmax*Ns in old releases
+    _maximumModelCount=10000;         // Similar to Itmax*Ns in old releases
     _maximumBestModelCount=INT_MAX;   // by default no limit
     _maximumQueueLenght=50;           // Similar to ns=50 in old releases
     _maximumSavedMisfit=std::numeric_limits<double>::infinity();     // by default no limit
@@ -58,8 +60,8 @@ namespace DinverCore {
     _significantMisfitReduction=0.01; // Defines what's a significant misfit reduction (for misfit stability assessment)
                                       // Default=1%
     _lastBestMisfit=std::numeric_limits<double>::infinity();         // Misfit increases when many atomic cells are detected
-                                      // Best cell has already been found, it start exploring elsewhere
-    _lastBestIndex=INT_MAX;          // Prevent stopping too early
+                                      // Best cell has already been found, it starts exploring elsewhere
+    _lastBestIndex=INT_MAX;           // Prevent stopping too early
     _giveUp=0.9;
     _walkCount=2;
 
@@ -98,10 +100,10 @@ namespace DinverCore {
   void ModelRepository::setMaximumModelCount(int n)
   {
     TRACE;
-    _maximumModelCount=n;
-    if(_maximumQueueLenght>_maximumModelCount) {
-      App::log(tr("Queue length reduced to %1\n").arg(_maximumModelCount) );
-      _maximumQueueLenght=_maximumModelCount;
+    _maximumModelCount=n+_allModels->count();
+    if(_maximumQueueLenght>n) {
+      App::log(tr("Queue length reduced to %1\n").arg(n) );
+      _maximumQueueLenght=n;
     }
   }
 
@@ -141,6 +143,7 @@ namespace DinverCore {
     _generatorModels=new GeneratorModels(_allModels);
     _generatorModels->addReference();
   }
+
   void ModelRepository::setBestModelCount(int nr)
   {
     TRACE;
@@ -165,6 +168,17 @@ namespace DinverCore {
     }
     return true;
   }
+  /*!
+  */
+  void ModelRepository::closeReport()
+  {
+    TRACE;
+    if(_report) {
+      _report->close();
+      delete _report;
+      _report=nullptr;
+    }
+  }
 
   void ModelRepository::init()
   {
@@ -175,6 +189,9 @@ namespace DinverCore {
   bool ModelRepository::start(int nThreads, Generator::Type type)
   {
     TRACE;
+    // delete existing generators but before wait eventually for their termination.
+    wait();
+
     // Adjust number of threads
     if(nThreads>CoreApplication::instance()->maximumThreadCount()) {
       nThreads=CoreApplication::instance()->maximumThreadCount();
@@ -183,40 +200,24 @@ namespace DinverCore {
       nThreads=_forward->maximumThreadCount();
     }
     if(nThreads<1) nThreads=1;
-    App::log(tr("Starting %1 generator thread(s)\n").arg(nThreads) );
+    App::log(tr("Starting %1 generator thread(s): %2\n").arg(nThreads).arg(Generator::type(type)));
 
-    // Calculate seed model
-    Random randomNumber;
-    if(_generatorModels->activeModels()->count()==0) {
-      if(!_forward->firstModel(randomNumber, _terminated)) {
-        return false;
-      }
-    } else {
-      // Load one of the best models into _forward
-      RealSpace& parameterSpace=_forward->parameterSpace();
-      int ndVar=parameterSpace.variableParameterCount();
-      const int * model=_generatorModels->bestModels()->model(randomNumber.uniform(0, _generatorModels->bestModels()->count()-1));
-      for(int ip=0; ip<ndVar; ip++) {
-        parameterSpace.variableParameter(ip)->setGridValue(model[ip]);
-      }
-      // Some parametrizations may need some updates before proceeding
-      _forward->valueChanged();
-      // Check validity of existing best models
-      if(!parameterSpace.isOkDebug()) {
-        return false;
-      }
+    // Init forward with one valid model
+    Random randomNumber(seed);
+    if(type==Generator::MonteCarlo) {
+      setSeedModel(randomNumber);
     }
 
     // Init all forwards with this first model
-    qDeleteAll(_generators);
-    _generators.clear();
     for(int i=0; i<nThreads; i++) {
       AbstractForward * f=_forward->clone();
       RealSpace& parameterSpace=f->parameterSpace();
       parameterSpace.setVariableParameters();
       parameterSpace.adjustRanges(); // Assumed to be ok, because checked before for the first one
-      // Init first model for f
-      f->copyValues(*_forward);
+      // Init with the first model
+      if(type==Generator::MonteCarlo) {
+        f->copyValues(*_forward);
+      }
       Generator * g=new Generator(this, f, randomNumber.uniform(0, INT_MAX));
       g->setObjectName(QString("Generator_%1").arg(i));
       App::setStream(App::stream(nullptr), g);
@@ -282,18 +283,56 @@ namespace DinverCore {
   {
     static int index=0;
     const ActiveModels * ms=_generatorModels->activeModels();
-    for(int i=0; i<ms->count(); i++) {
+    ActiveIndex end;
+    end.setValue(ms->count());
+    for(ActiveIndex i; i<end; i++) {
       const ActiveModel& m=ms->at(i);
       App::log(1, tr("navigator hit %4 %1 %2 %3 %5\n")
-                       .arg(i)
+                       .arg(i.value())
                        .arg(m.navigatorHitCount())
                        .arg(m.navigatorHitTime())
                        .arg(index)
-                       .arg(m.isQuarantined() ? 1 : 0));
+                       .arg(m.isAtomic() ? 1 : 0));
     }
     index++;
   }
 
+  /*!
+    Monte-Carlo requires a forward object initialized with valid parameter values.
+    If there is no model in the repository, a new one is created from scratch,
+    else a random model is chosen among the best models. This operation is not
+    performed inside the threads because in case of error (inconsistent parameterization)
+    there are to many output messages, difficult to read.
+
+    Neighborhood does not require such initialization because starting models are
+    directly taken from best models.
+  */
+  bool ModelRepository::setSeedModel(Random& randomNumber)
+  {
+    if(_generatorModels->activeModels()->count()==0) {
+      if(!_forward->firstModel(randomNumber, _terminated)) {
+        return false;
+      }
+    } else {
+      // Load one of the best models into _forward
+      RealSpace& parameterSpace=_forward->parameterSpace();
+      int ndVar=parameterSpace.variableParameterCount();
+      BestIndex randomIndex;
+      randomIndex.setValue(randomNumber.uniform(0, _generatorModels->bestModels()->count()-1));
+      const int * model=_generatorModels->bestModels()->model(randomIndex);
+      for(int ip=0; ip<ndVar; ip++) {
+        parameterSpace.variableParameter(ip)->setGridValue(model[ip]);
+      }
+      // Some parametrizations may need some updates before proceeding
+      _forward->valueChanged();
+      // Check validity of existing best models
+      if(!parameterSpace.isOkDebug()) {
+        return false;
+      }
+    }
+    return true;
+  }
+
   /*!
     Called by all Generator before creating one new model.
     Thread-safe.
@@ -302,25 +341,49 @@ namespace DinverCore {
 
     Returns a ParentIndex ready for model generation. \a parameterSpace is
     populated with the values of the selected parent model.
+
+    Returns a nullptr if best models is empty.
   */
-  ParentIndex * ModelRepository::createModel(double randomValue, RealSpace& parameterSpace) const
+  ParentIndex * ModelRepository::createModel(double randomValue) const
   {
     TRACE;
-    int ndVar=parameterSpace.variableParameterCount();
     // Selects randomly one of the best cells
     _generatorModelLock.lockForRead();
     const BestModels * bestModels=_generatorModels->bestModels();
-    int bestModelIndex=qRound((bestModels->count()-1)*randomValue);
-    ParentIndex * parentIndex=new ParentIndex(_generatorModels, bestModels->modelIndex(bestModelIndex));
-    // Copy model to parameter structure to enable condition testing
+    if(bestModels->count()>0) {
+      ParentIndex * parentIndex=new ParentIndex(_generatorModels, bestModels->randomIndex(randomValue));
+      //printf("Best active index %i\n", parentIndex->activeIndex());
+      _generatorModelLock.unlock();
+      return parentIndex;
+    } else {
+      _generatorModelLock.unlock();
+      return nullptr;
+    }
+  }
+
+  /*!
+    Copies model \a index to parameter structure to enable condition testing.
+  */
+  void ModelRepository::modelToRealSpace(const ActiveIndex& index, RealSpace& parameterSpace)
+  {
+    _generatorModelLock.lockForRead();
     _allModels->lock();
-    const int * v=parentIndex->activeModels()->model(parentIndex->activeIndex());
+    const int * v=_generatorModels->activeModels()->model(index);
+    int ndVar=parameterSpace.variableParameterCount();
     for(int ip=0; ip<ndVar; ip++) {
       parameterSpace.variableParameter(ip)->setGridValue(v[ip]);
     }
     _allModels->unlock();
     _generatorModelLock.unlock();
-    return parentIndex;
+  }
+
+  int ModelRepository::serialNumber() const
+  {
+    TRACE;
+    _generatorModelLock.lockForRead();
+    int sn=_generatorModels->serialNumber();
+    _generatorModelLock.unlock();
+    return sn;
   }
 
   /*!
@@ -375,12 +438,12 @@ namespace DinverCore {
       // Update best models if there are less than nr models in best models
       newGenModels->bestModels()->update();
 
-      // Adds new models
+      // Adds new models and set misfits to the main repository (avoid multiple locks in Generator)
       for(QList<NewModel *>::iterator it=modelList.begin(); it!=modelList.end(); ++it) {
         NewModel * m=*it;
-        if(m->isValidParent() && m->isValidMisfit()) {
-          _allModels->setMisfit(m->modelIndex(), 0, m->misfit());
-          newGenModels->add(m->modelIndex());
+        if(m->isValidMisfit()) {
+          _allModels->setMisfit(m->index(), TargetIndex::first, m->misfit());
+          newGenModels->add(m->index(), m->parentVolume());
         }
         delete m;
       }
@@ -389,23 +452,15 @@ namespace DinverCore {
       // Regenerates an up-to-date ScaledModels
       newGenModels->setScaledModels(_forward->parameterSpace());
       int nGenerated=_allModels->count();
+      int nBestModels=newGenModels->bestModels()->count();
       //int scaleTime=_chrono.elapsed();
 
       _allModels->unlock();
 
-      if(nGenerated==0) {
+      if(nGenerated==0 || nBestModels==0) {
         stop();
       } else {
-        double bestMisfit=newGenModels->bestModels()->misfit(0)[0];
-        App::log(tr("[stat] %1 models, %2 models/s, %3 active models, %4 rejected, %5 quarantined, %6 best models, best misfit %7 last best %8 at index %9\n")
-            .arg(nGenerated)
-            .arg(1000.0*modelList.count()/generationTime)
-            .arg(newGenModels->activeModels()->count())
-            .arg(newGenModels->rejectedCount())
-            .arg(newGenModels->quarantinedCount())
-            .arg(newGenModels->bestModels()->count())
-            .arg(bestMisfit).arg(_lastBestMisfit).arg(_lastBestIndex));
-
+        double bestMisfit=newGenModels->bestModels()->misfit(BestIndex::first, TargetIndex::first);
         int effectiveModelCount=nGenerated-newGenModels->rejectedCount();
         if(effectiveModelCount>=_maximumModelCount ||
            newGenModels->bestModels()->count()>=_maximumBestModelCount) {
@@ -415,8 +470,9 @@ namespace DinverCore {
         if(bestMisfit<=_lastBestMisfit) { // If misfit increases because of atomic cells
                                           // we want to keep track of the very best misfit
                                           // and not just the current one
-          if(_lastBestMisfit-bestMisfit>_significantMisfitReduction*_lastBestMisfit) {
-            _lastBestIndex=newGenModels->activeModels()->at(newGenModels->bestModels()->modelIndex(0)).modelIndex();
+          // _lastBestMisfit can be infinite while bestMisift is always finite
+          if(_lastBestMisfit-bestMisfit>_significantMisfitReduction*bestMisfit) {
+            _lastBestIndex=newGenModels->activeModels()->at(newGenModels->bestModels()->activeIndex(BestIndex::first)).index().value();
           }
           _lastBestMisfit=bestMisfit;
         }
@@ -425,7 +481,19 @@ namespace DinverCore {
             stop();
           }
         }
+        App::log(tr("[stat] %1 models, %2 models/s, %3 active models, %4 rejected, "
+                    "%5 atomic, %6 best models, best misfit %7 last best %8 at index %9, "
+                    "best average volume %10\n")
+            .arg(nGenerated)
+            .arg(1000.0*modelList.count()/generationTime)
+            .arg(newGenModels->activeModels()->count())
+            .arg(newGenModels->rejectedCount())
+            .arg(newGenModels->atomicCount())
+            .arg(newGenModels->bestModels()->count())
+            .arg(bestMisfit).arg(_lastBestMisfit).arg(_lastBestIndex)
+            .arg(newGenModels->bestModels()->averageVolume()));
       }
+      //newGenModels->bestModels()->print();
 
       // Switches to new active models
       _generatorModelLock.lockForWrite();
@@ -467,7 +535,8 @@ namespace DinverCore {
   bool ModelRepository::importModels(QString fileName, bool strict)
   {
     TRACE;
-    int i=_allModels->count();
+    SetIndex i;
+    i.setValue(_allModels->count());
 
     if(ReportReader::isReportFormat(fileName)) {
       ReportReader report(fileName);
@@ -486,10 +555,12 @@ namespace DinverCore {
         return false;
       }
     }
-    int n=_allModels->count();
-    for(;i<n;i++) {
-      _generatorModels->add(i);
+    SetIndex end;
+    end.setValue(_allModels->count());
+    for(; i<end; i++) {
+      _generatorModels->add(i, std::numeric_limits<double>::infinity());
     }
+    _generatorModels->setScaledModels(_forward->parameterSpace());
     return true;
   }
 
@@ -551,18 +622,18 @@ namespace DinverCore {
     return _allModels ? _allModels->parameterCount() : 0;
   }
 
-  double ModelRepository::misfit(int modelIndex) const
+  double ModelRepository::misfit(const SetIndex& index) const
   {
     // TODO: multi-dimensional misfit
-    return _allModels ? _allModels->misfit(modelIndex)[0] : 0;
+    return _allModels ? _allModels->misfit(index, TargetIndex::first) : 0.0;
   }
 
-  const int * ModelRepository::model(int modelIndex) const
+  const int * ModelRepository::model(const SetIndex& index) const
   {
-    return _allModels->model(modelIndex);
+    return _allModels->model(index);
   }
 
-  int ModelRepository::bestModelIndex() const
+  SetIndex ModelRepository::bestModelIndex() const
   {
     // TODO: multi-dimensional misfit
     return _allModels->bestModel();
@@ -574,9 +645,9 @@ namespace DinverCore {
     return parameterSpace.variableParameter(paramIndex);
   }
 
-  double ModelRepository::variableParameterValue(int modelIndex, int paramIndex) const
+  double ModelRepository::variableParameterValue(const SetIndex& index, int param) const
   {
-    return variableParameter(paramIndex)->realValue(_allModels->model(modelIndex)[paramIndex]);
+    return variableParameter(param)->realValue(_allModels->model(index)[param]);
   }
 
 } // namespace DinverCore
