@@ -229,8 +229,13 @@ bool ParamLayer::valueParameters(RealSpace& ps, const ParamGroundModel * groundM
   TRACE;
   _topP=ps.addParameter(new GroundParameter(groundModel, this, GroundParameter::ValueTop,
                                             _topMin, _topMax));
-  _topP->setScale(ParameterGrid::Log);
-  _topP->setPrecision(0.01);
+  /*
+  // Comparison of linear versus log scale
+  if(_profile->shortName().startsWith("Vs")) {
+    _topP->setCount(426);
+  } else {
+    _topP->setCount(323);
+  }*/
   if(!_topP->initGrid()) {
     return false;
   }
@@ -281,6 +286,7 @@ bool ParamLayer::valueParameters(RealSpace& ps, const ParamGroundModel * groundM
     case PowerLaw:
       _bottomP=ps.addParameter(new GroundParameter(groundModel,  this, GroundParameter::ValueBottom,
                                                    _bottomMin, _bottomMax));
+      _bottomP->setScale(ParameterGrid::Linear);
       break;
     }
     switch(_shape) {
@@ -310,12 +316,11 @@ bool ParamLayer::depthParameters(bool last, ParamLayer * prev, RealSpace& ps,
   TRACE;
   if(!_ptrLink && !last) {
     _dhP=ps.addParameter(new GroundParameter(groundModel,  this,
-                         _isDepth ? GroundParameter::Depth : GroundParameter::Thickness, _dhMin, _dhMax) );
-    // Currently a debate: log or lin? log has a higher probability to generate shallow models...
-    _dhP->setScale(ParameterGrid::Log);
-    _dhP->setPrecision(0.01);
-    //_dhP->setScale(ParameterGrid::Linear);
-    //_dhP->setPrecision(0.5);
+                         _isDepth ? GroundParameter::Depth : GroundParameter::Thickness, _dhMin, _dhMax));
+    /*
+    // Comparison of linear versus log scale
+    _dhP->setCount(452);
+    */
     if(!_dhP->initGrid()) {
       return false;
     }
@@ -690,7 +695,7 @@ bool ParamLayer::xml_setProperty(XML_SETPROPERTY_ARGS)
   Q_UNUSED(tag)
   Q_UNUSED(attributes)
   Q_UNUSED(context)
-  switch (memberID) {
+  switch(memberID) {
   case 0:
     if(content=="Uniform") setShape(Uniform);
     else if(content=="Linear") setShape(Linear);
