@@ -407,7 +407,13 @@ void TargetList::surfaceMisfit(double& totalMisfit, double& totalWeight,
   if(_ellipticityPeakTarget.selected() && _ellipticityPeakTarget.value().isValid()) {
     // TODO includes misfitType
     // Ellipticity peak misfit is not a L2 norm but just a L1...
-    individualMisfit=_ellipticityFactory->peakMisfit(_ellipticityPeakTarget.value(), *_dispersionFactory, surfModel);
+    individualMisfit=_ellipticityFactory->peakMisfit(_ellipticityPeakTarget.value(),
+                                                     _ellipticityPeakTarget.minimumAmplitude(),
+                                                     *_dispersionFactory, surfModel);
+    if(individualMisfit==std::numeric_limits<double>::infinity()) {
+      ok=false;
+      return;
+    }
     if(individualMisfit<_ellipticityPeakTarget.minimumMisfit()) {
       individualMisfit=_ellipticityPeakTarget.minimumMisfit();
     }
@@ -416,13 +422,7 @@ void TargetList::surfaceMisfit(double& totalMisfit, double& totalWeight,
   }
   if(_ellipticityFactory) { // Switch to angular degree ellipticity even if no misfit on the curve
                             // Required for consistent ploting
-    // Work around for 3.3.6 without modifying EllipticityFactory API
-    // On 3.4 branch, implemented in EllipticityFactory::atanDeg()
-    _ellipticityFactory->signedValues()->atanDeg();
-    if(_ellipticityFactory->absoluteValues()) {
-      *_ellipticityFactory->absoluteValues()=*_ellipticityFactory->signedValues();
-      _ellipticityFactory->absoluteValues()->abs();
-    }
+    _ellipticityFactory->atanDeg();
   }
   if(_ellipticityCurveTarget.selected() && !_ellipticityCurveTarget.curves().isEmpty()) {
     individualMisfit=curveMisfit(_ellipticityCurveTarget.curves(), *_ellipticityFactory, _ellipticityCurveTarget, nDimensions);
@@ -732,7 +732,7 @@ XMLMember TargetList::xml_member(XML_MEMBER_ARGS)
   static const QString attName("type");
   if(tag=="DispersionTarget") {
     return XMLMember(&_dispersionTarget);
-  } else if(tag=="ModalCurveTarget") {
+  } else if(tag==ModalCurveTarget::xmlModalCurveTargetTag) {
     XMLRestoreAttributeIterator it=attributes.find(attName);
     if(it!=attributes.end()) {
       if(it.value()=="ellipticity") {
@@ -748,9 +748,10 @@ XMLMember TargetList::xml_member(XML_MEMBER_ARGS)
       App::log(tr("No type defined for modal curve target\n") );
       return XMLMember(XMLMember::Unknown);
     }
-  } else if(tag=="AutocorrTarget") {
+  } else if(tag==AutocorrTarget::xmlAutocorrTargetTag) {
     return XMLMember(&_autocorrTarget);
-  } else if(tag=="ValueTarget") {
+  } else if(tag==EllipticityPeakTarget::xmlEllipticityPeakTargetTag ||
+            tag=="ValueTarget") {                               // Kept for compatibility
     XMLRestoreAttributeIterator it=attributes.find(attName);
     if(it!=attributes.end()) {
       if(it.value()=="ellipticity peak") {
@@ -763,7 +764,7 @@ XMLMember TargetList::xml_member(XML_MEMBER_ARGS)
       App::log(tr("No type defined for value target\n") );
       return XMLMember(XMLMember::Unknown);
     }
-  } else if(tag=="RefractionTarget") {
+  } else if(tag==RefractionTarget::xmlRefractionTargetTag) {
     XMLRestoreAttributeIterator it=attributes.find(attName);
     if(it!=attributes.end()) {
       if(it.value()=="Vp") {
@@ -778,7 +779,7 @@ XMLMember TargetList::xml_member(XML_MEMBER_ARGS)
       App::log(tr("No type defined for refraction target\n") );
       return XMLMember(XMLMember::Unknown);
     }
-  } else if(tag=="MagnetoTelluricTarget") {
+  } else if(tag==MagnetoTelluricTarget::xmlMagnetoTelluricTargetTag) {
     return XMLMember(&_magnetoTelluricTarget);
   /////////////////////////////////////////////////////////////////////////
   //
@@ -863,7 +864,7 @@ bool TargetList::xml_setProperty(XML_SETPROPERTY_ARGS)
   Q_UNUSED(tag)
   Q_UNUSED(attributes)
   Q_UNUSED(context)
-  switch (memberID) {
+  switch(memberID) {
   case 19: _position.fromString(content.toString()); return true;
   case 0: return true; // kept for compatibility only
   case 1: _dispersionTarget.setMisfitWeight(content.toDouble()); return true; // kept for compatibility only
