@@ -230,6 +230,9 @@ bool SignalFile::load()
   case SignalFileFormat::Fourier:
     ret=loadFourier();
     break;
+  case SignalFileFormat::Efispec3D:
+    ret=loadEfispec3D();
+    break;
   case SignalFileFormat::Tomo:
   case SignalFileFormat::Unknown:
   case SignalFileFormat::Temporary:
@@ -1523,8 +1526,9 @@ bool SignalFile::loadSac(QDataStream::ByteOrder bo)
   QByteArray cmp_str(h.chars.field.KCMPNM, 8);
   cmp_str=cmp_str.trimmed().toUpper();
   Signal::Components cmp=Signal::globalSeismographicNetworkComponent(cmp_str.data());
-  if(cmp==Signal::UndefinedComponent) { // not recognized as a standard name, revert to
-    if(cmp_str.contains("Z"))          // a more basic technique.
+  if(cmp==Signal::UndefinedComponent) { // not recognized as a standard name, revert to more basic technique.
+    APP_LOG(1, tr("SAC component for signal ID %1 '%2'\n").arg(newSignal->id()).arg(cmp_str.data()));
+    if(cmp_str.contains("Z"))
       cmp=Signal::Vertical;
     else if(cmp_str.contains("E"))
       cmp=Signal::East;
@@ -1971,6 +1975,88 @@ bool SignalFile::loadMiniSeed()
   return true;
 }
 
+/*!
+  Load files produced by efispec3D (BRGM, Florent Demartin)
+
+  Files contain blocks of 40 bytes (10 floats: time disp*3 vel*3 acc*3)
+*/
+bool SignalFile::loadEfispec3D()
+{
+  TRACE;
+
+  QFile f(_name);
+  if(!f.open(QIODevice::ReadOnly)) {
+    App::log(tr("Loading Efispec3D values: unable to open file %1\n").arg(name()));
+    return false;
+  }
+  QDataStream s(&f);
+  s.setFloatingPointPrecision(QDataStream::SinglePrecision);
+  // Assuming little endian... according to the simulating architecture,
+  // it can be also big endian. Currently not supported.
+  s.setByteOrder(QDataStream::LittleEndian);
+  float val;
+
+  // Number of samples
+  int nSamples=f.size()/40;
+
+  // Get sampling period from the first and the last blocks
+  // Avoid rounding errors notes when comparing two individual lines
+  s >> val;
+  double t1=Number::toDouble(val);
+  f.skip((nSamples-2)*40);
+  s >> val;
+  double t2=Number::toDouble(val);
+  double samplingPeriod=(t2-t1)/static_cast<double>(nSamples-1);
+
+  // Arbitrary absolute time
+  DateTime startTime(QDate(2000, 1, 1), 0, 0.0);
+
+  // Signal base name from file name (according to efispec3D file structure)
+  // prefix.fsr.XXXXXX.gpl
+  QString name=shortName();
+  QRegularExpression namePattern("^(.+)\\.fsr\\.([0-9]{6})\\.gpl$");
+  QRegularExpressionMatch nameMatch=namePattern.match(name);
+  if(nameMatch.hasMatch()) {
+    name=nameMatch.captured(1)+"_"+nameMatch.captured(2);
+  } else {
+    App::log(tr("Not a standard efispec3D file name\n"));
+    if(name.endsWith(".gpl")) {
+      name.chop(4);
+    }
+  }
+
+  static Signal::AmplitudeUnits units[]={Signal::Displacement,
+                                         Signal::Velocity,
+                                         Signal::Acceleration};
+  for(int unitIndex=0; unitIndex<3; unitIndex++) {
+    for(int compIndex=0; compIndex<3; compIndex++) {
+      int signalIndex=unitIndex*3+compIndex;
+      Signal * newSignal=new Signal(_db);
+      newSignal->setFile(this);
+      newSignal->setNumberInFile(signalIndex);
+      newSignal->setNSamples(nSamples);
+      newSignal->setOffsetInFile(4+signalIndex*4);
+      newSignal->setStartTime(startTime);
+      newSignal->setSamplingPeriod(samplingPeriod);
+      newSignal->setType(Signal::Waveform);
+      newSignal->setAmplitudeUnit(units[unitIndex]);
+      setSignalName(newSignal, name, "", signalIndex, shortName());
+      switch(compIndex%3) {
+      case 1:
+        newSignal->setComponent(Signal::North);
+        break;
+      case 2:
+        newSignal->setComponent(Signal::Vertical);
+        break;
+      default:
+        newSignal->setComponent(Signal::East);
+        break;
+      }
+    }
+  }
+  return true;
+}
+
 void SignalFile::setSignalName(Signal * sig, QString stationName,
                                    QString sufix, int recNum, QString fileName)
 {
@@ -2035,7 +2121,7 @@ bool SignalFile::xml_setProperty(XML_SETPROPERTY_ARGS)
   Q_UNUSED(attributes)
   Q_UNUSED(context)
   bool ok=true;
-  switch (memberID) {
+  switch(memberID) {
   case 0:
     setName(content.toString());
     return true;
@@ -2123,8 +2209,7 @@ bool SignalFile::xml_polish(XML_POLISH_ARGS)
   // In server mode, the wrong file paths, sizes or hashes are not resolved
   // They are checked afterwards.
   QString fileName=name();
-  if(CoreApplication::instance()->hasGui() &&
-     !CoreApplication::instance()->isBatch()) {
+  if(!CoreApplication::instance()->isBatch()) {
     QString log;
     StreamRedirection sr(new StringStream(&log));
     if(!hasValidContent(fileName)) {
