@@ -172,7 +172,7 @@ p, li {white-space: pre-wrap;}
         <source>&lt;html&gt;&lt;head&gt;&lt;meta name=&quot;qrichtext&quot; content=&quot;1&quot; /&gt;&lt;style type=&quot;text/css&quot;&gt;
 p, li {white-space: pre-wrap;}
 &lt;/style&gt;&lt;/head&gt;&lt;body &gt;
-&lt;p&gt;Compute the H/V ratio with an horizontal component having the direction of azimuth (counted clockwize from north).&lt;/p&gt;
+&lt;p&gt;Compute the H/V ratio with an horizontal component having the direction of azimuth (counted clockwise from north).&lt;/p&gt;
 &lt;p &gt;&lt;/p&gt;
 &lt;p&gt;It is useful to quantify the variability of the horizontal energy along the different directions. You might be also interested by the results of the Rotated H/V (another tool provided in this plugin).&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
         <translation type="unfinished"></translation>
