@@ -69,10 +69,7 @@ namespace HVGui {
     connect(winParam, SIGNAL(clearWindows()), this, SLOT(clearWindows()));
     connect(winParam, SIGNAL(loadWindows()), this, SLOT(loadWindows()));
 
-    sampling->setUnit(tr(" Hz"));
-    sampling->setPrecision(2);
-    sampling->setSingleStep(0.25);
-    sampling->setAdmissibleRange(1e-99, std::numeric_limits<double>::infinity());
+    sampling->setFrequency();
 
     WindowFunctionParameters t;
     t.setAlpha(0.1);
@@ -103,10 +100,10 @@ namespace HVGui {
     TRACE;
     AbstractToolWidget::restoreFields();
     if(timeWindowLayer()) {
-      connect(timeWindowLayer(), SIGNAL(addWindows(Signal *, const TimeRange&)),
-              this, SLOT(manualAddWindows(Signal *, const TimeRange&)));
-      connect(timeWindowLayer(), SIGNAL(removeWindows(Signal *, const TimeRange&)),
-              this, SLOT(manualRemoveWindows(Signal *, const TimeRange&)));
+      connect(timeWindowLayer(), SIGNAL(addWindows(const Signal *, const TimeRange&)),
+              this, SLOT(manualAddWindows(const Signal *, const TimeRange&)));
+      connect(timeWindowLayer(), SIGNAL(removeWindows(const Signal *, const TimeRange&)),
+              this, SLOT(manualRemoveWindows(const Signal *, const TimeRange&)));
     }
     if(pickLayer()) {
       connect(pickLayer(), SIGNAL(pickChanged(Signal *, QString)), timeLimits, SLOT(updatePicks()));
@@ -131,7 +128,7 @@ namespace HVGui {
 
   bool AbstractHVWidget::setSubPool(SubSignalPool * subPool)
   {
-    // Connection delayed here because tool is notyet available in base class constructor
+    // Connection delayed here because tool is not yet available in base class constructor
     // Here it must be initialized.
     connect(tool(), SIGNAL(finished()), this, SLOT(finish()));
 
@@ -166,7 +163,7 @@ namespace HVGui {
     _winResults->addActions();
     _winResults->setStations(tool()->stations());
     _winResults->setWindowTitle(subPoolName);
-    connect(_winResults, SIGNAL(categoryChanged()), this, SLOT(updateSummary()));
+    connect(_winResults, SIGNAL(resultsChanged()), this, SLOT(updateSummary()));
     GeopsyGuiEngine::instance()->addSubWindow(this, _winResults)->setUserClosable(false);
 
     if(tool()->stations().count()>1) {
@@ -174,9 +171,9 @@ namespace HVGui {
       if(_summary) {
         _summary->setObjectName("HVGuiSummary");
         _summary->addActions();
-        _summary->setStations(tool()->stations());
         _summary->setWindowTitle(subPoolName);
         GeopsyGuiEngine::instance()->addSubWindow(this, _summary)->setUserClosable(false);
+        updateSummary();
       }
     }
     samplingPeriod();
@@ -205,7 +202,7 @@ namespace HVGui {
     return sp;
   }
 
-  AbstractStation * AbstractHVWidget::belongsTo(Signal * sig)
+  AbstractStation * AbstractHVWidget::belongsTo(const Signal * sig)
   {
     TRACE;
     const QList<AbstractStation *> list=tool()->stations();
@@ -291,7 +288,7 @@ namespace HVGui {
     }
   }
 
-  void AbstractHVWidget::manualAddWindows(Signal * sig, const TimeRange& r)
+  void AbstractHVWidget::manualAddWindows(const Signal * sig, const TimeRange& r)
   {
     TRACE;
     AbstractStation * stat=belongsTo(sig);
@@ -309,7 +306,7 @@ namespace HVGui {
     }
   }
 
-  void AbstractHVWidget::manualRemoveWindows(Signal * sig, const TimeRange& r)
+  void AbstractHVWidget::manualRemoveWindows(const Signal * sig, const TimeRange& r)
   {
     TRACE;
     AbstractStation * stat=belongsTo(sig);
@@ -436,11 +433,17 @@ namespace HVGui {
     if(subPoolLocked()) {
       return;
     }
-    if(!hasWindows()) return;
+    if(!hasWindows()) {
+      return;
+    }
     QList<AbstractStation *> statList=selectedStations();
-    if(statList.isEmpty()) return;
-
-    updateParameters();
+    if(statList.isEmpty()) {
+      return;
+    }
+    if(!updateParameters()) {
+      Message::warning(MSG_ID, tr("Starting"), tr("Error(s) found in parameters, check log."));
+      return;
+    }
     lockSubPool();
     detailedStatus->setLoop(tool()->loop());
     setRunning(true);
@@ -506,12 +509,14 @@ namespace HVGui {
     TRACE;
     HVParameters param;
     getParameters(param);
-    if(!param.makeUpResults().isEmpty()) {
-      _winResults->restoreMakeUp(param.makeUpResults());
-    }
-    _winResults->setLayout(param.plotCountPerLineResults(), param.pageHeightResults());
-    if(_summary && !param.makeUpSummary().isEmpty()) {
-      _summary->sheet()->restoreMakeUp(param.makeUpSummary());
+    if(param.doMakeUp()) {
+      if(!param.makeUpResults().isEmpty()) {
+        _winResults->restoreMakeUp(param.makeUpResults());
+      }
+      _winResults->setLayout(param.plotCountPerLineResults(), param.pageHeightResults());
+      if(_summary && !param.makeUpSummary().isEmpty()) {
+        _summary->sheet()->restoreMakeUp(param.makeUpSummary());
+      }
     }
   }
 
@@ -559,8 +564,9 @@ namespace HVGui {
     }
     LayerLocker ll(timeWindowLayer());
     tool()->loadWindows(fileNames);
-    procParamChanged();
-    updateNofWindows();
+    ll.unlock();
+    windowsChanged();
+    setWinParamChanged(false);
   }
 
   void AbstractHVWidget::setCurrentDisplayStation()
@@ -616,7 +622,7 @@ namespace HVGui {
   {
     TRACE;
     if(_summary) {
-      _summary->setStations(tool()->stations());
+      _summary->setResults(_winResults->results());
     }
   }
 
@@ -716,11 +722,11 @@ namespace HVGui {
     makeUpSummaryBrowse->setEnabled(b);
   }
 
-  void AbstractHVWidget::updateParameters()
+  bool AbstractHVWidget::updateParameters()
   {
     HVParameters param;
     getParameters(param);
-    tool()->setParameters(param);
+    return tool()->setParameters(param);
   }
 
   void AbstractHVWidget::setParameters(const HVParameters& param)
@@ -744,7 +750,7 @@ namespace HVGui {
 
     timeLimits->setParameters(param.timeRange());
     winParam->setParameters(param.windowing());
-    sampling->setParameters(param.sampling());
+    sampling->setParameters(param.frequencySampling());
     smoothing->setParameters(param.smoothing());
     tapering->setParameters(param.tapering());
     if(hComponentsGroup) {
@@ -759,6 +765,9 @@ namespace HVGui {
         case HVParameters::Squared:
           squaredAverage->setChecked(true);
           break;
+        case HVParameters::Geometric:
+          geometricMean->setChecked(true);
+          break;
         }
         azimuthValue->setValue(param.horizontalAzimuth());
       }
@@ -789,7 +798,7 @@ namespace HVGui {
 
     timeLimits->getParameters(param.timeRange());
     winParam->getParameters(param.windowing());
-    sampling->getParameters(param.sampling());
+    sampling->getParameters(param.frequencySampling());
     smoothing->getParameters(param.smoothing());
     tapering->getParameters(param.tapering());
     if(hComponentsGroup) {
@@ -799,6 +808,8 @@ namespace HVGui {
         } else if(azimuth->isChecked()) {
           param.setHorizontal(HVParameters::Azimuth);
           param.setHorizontalAzimuth(azimuthValue->value());
+        } else if(geometricMean->isChecked()) {
+          param.setHorizontal(HVParameters::Geometric);
         } else {
           param.setHorizontal(HVParameters::Squared);
         }
