@@ -112,12 +112,14 @@ void AbstractSummary::addActions()
   menuTools->addAction(a);
 }
 
-void AbstractSummary::setStations(const QList<AbstractStation *>& stations)
+void AbstractSummary::setResults(const QList<AbstractResults *>& results)
 {
   TRACE;
   _results.clear();
-  for(QList<AbstractStation *>::const_iterator it=stations.begin(); it!=stations.end(); ++it) {
-    _results.append(static_cast<CurveResults *>((*it)->results(0)));
+  int n=results.count();
+  for(int i=0; i<n; i++) {
+    CurveResults * r=qobject_cast<CurveResults *>(results.at(i));
+    _results.append(r);
   }
 
   // Adjust the list of categories keeping format for exiting ones
@@ -171,7 +173,7 @@ void AbstractSummary::setAverageValues()
     const CurveResults& res=**it;
     curve=res.average();
     if(!curve.isEmpty()) {
-      curve.yLog10();
+      curve.yLog();
       sum.ySum(curve);
       curve.ySquare();
       sum2.ySum(curve);
@@ -192,8 +194,8 @@ void AbstractSummary::setAverageValues()
       sum2.ySetMinimumValue(0.0);
       sum2.ySqrt();
     }
-    sum.yExp10();
-    sum2.yExp10();
+    sum.yExp();
+    sum2.yExp();
     Curve<Point2D> max=sum;
     max.yMultiply(sum2);
     Curve<Point2D> min=sum;
@@ -275,7 +277,7 @@ void AbstractSummary::setSpectrumGridValues(const QMap<double, CurveResults *> *
 
   // Set the sampling of the grid
   for(int i=0;i < nFreqSamp;i++ ) {
-    grid.setX(i, f[i].x());
+    grid.setX(i, f.x(i));
   }
   // Get the minimum to shift all from 0 to ...
   int i=0;
@@ -293,7 +295,7 @@ void AbstractSummary::setSpectrumGridValues(const QMap<double, CurveResults *> *
       const Curve<Point2D>& c=res->average();
       if(c.count()==nFreqSamp) {
         for(int j=0; j<nFreqSamp; j++) {
-          gval[j]=c[j].y();
+          gval[j]=c.y(j);
         }
       }
     } else {
