@@ -55,11 +55,10 @@ namespace QGpCoreMath {
     void sort();
     void checkSort();
     bool isSorted() const {return _sorted;}
-
     void unique();
 
-    // Vector functions
     bool operator==(const Curve<pointType>& o) const {return QVector<pointType>::operator==(o);}
+    bool operator<(const Curve<pointType>& o) const;
     int count() const {return QVector<pointType>::count();}
     bool isEmpty() const {return QVector<pointType>::isEmpty();}
     void resize(int n) {if(n>count()) {_sorted=false;} QVector<pointType>::resize(n);}
@@ -67,15 +66,25 @@ namespace QGpCoreMath {
     int indexOf(double valX, double relPrec) const;
     int indexOf(const pointType& p) const {return QVector<pointType>::indexOf(p);}
     void indexRange(double xMin, double xMax, int& indexMin, int& indexMax) const;
-    pointType at(double x) const;
+    pointType interpole(double x) const;
     bool isInsideRange(double x) const;
-    const pointType& at(int index) const {return QVector<pointType>::at(index);}
-    pointType& operator[](int index) {_sorted=false; return QVector<pointType>::operator[](index);}
-    const pointType& operator[](int index) const {return QVector<pointType>::at(index);}
-    const pointType& first() const {return QVector<pointType>::first();}
-    const pointType& last() const {return QVector<pointType>::last();}
-    pointType& first() {_sorted=false; return QVector<pointType>::first();}
-    pointType& last() {_sorted=false; return QVector<pointType>::last();}
+
+    const pointType& constAt(int index) const {return QVector<pointType>::at(index);}
+    const pointType& constAt(int index) {return QVector<pointType>::at(index);}
+    pointType& constXAt(int index) {return QVector<pointType>::operator[](index);}
+    pointType& at(int index) {_sorted=false; return QVector<pointType>::operator[](index);}
+
+    double x(int index) const {return constAt(index).x();}
+    double firstX() const {return QVector<pointType>::first().x();}
+    double lastX() const {return QVector<pointType>::last().x();}
+    void setX(int index, double x) {at(index).setX(x);}
+    inline double y(int index, const CurvePointOptions * pointOptions=nullptr) const;
+    double firstY(const CurvePointOptions * pointOptions=nullptr) const {return QVector<pointType>::first().y(pointOptions);}
+    double lastY(const CurvePointOptions * pointOptions=nullptr) const {return QVector<pointType>::last().y(pointOptions);}
+    inline void setY(int index, double y, const CurvePointOptions * pointOptions=nullptr);
+    bool isValid(int index) const {return constAt(index).isValid();}
+    void setValid(int index, bool v) {constXAt(index).setValid(v);}
+
     void remove(int index) {QVector<pointType>::remove(index);}
     void insert(int index, const pointType& p) {_sorted=false; QVector<pointType>::insert(index, p);}
     void insert(const pointType& p);
@@ -92,8 +101,8 @@ namespace QGpCoreMath {
 
     bool sameSampling(const Curve<pointType>& o) const;
 
-    void xLog10();
-    void xExp10();
+    void xLog();
+    void xExp();
     void xMultiply(double factor);
     void xInverse();
 
@@ -110,8 +119,8 @@ namespace QGpCoreMath {
     void ySquare(const CurvePointOptions * pointOptions=nullptr);
     void ySqrt(const CurvePointOptions * pointOptions=nullptr);
     void yInverse(const CurvePointOptions * pointOptions=nullptr);
-    void yLog10(const CurvePointOptions * pointOptions=nullptr);
-    void yExp10(const CurvePointOptions * pointOptions=nullptr);
+    void yLog(const CurvePointOptions * pointOptions=nullptr);
+    void yExp(const CurvePointOptions * pointOptions=nullptr);
     void yAbs(const CurvePointOptions * pointOptions=nullptr);
 
     void line(const pointType& p1, const pointType& p2);
@@ -123,8 +132,10 @@ namespace QGpCoreMath {
     void resample(const QVector<double>& xModel);
     void cut(double min, double max, SamplingOptions options=LinearScale);
     void validate(double min, double max, bool value, SamplingOptions options=LinearScale);
-    void smooth(const SamplingParameters& sampling, const SmoothingParameters& param);
-    pointType smooth(double x, const SmoothingParameters& param);
+    void smooth(const SamplingParameters& sampling, const SmoothingParameters& param,
+                const CurvePointOptions * pointOptions=nullptr);
+    pointType smooth(double x, const SmoothingParameters& param,
+                     const CurvePointOptions * pointOptions=nullptr);
     void average(const Curve<pointType>& o);
     bool extrapolateFromStart(int leastSquareCount, bool positiveSlopeOnly,
                               SamplingOptions options=LinearScale, const CurvePointOptions * pointOptions=nullptr);
@@ -155,6 +166,8 @@ namespace QGpCoreMath {
     int closestMax(int index);
 
     QVector<double> xVector() const;
+    Curve<Point2D> xyCurve() const;
+
     QString toString(int precision=6, char format='g') const;
     bool fromStream(QTextStream& s, QString * comments=nullptr);
 
@@ -166,13 +179,25 @@ namespace QGpCoreMath {
     double sumXY(const CurvePointOptions * pointOptions=nullptr) const;
     double sumX2() const;
     double sumY2(const CurvePointOptions * pointOptions=nullptr) const;
+  protected:
+    bool _sorted;
   private:
     void firstValid(int& startIndex) const;
     inline static bool lessThan(const pointType& p1, const pointType& p2);
-
-    bool _sorted;
   };
 
+  template <class pointType>
+  inline double Curve<pointType>::y(int index, const CurvePointOptions * pointOptions) const
+  {
+    return constAt(index).y(pointOptions);
+  }
+
+  template <class pointType>
+  inline void Curve<pointType>::setY(int index, double y, const CurvePointOptions * pointOptions)
+  {
+    constXAt(index).setY(y, pointOptions);
+  }
+
   template <class pointType>
   void Curve<pointType>::prepend(const pointType& p)
   {
@@ -214,10 +239,10 @@ namespace QGpCoreMath {
       // In case of lot of double entries, this is function was quite long with a remove()
       Curve<pointType> f;
       f.reserve(f.count());
-      f.append(at(0));
+      f.append(constAt(0));
       for(int i=1; i<count(); i++) {
-        if(at(i-1).x()!=at(i).x()) {
-          f.append(at(i));
+        if(x(i-1)!=x(i)) {
+          f.append(constAt(i));
         }
       }
       f.squeeze();
@@ -232,7 +257,7 @@ namespace QGpCoreMath {
   {
     int n=count();
     for(int i=1; i<n; i++) {
-      if(at(i-1).x()>=at(i).x()) {
+      if(x(i-1)>=x(i)) {
         _sorted=false;
         return;
       }
@@ -240,6 +265,23 @@ namespace QGpCoreMath {
     _sorted=true;
   }
 
+  template <class pointType>
+  bool Curve<pointType>::operator<(const Curve<pointType>& o) const
+  {
+    int n=count();
+    if(o.count()<n) {
+      n=o.count();
+    }
+    for(int i=0; i<n; i++) {
+      if(constAt(i)<o.constAt(i)) {
+        return true;
+      } else if(constAt(i)>o.constAt(i)) {
+        return false;
+      }
+    }
+    return count()<o.count();
+  }
+
   template <class pointType>
   void Curve<pointType>::unique()
   {
@@ -257,11 +299,11 @@ namespace QGpCoreMath {
   {
     ASSERT(_sorted);
     ASSERT(!isEmpty());
-    if(valX<=at(0).x()) {
+    if(valX<=firstX()) {
       return 0;
     }
     int lasti=count()-1;
-    if(valX>at(lasti).x()) {
+    if(valX>x(lasti)) {
       return lasti+1;
     }
     int i=1;
@@ -272,8 +314,8 @@ namespace QGpCoreMath {
     while(step2>0) {
       if(i>lasti) {
         i-=step2;
-      } else if(valX <= at(i).x()) {
-        if(valX>at(i-1).x()) break;
+      } else if(valX <= x(i)) {
+        if(valX>x(i-1)) break;
         i-=step2;
       } else {
         i+=step2;
@@ -296,13 +338,13 @@ namespace QGpCoreMath {
       valXHigh=valX*(1.0-relPrec);
     }
     ASSERT(!isEmpty());
-    if(valXHigh<at(0).x()) {
+    if(valXHigh<firstX()) {
       return -1;
-    } else if(valXLow<at(0).x()) {
+    } else if(valXLow<firstX()) {
       return 0;
     }
     int lasti=count()-1;
-    if(valXLow>at(lasti).x()) {
+    if(valXLow>x(lasti)) {
       return -1;
     }
     int i=1;
@@ -313,16 +355,16 @@ namespace QGpCoreMath {
     while(step2>0) {
       if(i>lasti) {
         i-=step2;
-      } else if(valXHigh<at(i).x()) {
+      } else if(valXHigh<x(i)) {
         i-=step2;
-      } else if(valXLow<at(i).x()) {
+      } else if(valXLow<x(i)) {
         return i;
       } else {
         i+=step2;
       }
       step2=step2 >> 1;
     }
-    if(valXLow<at(i).x() && valXHigh>at(i).x()) {
+    if(valXLow<x(i) && valXHigh>x(i)) {
       return i;
     } else {
       return -1;
@@ -356,22 +398,22 @@ namespace QGpCoreMath {
     ASSERT(_sorted);
     if(count()<2) {
       if(count()==1) {
-        return at(0).x()==x;
+        return firstX()==x;
       } else {
         return false;
       }
     } else {
-      return first().x()<=x && x<=last().x();
+      return firstX()<=x && x<=lastX();
     }
   }
 
   template <class pointType>
-  pointType Curve<pointType>::at(double x) const
+  pointType Curve<pointType>::interpole(double x) const
   {
     ASSERT(_sorted);
     if(count()<2) {
       if(count()==1) {
-        return at(0);
+        return constAt(0);
       } else {
         return pointType();
       }
@@ -380,7 +422,7 @@ namespace QGpCoreMath {
       if(i==0) i++;                // Force interpolation if outside range
       else if(i==count()) i--;
       pointType p;
-      p.interpole(x, at(i-1), at(i));
+      p.interpole(x, constAt(i-1), constAt(i));
       return p;
     }
   }
@@ -389,23 +431,23 @@ namespace QGpCoreMath {
   void Curve<pointType>::insert(const pointType& p)
   {
     TRACE;
-    ASSERT(_sorted);
     if(count()<2) {
       if(isEmpty()) {
         QVector<pointType>::append(p);
-      } else if(p.x()<at(0).x()) {
+      } else if(p.x()<firstX()) {
         QVector<pointType>::prepend(p);
-      } else if(p.x()>at(0).x()) {
+      } else if(p.x()>firstX()) {
         QVector<pointType>::append(p);
       } else {
         QVector<pointType>::operator[](0)=p;
       }
+      _sorted=true;
     } else {
       int i=indexAfter(p.x());
-      if(i<count() && at(i).x()==p.x()) {
+      if(i<count() && x(i)==p.x()) {
         (*this)[i]=p;
       } else {
-        if(at(p.x()).y()!=p.y()) {
+        if(interpole(p.x()).y()!=p.y()) {
           insert(i, p);
         }
       }
@@ -423,11 +465,11 @@ namespace QGpCoreMath {
       valX=1.0/valX;
     }
     if(options & LogScale) {
-      xLog10();
-      valX=::log10(valX);
+      xLog();
+      valX=::log(valX);
       if(_sorted) {
-        min=::log10(min);
-        max=::log10(max);
+        min=::log(min);
+        max=::log(max);
       }
     }
     if(_sorted) {
@@ -440,15 +482,15 @@ namespace QGpCoreMath {
 
     // Calculate the total "distance" curve
     Curve<Point2D> distances(currentN);
-    distances[0].setX(at(0).x());
-    distances[0].setY(0.0);
+    distances.setX(0, firstX());
+    distances.setY(0, 0.0);
     for(int i=1; i<currentN; i++) {
-      const pointType& p1=at(i-1);
-      const pointType& p2=at(i);
-      distances[i].setX(p2.x());
+      const pointType& p1=constAt(i-1);
+      const pointType& p2=constAt(i);
+      distances.setX(i, p2.x());
       double deltaX=(p2.x()-p1.x())/valX;
       double deltaY=(p2.y(pointOptions)-p1.y(pointOptions))/valY;
-      distances[i].setY(distances[i-1].y()+::sqrt(deltaX*deltaX+deltaY*deltaY));
+      distances.setY(i, distances.y(i-1)+::sqrt(deltaX*deltaX+deltaY*deltaY));
     }
     distances.sort();
 
@@ -458,14 +500,14 @@ namespace QGpCoreMath {
       Point2D p;
       int i;
       i=distances.indexAfter(min);
-      p.interpole(min, distances.at(i-1), distances.at(i));
+      p.interpole(min, distances.constAt(i-1), distances.constAt(i));
       distMin=p.y();
       i=distances.indexAfter(max);
-      p.interpole(max, distances.at(i-1), distances.at(i));
+      p.interpole(max, distances.constAt(i-1), distances.constAt(i));
       distMax=p.y();
     } else {
       distMin=0;
-      distMax=distances[currentN-1].y();
+      distMax=distances.y(currentN-1);
     }
     double cstStep=(distMax-distMin)/(currentN-1);
 
@@ -478,14 +520,14 @@ namespace QGpCoreMath {
     Point2D p;
     for(int i=0; i<n; i++, dist+=cstStep) {
       int iDist=distances.indexAfter(dist );
-      p.interpole(dist, distances.at(iDist-1), distances.at(iDist));
+      p.interpole(dist, distances.constAt(iDist-1), distances.constAt(iDist));
       x[i]=p.y();
     }
     resample(x);
 
     // Re-Transforms axis according options (log and inv)
     if(options & LogScale) {
-      xExp10();
+      xExp();
     }
     if(options & InversedScale) {
       xInverse();
@@ -500,11 +542,14 @@ namespace QGpCoreMath {
     // Transforms X scale according to options (log and inv)
     if(options & InversedScale) {
       xInverse();
+      double tmp=min;
+      min=1.0/max;
+      max=1.0/tmp;
     }
     if(options & LogScale) {
-      xLog10();
-      min=::log10(min);
-      max=::log10(max);
+      xLog();
+      min=::log(min);
+      max=::log(max);
     }
     // Calculate the constant step
     double cstStep=(max-min)/(n-1);
@@ -516,7 +561,7 @@ namespace QGpCoreMath {
 
     // Re-Transforms axis according options (log and inv)
     if(options & LogScale) {
-      xExp10();
+      xExp();
     }
     if(options & InversedScale) {
       xInverse();
@@ -530,7 +575,7 @@ namespace QGpCoreMath {
     ASSERT(_sorted);
     QVector<double> x(xModel.count());
     for(int i=0; i<xModel.count(); i++) {
-      x[i]=xModel.at(i).x();
+      x[i]=xModel.x(i);
     }
     resample(x);
   }
@@ -549,11 +594,12 @@ namespace QGpCoreMath {
     int lastCurveIndex=nCurve-1;
     for(int i=0; i<nModel; i++) {
       double x=xModel.at(i);
-      while(iCurve<lastCurveIndex && at(iCurve).x()<x) {
+      while(iCurve<lastCurveIndex &&
+            Curve<pointType>::x(iCurve)<x) {
         iCurve++;
       }
-      const pointType& p1=at(iCurve-1);
-      const pointType& p2=at(iCurve);
+      const pointType& p1=constAt(iCurve-1);
+      const pointType& p2=constAt(iCurve);
       interpolated[i].interpole(x, p1, p2);
       if(p1.isValid()) {
         if(p2.isValid()) {
@@ -584,7 +630,7 @@ namespace QGpCoreMath {
       double min=xModel.at(0)*(1+(xModel.at(0)>0 ? -1e-15 : 1e-15));
       double max=xModel.at(nModel-1)*(1+(xModel.at(nModel-1)>0 ? 1e-15 : -1e-15));
       for(int i=0;i<nCurve;i++) {
-        const pointType& p=at(i);
+        const pointType& p=constAt(i);
         if(p.x()<min || max<p.x()) {
           interpolated.append(p);
         }
@@ -612,11 +658,11 @@ namespace QGpCoreMath {
     // Find the first point
     int ix=indexAfter(min);
     if(ix>0) {
-      if(fabs(min-at(ix-1).x()) < 1e-15) {
+      if(fabs(min-x(ix-1)) < 1e-15) {
         ix--;
-      } else if((options & Interpole) && fabs(min - at(ix).x()) > 1e-15) {
+      } else if((options & Interpole) && fabs(min-x(ix))>1e-15) {
         ix--;
-        QVector<pointType>::operator[](ix).interpole(min, at(ix), at(ix+1));
+        QVector<pointType>::operator[](ix).interpole(min, constAt(ix), constAt(ix+1));
       }
     }
     // Remove everything before
@@ -626,10 +672,10 @@ namespace QGpCoreMath {
     // Find the last point
     ix=indexAfter(max);
     if(ix < count()) {
-      if(fabs(max-at(ix).x())<1e-15) {
+      if(fabs(max-x(ix))<1e-15) {
         ix++;
-      } else if((options & Interpole) && fabs(max - at(ix-1).x()) > 1e-15) {
-        QVector<pointType>::operator[](ix).interpole(max, at(ix-1), at(ix));
+      } else if((options & Interpole) && fabs(max-x(ix-1))>1e-15) {
+        QVector<pointType>::operator[](ix).interpole(max, constAt(ix-1), constAt(ix));
         ix++;
       }
     }
@@ -661,12 +707,12 @@ namespace QGpCoreMath {
     int ixs=indexAfter(min);
     int ixe=indexAfter(max);
     if(ixs>0) { // convenient rounding errors
-      if(fabs(min-at(ixs-1).x())<1e-15) {
+      if(fabs(min-x(ixs-1))<1e-15) {
         ixs--;
       }
     }
     if(ixe>0) { // convenient rounding errors
-      if(fabs(min-at(ixe-1).x())<1e-15) {
+      if(fabs(min-x(ixe-1))<1e-15) {
         ixe--;
       }
     }
@@ -683,27 +729,29 @@ namespace QGpCoreMath {
   }
 
   template <class pointType>
-  void Curve<pointType>::smooth(const SamplingParameters& sampling, const SmoothingParameters& param)
+  void Curve<pointType>::smooth(const SamplingParameters& sampling, const SmoothingParameters& param,
+                                const CurvePointOptions * pointOption)
   {
     TRACE;
     ASSERT(_sorted);
     int n=sampling.count();
     Curve<pointType> interpolated(n);
     for(int i=0; i<n; i++) {
-      interpolated[i]=smooth(sampling.value(i), param);
+      interpolated[i]=smooth(sampling.value(i), param, pointOption);
     }
     interpolated.sort();
     *this=interpolated;
   }
 
   template <class pointType>
-  pointType Curve<pointType>::smooth(double x, const SmoothingParameters& param)
+  pointType Curve<pointType>::smooth(double x, const SmoothingParameters& param,
+                                     const CurvePointOptions * pointOption)
   {
     TRACE;
     pointType sum;
     if(count()<2) {
       if(count()==1) {
-        return at(0);
+        return constAt(0);
       } else {
         return sum;
       }
@@ -719,7 +767,7 @@ namespace QGpCoreMath {
       if(i0==0) {
         i0++;
       }
-      sum.interpole(x, at(i0-1), at(i0));
+      sum.interpole(x, constAt(i0-1), constAt(i0));
       return sum;
     }
     switch(param.method()) {
@@ -728,7 +776,7 @@ namespace QGpCoreMath {
         switch(param.scaleType()) {
         case SamplingParameters::Linear:
           for(int i=i0; i<n; i++) {
-            pointType p=at(i);
+            pointType p=constAt(i);
             double weight=WindowFunction::value(p.x(), x, min, max, param.windowFunction());
             sumWeight+=weight;
             p*=weight;
@@ -738,15 +786,27 @@ namespace QGpCoreMath {
           break;
         case SamplingParameters::Log:
           for(int i=i0; i<n; i++) {
-            pointType p=at(i);
-            p.yLog10();
+            pointType p=constAt(i);
+            p.yLog();
             double weight=WindowFunction::value(p.x(), x, min, max, param.windowFunction());
             sumWeight+=weight;
             p*=weight;
             sum+=p;
           }
           sum/=sumWeight;
-          sum.yExp10();
+          sum.yExp();
+          break;
+        case SamplingParameters::Inversed:
+          for(int i=i0; i<n; i++) {
+            pointType p=constAt(i);
+            p.yInverse();
+            double weight=WindowFunction::value(p.x(), x, min, max, param.windowFunction());
+            sumWeight+=weight;
+            p*=weight;
+            sum+=p;
+          }
+          sum/=sumWeight;
+          sum.yInverse();
           break;
         }
       }
@@ -755,11 +815,11 @@ namespace QGpCoreMath {
         LeastSquare s(n-i0, param.order());
         s.setScaleType(param.scaleType());
         for(int i=i0; i<n; i++) {
-          const pointType& p=at(i);
-          s.setData(i-i0, p.x(), p.y());
+          const pointType& p=constAt(i);
+          s.setData(i-i0, p.x(), p.y(pointOption));
         }
         s.invert();
-        sum.setY(s.value(x));
+        sum.setY(s.value(x), pointOption);
       }
       break;
     }
@@ -811,8 +871,8 @@ namespace QGpCoreMath {
   void Curve<pointType>::removeInvalid()
   {
     TRACE;
-    for(int i=count() - 1; i>=0; i-- ) {
-      if(!at(i).isValid()) {
+    for(int i=count()-1; i>=0; i--) {
+      if(!isValid(i)) {
         remove(i);
       }
     }
@@ -840,13 +900,13 @@ namespace QGpCoreMath {
     ASSERT(n>1);
     Curve<pointType> d(n-1);
     for(int i=1; i<n; i++) {
-      double v=(at(i).y(pointOptions)-at(i-1).y(pointOptions))
-                 /(at(i).x()-at(i-1).x());
-      v+=(at(i+1).y(pointOptions)-at(i).y(pointOptions))
-        /(at(i+1).x()-at(i).x());
+      double v=(y(i, pointOptions)-y(i-1, pointOptions))
+                 /(x(i)-x(i-1));
+      v+=(y(i+1, pointOptions)-y(i, pointOptions))
+        /(x(i+1)-x(i));
       v*=0.5;
       pointType& p=d[i-1];
-      p.setX(at(i).x());
+      p.setX(x(i));
       p.setY(v, pointOptions);
     }
     return d;
@@ -861,11 +921,11 @@ namespace QGpCoreMath {
     ASSERT(n>1);
     Curve<pointType> d(n-1);
     for(int i=1; i<n; i++) {
-      Circle c(at(i-1).point2D(pointOptions),
-               at(i).point2D(pointOptions),
-               at(i+1).point2D(pointOptions));
+      Circle c(constAt(i-1).point2D(pointOptions),
+               constAt(i).point2D(pointOptions),
+               constAt(i+1).point2D(pointOptions));
       pointType& p=d[i-1];
-      p.setX(at(i).x());
+      p.setX(x(i));
       p.setY(c.radius(), pointOptions);
     }
     return d;
@@ -880,8 +940,8 @@ namespace QGpCoreMath {
     ASSERT(n>1);
     Curve<pointType> d(n-1);
     for(int i=1; i<n; i++) {
-      const pointType& p1=at(i-1);
-      const pointType& p2=at(i);
+      const pointType& p1=constAt(i-1);
+      const pointType& p2=constAt(i);
       pointType& a=d[i-1];
       a.setX(p1.x());
       a.setY(p1.point2D(pointOptions).azimuthTo(p2.point2D(pointOptions)), pointOptions);
@@ -895,7 +955,7 @@ namespace QGpCoreMath {
     TRACE;
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i); // Not really const but sort afterwards
       p.setX(p.x()*factor);
     }
     if(factor<0.0 && _sorted) {
@@ -909,7 +969,7 @@ namespace QGpCoreMath {
     TRACE;
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i); // Not really const but sort afterwards
       p.setX(1.0/p.x());
     }
     if(_sorted) {
@@ -918,25 +978,25 @@ namespace QGpCoreMath {
   }
 
   template <class pointType>
-  void Curve<pointType>::xExp10()
+  void Curve<pointType>::xExp()
   {
     TRACE;
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
-      p.setX(pow(10, p.x()));
+      pointType& p=constXAt(i); // Not really const but exp does not alter order
+      p.setX(exp(p.x()));
     }
   }
 
   template <class pointType>
-  void Curve<pointType>::xLog10()
+  void Curve<pointType>::xLog()
   {
     TRACE;
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i); // Not really const but log does not alter order
       if(p.x()>0.0) {
-        p.setX(::log10(p.x()));
+        p.setX(::log(p.x()));
       }
     }
   }
@@ -946,8 +1006,7 @@ namespace QGpCoreMath {
   {
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
-      p.setY(v, pointOptions);
+      setY(i, v, pointOptions);
     }
   }
 
@@ -956,7 +1015,7 @@ namespace QGpCoreMath {
   {
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i);
       if(p.y(pointOptions)<v) {
         p.setY(v, pointOptions);
       }
@@ -968,7 +1027,7 @@ namespace QGpCoreMath {
   {
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i);
       if(p.y(pointOptions)>v) {
         p.setY(v, pointOptions);
       }
@@ -981,7 +1040,7 @@ namespace QGpCoreMath {
     TRACE;
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i);
       p.setY(p.y(pointOptions)+value, pointOptions);
     }
   }
@@ -999,8 +1058,8 @@ namespace QGpCoreMath {
       return;
     }
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
-      p.setY(p.y(pointOptions)+o.at(i).y(pointOptions), pointOptions);
+      pointType& p=constXAt(i);
+      p.setY(p.y(pointOptions)+o.y(i, pointOptions), pointOptions);
     }
   }
 
@@ -1010,7 +1069,7 @@ namespace QGpCoreMath {
     TRACE;
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i);
       p.setY(p.y(pointOptions)*factor, pointOptions);
     }
   }
@@ -1028,8 +1087,8 @@ namespace QGpCoreMath {
       return;
     }
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
-      p.setY(p.y(pointOptions)*o.at(i).y(pointOptions), pointOptions);
+      pointType& p=constXAt(i);
+      p.setY(p.y(pointOptions)*o.y(i, pointOptions), pointOptions);
     }
   }
 
@@ -1046,8 +1105,8 @@ namespace QGpCoreMath {
       return;
     }
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
-      p.setY(p.y(pointOptions)/o.at(i).y(pointOptions), pointOptions);
+      pointType& p=constXAt(i);
+      p.setY(p.y(pointOptions)/o.y(i, pointOptions), pointOptions);
     }
   }
 
@@ -1057,7 +1116,7 @@ namespace QGpCoreMath {
     TRACE;
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i);
       p.setY(p.y(pointOptions)*p.x()*factor, pointOptions);
     }
   }
@@ -1068,7 +1127,7 @@ namespace QGpCoreMath {
     TRACE;
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i);
       p.setY(p.y(pointOptions)/(p.x()*factor), pointOptions);
     }
   }
@@ -1078,7 +1137,7 @@ namespace QGpCoreMath {
   {
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i);
       double v=p.y(pointOptions);
       p.setY(v*v, pointOptions);
     }
@@ -1089,7 +1148,7 @@ namespace QGpCoreMath {
   {
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i);
       double v=p.y(pointOptions);
       if(v>=0.0) p.setY(::sqrt(v), pointOptions);
     }
@@ -1101,32 +1160,32 @@ namespace QGpCoreMath {
     TRACE;
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i);
       double v=p.y(pointOptions);
       if(v!=0.0) p.setY(1.0/v, pointOptions);
     }
   }
 
   template <class pointType>
-  void Curve<pointType>::yLog10(const CurvePointOptions * pointOptions)
+  void Curve<pointType>::yLog(const CurvePointOptions * pointOptions)
   {
     TRACE;
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i);
       double v=p.y(pointOptions);
-      if(v>0.0) p.setY(::log10(v), pointOptions);
+      if(v>0.0) p.setY(::log(v), pointOptions);
     }
   }
 
   template <class pointType>
-  void Curve<pointType>::yExp10(const CurvePointOptions * pointOptions)
+  void Curve<pointType>::yExp(const CurvePointOptions * pointOptions)
   {
     TRACE;
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
-      p.setY(pow(10, p.y(pointOptions)), pointOptions);
+      pointType& p=constXAt(i);
+      p.setY(exp(p.y(pointOptions)), pointOptions);
     }
   }
 
@@ -1136,7 +1195,7 @@ namespace QGpCoreMath {
     TRACE;
     int n=count();
     for(int i=0; i<n; i++) {
-      pointType& p=QVector<pointType>::operator[](i); // Avoid touching _sorted flag
+      pointType& p=constXAt(i);
       p.setY(fabs(p.y(pointOptions)), pointOptions);
     }
   }
@@ -1162,11 +1221,22 @@ namespace QGpCoreMath {
     int n=count();
     QVector<double> x(n);
     for(int i=0; i<n; i++) {
-      x[i]=at(i).x();
+      x[i]=Curve<pointType>::x(i);
     }
     return x;
   }
 
+  template <class pointType>
+  Curve<Point2D> Curve<pointType>::xyCurve() const
+  {
+    int n=count();
+    Curve<Point2D> c(n);
+    for(int i=0; i<n; i++) {
+      c.at(i)=Point2D(x(i), y(i));
+    }
+    return c;
+  }
+
   template <class pointType>
   void Curve<pointType>::firstValid(int& startIndex) const
   {
@@ -1176,7 +1246,7 @@ namespace QGpCoreMath {
     } else if(startIndex>=n) {
       startIndex=n-1;
     }
-    while(startIndex<n && !at(startIndex).isValid()) {
+    while(startIndex<n && !isValid(startIndex)) {
       startIndex++;
     }
   }
@@ -1190,10 +1260,10 @@ namespace QGpCoreMath {
     }
     firstValid(startIndex);
     if(startIndex<n) {
-      const pointType * p=&at(startIndex);
+      const pointType * p=&constAt(startIndex);
       int iMin=startIndex;
       for(int i=startIndex+1; i<n; i++) {
-        const pointType& pi=at(i);
+        const pointType& pi=constAt(i);
         if(pi.isValid() && pi.x()<p->x()) {
           p=&pi;
           iMin=i;
@@ -1214,10 +1284,10 @@ namespace QGpCoreMath {
     }
     firstValid(startIndex);
     if(startIndex<n) {
-      const pointType * p=&at(startIndex);
+      const pointType * p=&constAt(startIndex);
       int iMax=startIndex;
       for(int i=startIndex+1; i<n; i++) {
-        const pointType& pi=at(i);
+        const pointType& pi=constAt(i);
         if(pi.isValid() && pi.x()>p->x()) {
           p=&pi;
           iMax=i;
@@ -1238,10 +1308,10 @@ namespace QGpCoreMath {
     }
     firstValid(startIndex);
     if(startIndex<n) {
-      const pointType * p=&at(startIndex);
+      const pointType * p=&constAt(startIndex);
       int iMin=startIndex;
       for(int i=startIndex+1; i<n; i++) {
-        const pointType& pi=at(i);
+        const pointType& pi=constAt(i);
         if(pi.isValid() && pi.y(pointOptions)<p->y(pointOptions)) {
           p=&pi;
           iMin=i;
@@ -1262,10 +1332,10 @@ namespace QGpCoreMath {
     }
     firstValid(startIndex);
     if(startIndex<n) {
-      const pointType * p=&at(startIndex);
+      const pointType * p=&constAt(startIndex);
       int iMax=startIndex;
       for(int i=startIndex+1; i<n; i++) {
-        const pointType& pi=at(i);
+        const pointType& pi=constAt(i);
         if(pi.isValid() && pi.y(pointOptions)>p->y(pointOptions)) {
           p=&pi;
           iMax=i;
@@ -1288,14 +1358,14 @@ namespace QGpCoreMath {
     if(startIndex==n) {
       return -1;
     }
-    const pointType * p0=&at(startIndex);
+    const pointType * p0=&constAt(startIndex);
     firstValid(startIndex);
     if(startIndex==n) {
       return -1;
     }
-    const pointType * p1=&at(startIndex);
+    const pointType * p1=&constAt(startIndex);
     for(int i=startIndex; i<n; i++) {
-      const pointType * p2=&at(i);
+      const pointType * p2=&constAt(i);
       if(p2->isValid()) {
         if(p1->y(pointOptions)>p0->y(pointOptions) &&
            p1->y(pointOptions)>p2->y(pointOptions)) {
@@ -1319,14 +1389,14 @@ namespace QGpCoreMath {
     if(startIndex==n) {
       return -1;
     }
-    const pointType * p0=&at(startIndex);
+    const pointType * p0=&constAt(startIndex);
     firstValid(startIndex);
     if(startIndex==n) {
       return -1;
     }
-    const pointType * p1=&at(startIndex);
+    const pointType * p1=&constAt(startIndex);
     for(int i=startIndex; i<n; i++) {
-      const pointType * p2=&at(i);
+      const pointType * p2=&constAt(i);
       if(p2->isValid()) {
         if(p1->y(pointOptions)<p0->y(pointOptions) &&
            p1->y(pointOptions)<p2->y(pointOptions)) {
@@ -1353,13 +1423,13 @@ namespace QGpCoreMath {
     else if(index>=n) {
       index=n;
       di=-1;
-    } else if(at(index).y()>at(index-1).y()) di=1;
-    else if(at(index).y()<at(index-1).y()) di=-1;
+    } else if(y(index)>y(index-1)) di=1;
+    else if(y(index)<y(index-1)) di=-1;
     else return index;
 
     if(di>0) {
       while(index<n) {
-        if(at(index).y()<at(index-1).y()) {
+        if(y(index)<y(index-1)) {
           index--;
           break;
         }
@@ -1368,7 +1438,7 @@ namespace QGpCoreMath {
       if(index==n) index--;
     } else {
       while(index>0) {
-        if(at(index).y()>at(index-1).y()) break;
+        if(y(index)>y(index-1)) break;
         index+=di;
       }
     }
@@ -1521,7 +1591,7 @@ namespace QGpCoreMath {
       double min=std::numeric_limits<double>::infinity();
       if(leastSquare(a, b, pointOptions)) {
         for(int i=0; i<n; i++) {
-          const pointType& p=at(i);
+          const pointType& p=constAt(i);
           double denom=a*a+1.0;
           double cp=-p.x()-a*p.y(pointOptions);
           double xp=(-cp-a*b)/denom;
@@ -1532,7 +1602,7 @@ namespace QGpCoreMath {
         }
       } else {
         for(int i=0; i<n; i++) {
-          const pointType& p=at(i);
+          const pointType& p=constAt(i);
           projections[i]=p.y(pointOptions);
           if(projections[i]<min) min=projections[i];
         }
@@ -1554,7 +1624,7 @@ namespace QGpCoreMath {
       return false;
     }
     for(int i=0; i<n; i++) {
-      if(at(i).x()!=o.at(i).x()) {
+      if(x(i)!=o.x(i)) {
         return false;
       }
     }
@@ -1570,14 +1640,14 @@ namespace QGpCoreMath {
     double s, min=std::numeric_limits<double>::infinity();
     if(options & LogScale) {
       for(int i=1; i<n; i++) {
-        s=at(i).x()/at(i-1).x();
+        s=x(i)/x(i-1);
         if(s<min) {
           min=s;
         }
       }
     } else {
       for(int i=1; i<n; i++) {
-        s=at(i).x()-at(i-1).x();
+        s=x(i)-x(i-1);
         if(s<min) {
           min=s;
         }
@@ -1602,14 +1672,15 @@ namespace QGpCoreMath {
         qSwap(maxX, invMaxX);
       }
       for(int i=0; i<n; i++) {
-        if(at(i).isValid()) {
-          double y0=at(i).y(pointOptions);
+        if(isValid(i)) {
+          double x0=x(i);
+          double y0=y(i, pointOptions);
           for(int j=0; j<n; j++) {
-            if(i!=j && at(j).isValid()) {
-              double dx=at(j).x()/at(i).x();
+            if(i!=j && isValid(j)) {
+              double dx=x(j)/x(i);
               if((invMaxX<dx && dx<1.0-1e-12) || // Never add a pair at the same x value
                  (dx>1.0+1e-12 && dx<maxX)) {
-                splitter.addPair(i, j, y0, ::log(dx), at(j).y(pointOptions));
+                splitter.addPair(i, j, x0, y0, ::log(dx), y(j, pointOptions));
               }
             }
           }
@@ -1617,13 +1688,14 @@ namespace QGpCoreMath {
       }
     } else {
       for(int i=0; i<n; i++) {
-        if(at(i).isValid()) {
-          double y0=at(i).y(pointOptions);
+        if(isValid(i)) {
+          double x0=x(i);
+          double y0=y(i, pointOptions);
           for(int j=0; j<n; j++) {
-            double dx=at(j).x()-at(i).x();
-            if(i!=j  && at(j).isValid() && fabs(dx)<maxX) {
+            double dx=x(j)-x(i);
+            if(i!=j && isValid(j) && fabs(dx)<maxX) {
               if(fabs(dx)>1e-12) { // Never add a pair at the same x value
-                splitter.addPair(i, j, y0, dx, at(j).y(pointOptions));
+                splitter.addPair(i, j, x0, y0, dx, y(j, pointOptions));
              }
             }
           }
@@ -1631,14 +1703,17 @@ namespace QGpCoreMath {
       }
     }
     splitter.clusterize(maxErr);
-    n=splitter.clusterCount();
-    QList<Curve<pointType> > list;
-    for(int i=0; i<n; i++) {
-      int nc=splitter.valueCount(i);
-      if(nc>=minCount) {
-        Curve<pointType> c;
-        for(int j=0; j<nc; j++) {
-          c.append(at(splitter.index(i, j)));
+    splitter.validSegments(maxErr, xyCurve());
+    splitter.continuousSegments(xVector());
+    int nc=splitter.clusterCount();
+    QList<Curve<pointType>> list;
+    for(int ic=0; ic<nc; ic++) {
+      QVector<int> indexList=splitter.pointIndex(ic);
+      int ni=indexList.count();
+      if(ni>minCount) {
+        Curve<pointType> c(ni);
+        for(int ii=0; ii<ni; ii++) {
+          c.at(ii)=constAt(indexList.at(ii));
         }
         c.sort();
         if(c.count()>minCount) { // If several samples have the same x, sort may reduce the number of samples
@@ -1663,7 +1738,7 @@ namespace QGpCoreMath {
 
     Curve<Point2D> curveBegin;
     for(int i=0; i<n; i++) {
-      const pointType& p=at(i);
+      const pointType& p=constAt(i);
       curveBegin.append(Point2D(p.x(), p.y(pointOptions)));
       if(curveBegin.count()==leastSquareCount) {
         break;
@@ -1675,16 +1750,16 @@ namespace QGpCoreMath {
 
     // first and last for a non-const function mark this curve as not sorted
     bool extrapolated=false;
-    double x=first().x();
+    double x=firstX();
     if(options & LogScale) {
-      curveBegin.xLog10();
+      curveBegin.xLog();
       curveBegin.leastSquare(a, b, pointOptions);
       if(!positiveSlopeOnly || a>0.0) {
         for(int i=leastSquareCount-1; i>=0; i--) {
           pointType p;
           x/=step;
           p.setX(x);
-          p.setY(a*log10(x)+b, pointOptions);
+          p.setY(a*log(x)+b, pointOptions);
           prepend(p);
         }
         extrapolated=true;
@@ -1722,7 +1797,7 @@ namespace QGpCoreMath {
 
     Curve<Point2D> curveEnd;
     for(int i=n-1; i>=0; i--) {
-      const pointType& p=at(i);
+      const pointType& p=constAt(i);
       curveEnd.append(Point2D(p.x(), p.y(pointOptions)));
       if(curveEnd.count()==leastSquareCount) {
         break;
@@ -1734,16 +1809,16 @@ namespace QGpCoreMath {
 
     // first and last for a non-const function mark this curve as not sorted
     bool extrapolated=false;
-    double x=last().x();
+    double x=Curve<pointType>::x(count()-1);
     if(options & LogScale) {
-      curveEnd.xLog10();
+      curveEnd.xLog();
       curveEnd.leastSquare(a, b, pointOptions);
       if(!positiveSlopeOnly || a>0.0) {
         for(int i=leastSquareCount-1; i>=0; i--) {
           pointType p;
           x*=step;
           p.setX(x);
-          p.setY(a*log10(x)+b, pointOptions);
+          p.setY(a*log(x)+b, pointOptions);
           append(p);
         }
         extrapolated=true;
