@@ -61,6 +61,7 @@ IrregularGrid2DData::IrregularGrid2DData(int nx, int ny)
   _nx=nx;
   _ny=ny;
   ASSERT(_nx>0 && _ny>0);
+  ASSERT(_nx<INT_MAX/2 && _ny<INT_MAX/2); // initLookup fails due to overflow in int32
   initLookup();
   _x=new double[nx];
   _y=new double[ny];
@@ -110,7 +111,7 @@ void IrregularGrid2DData::operator=(const IrregularGrid2DData& o)
   memcpy(_values, o._values, sizeof(double)*n);
 }
 
-void IrregularGrid2DData::copy(const GridSearch& o, AxisType normalTo, double at)
+/*void IrregularGrid2DData::copy(const GridSearch& o, AxisType normalTo, double at)
 {
   TRACE;
   clear();
@@ -192,7 +193,7 @@ void IrregularGrid2DData::copy(const GridSearch& o, AxisType normalTo, double at
     }
     break;
   }
-}
+}*/
 
 void IrregularGrid2DData::operator+=(const IrregularGrid2DData& o)
 {
@@ -359,42 +360,42 @@ void IrregularGrid2DData::inverse(AxisType axis)
   _values=newValues;
 }
 
-void IrregularGrid2DData::log10(AxisType axis)
+void IrregularGrid2DData::log(AxisType axis)
 {
   TRACE;
   ASSERT(_nx>0 && _ny>0);
   if(axis==XAxis) {
-    for(int ix=0; ix<_nx; ix++) _x[ix]=::log10(_x[ix]);
+    for(int ix=0; ix<_nx; ix++) _x[ix]=::log(_x[ix]);
   } else {
-    for(int iy=0; iy<_ny; iy++) _y[iy]=::log10(_y[iy]);
+    for(int iy=0; iy<_ny; iy++) _y[iy]=::log(_y[iy]);
   }
 }
 
-void IrregularGrid2DData::log10()
+void IrregularGrid2DData::log()
 {
   TRACE;
   ASSERT(_nx>0 && _ny>0);
   int n=_nx*_ny;
-  for(int i=0; i<n; i++) _values[i]=::log10(_values[i]);
+  for(int i=0; i<n; i++) _values[i]=::log(_values[i]);
 }
 
-void IrregularGrid2DData::pow10(AxisType axis)
+void IrregularGrid2DData::exp(AxisType axis)
 {
   TRACE;
   ASSERT(_nx>0 && _ny>0);
   if(axis==XAxis) {
-    for(int ix=0; ix<_nx; ix++) _x[ix]=pow(10.0, _x[ix]);
+    for(int ix=0; ix<_nx; ix++) _x[ix]=::exp(_x[ix]);
   } else {
-    for(int iy=0; iy<_ny; iy++) _y[iy]=pow(10.0, _y[iy]);
+    for(int iy=0; iy<_ny; iy++) _y[iy]=::exp(_y[iy]);
   }
 }
 
-void IrregularGrid2DData::pow10()
+void IrregularGrid2DData::exp()
 {
   TRACE;
   ASSERT(_nx>0 && _ny>0);
   int n=_nx*_ny;
-  for(int i=0; i<n; i++) _values[i]=pow(10.0, _values[i]);
+  for(int i=0; i<n; i++) _values[i]=::exp(_values[i]);
 }
 
 void IrregularGrid2DData::multiplyCoordinates(AxisType axis, double fac)
@@ -546,7 +547,7 @@ double IrregularGrid2DData::sum(AxisType along, int index) const
 /*!
   Return the maximum of values along axis \a along at \a index counted on the other axis
 */
-double IrregularGrid2DData::maximumValue(AxisType along, int index) const
+double IrregularGrid2DData::maximumValue(AxisType along, int index, int& maxIndex) const
 {
   TRACE;
   // Construct an iterator on grid values
@@ -556,15 +557,21 @@ double IrregularGrid2DData::maximumValue(AxisType along, int index) const
   for(int i=0; i<n; i+=di) {
     if(val[i]>v) {
       v=val[i];
+      maxIndex=i;
     }
   }
+  if(along==XAxis) {
+    maxIndex=maxIndex%_nx;
+  } else {
+    maxIndex=maxIndex/_nx;
+  }
   return v;
 }
 
 /*!
   Return the maximum of values along axis \a along at \a index counted on the other axis
 */
-double IrregularGrid2DData::minimumValue(AxisType along, int index) const
+double IrregularGrid2DData::minimumValue(AxisType along, int index, int& minIndex) const
 {
   TRACE;
   // Construct an iterator on grid values
@@ -574,8 +581,14 @@ double IrregularGrid2DData::minimumValue(AxisType along, int index) const
   for(int i=0; i<n; i+=di) {
     if(val[i]<v) {
       v=val[i];
+      minIndex=i;
     }
   }
+  if(along==XAxis) {
+    minIndex=minIndex%_nx;
+  } else {
+    minIndex=minIndex/_nx;
+  }
   return v;
 }
 
@@ -914,7 +927,7 @@ void IrregularGrid2DData::normalizeAmplitude(AxisType along)
 /*!
   Returns the integral of the grid at index along axis as a curve
   For correctness, normalize() must have been called before
-  */
+*/
 Curve<Point2D> IrregularGrid2DData::integralCrossSection(AxisType along, int index) const
 {
   Curve<Point2D> f;
@@ -925,29 +938,30 @@ Curve<Point2D> IrregularGrid2DData::integralCrossSection(AxisType along, int ind
   int na;
   double * vala=getAxis(along, na);
   ASSERT(na*di==n);
-  double sum=0, dsum;
+  double sum=0.0, dsum;
   bool continuous=true;
   Point2D p;
   for(int ia=0, i=0; ia<na; i+=di, ia++) {
     dsum=val[i]*widthOrHeight(vala, na, ia);
     if(dsum>1e-15) {
       if(!continuous) {
-        p.setX(sum+1e-15);
-        p.setY(rightOrTop(vala, na, ia-1));
+        p.setX(rightOrTop(vala, na, ia-1));
+        p.setY(sum+1e-15);
         f.append(p);
       }
       sum+=dsum;
-      p.setX(sum);
-      p.setY(rightOrTop(vala, na, ia));
+      p.setX(rightOrTop(vala, na, ia));
+      p.setY(sum);
       f.append(p);
       continuous=true;
-    } else continuous=false;
+    } else {
+      continuous=false;
+    }
   }
   f.sort();
   return f;
 }
 
-
 /*!
   Returns a cross section along \a axis at index \a ixy
 */
@@ -955,63 +969,71 @@ Curve<Point2D> IrregularGrid2DData::crossSection(AxisType axis, int ixy) const
 {
   int n;
   double * val=getAxis(axis, n);
-  Curve<Point2D> f (n);
-  for(int i=0; i<n; i++) f[i].setX(val[i]);
-  if(axis ==XAxis) {
-    for(int i=0; i<n; i++) f[i].setY(value(i, ixy));
+  Curve<Point2D> f(n);
+  for(int i=0; i<n; i++) f.setX(i, val[i]);
+  if(axis==XAxis) {
+    for(int i=0; i<n; i++) f.setY(i, value(i, ixy));
   } else {
-    for(int i=0; i<n; i++) f[i].setY(value(ixy, i));
+    for(int i=0; i<n; i++) f.setY(i, value(ixy, i));
   }
-  f.sort();
+  f.sort(); // x and y of a grid are not necessarily sorted
   return f;
 }
 
-void IrregularGrid2DData::smooth(AxisType along)
+/*!
+  Returns a cross section along \a axis at index \a ixy
+*/
+Curve<Point2D> IrregularGrid2DData::stepCrossSection(AxisType axis, int ixy) const
+{
+  int n;
+  double * val=getAxis(axis, n);
+  Curve<Point2D> f(n+1);
+  if(axis==XAxis) {
+    for(int i=0; i<n; i++) {
+      Point2D& p=f.constXAt(i); // Not true but sort afterwards
+      p.setX(left(i));
+      p.setY(value(i, ixy));
+    }
+  } else {
+    for(int i=0; i<n; i++) {
+      Point2D& p=f.constXAt(i); // Not true but sort afterwards
+      p.setX(bottom(i));
+      p.setY(value(ixy, i));
+    }
+  }
+  Point2D& p=f.constXAt(n); // Not true but sort afterwards
+  p.setX(rightOrTop(val, n, n-1));
+  p.setY(0.0);
+  f.sort(); // x and y of a grid are not necessarily sorted
+  return f;
+}
+
+void IrregularGrid2DData::smooth(AxisType along, const SmoothingParameters& param)
 {
   TRACE;
   if(along==XAxis) {
     for(int i=0; i<_ny; i++) {
-      smooth(along, i);
+      smooth(along, i, param);
     }
   } else {
     for(int i=0; i<_nx; i++) {
-      smooth(along, i);
+      smooth(along, i, param);
     }
   }
 }
 
-void IrregularGrid2DData::smooth(AxisType along, int index)
+void IrregularGrid2DData::smooth(AxisType along, int index, const SmoothingParameters& param)
 {
   TRACE;
-  int n;
+  Curve<Point2D> curve=crossSection(along, index);
   if(along==XAxis) {
-     n=_nx-1;
-    double * newValues=new double[n];
-    for(int ix=1; ix<n; ix++) {
-      newValues[ix]=(1.0/3.0)*(value(ix-1, index)+value(ix, index)+value(ix+1, index));
+    for(int i=1; i<_nx; i++) {
+      *valuePointer(i, index)=curve.smooth(_x[i], param).y();
     }
-    for(int ix=1; ix<n; ix++) {
-      *valuePointer(ix, index)=newValues[ix];
-    }
-    if(_nx>1) {
-      *valuePointer(0, index)=0.5*(value(0, index)+value(1, index));
-      *valuePointer(n, index)=0.5*(value(n, index)+value(n-1, index));
-    }
-    delete [] newValues;
   } else {
-    n=_ny-1;
-    double * newValues=new double[n];
-    for(int iy=1; iy<n; iy++) {
-      newValues[iy]=value(index, iy-1)+value(index, iy)+value(index, iy+1);
+    for(int i=1; i<_ny; i++) {
+      *valuePointer(index, i)=curve.smooth(_y[i], param).y();
     }
-    for(int iy=1; iy<n; iy++) {
-      *valuePointer(index, iy)=newValues[iy];
-    }
-    if(_ny>1) {
-      *valuePointer(index, 0)=0.5*(value(index, 0)+value(index, 1));
-      *valuePointer(index, n)=0.5*(value(index, n)+value(index, n-1));
-    }
-    delete [] newValues;
   }
 }
 
@@ -1182,13 +1204,13 @@ void IrregularGrid2DData::resample(AxisType axis, int n, double min, double max,
     for(int iy=0; iy<_ny; iy++) {
       valuesF=crossSection(axis, iy);
       valuesF.resample(n, min, max, options);
-      for(int ix=0; ix<n; ix++) val[ix]=valuesF[ix].y();
+      for(int ix=0; ix<n; ix++) val[ix]=valuesF.y(ix);
       val+=n;
     }
     x=new double[n];
     delete [] _x;
     _x=x;
-    for(int ix=0; ix<n; ix++) x[ix]=valuesF[ix].x();
+    for(int ix=0; ix<n; ix++) x[ix]=valuesF.x(ix);
     _nx=n;
   } else {
     values=new double[_nx*n];
@@ -1197,13 +1219,13 @@ void IrregularGrid2DData::resample(AxisType axis, int n, double min, double max,
       valuesF=crossSection(axis, ix);
       valuesF.resample(n, min, max, options);
       int offY=0;
-      for(int iy=0; iy<n; iy++, offY+=_nx) val[offY]=valuesF[iy].y();
+      for(int iy=0; iy<n; iy++, offY+=_nx) val[offY]=valuesF.y(iy);
       val++;
     }
     y=new double[n];
     delete [] _y;
     _y=y;
-    for(int iy=0; iy<n; iy++) y[iy]=valuesF[iy].x();
+    for(int iy=0; iy<n; iy++) y[iy]=valuesF.x(iy);
     _ny=n;
   }
   delete [] _values;
@@ -1224,13 +1246,13 @@ void IrregularGrid2DData::resample(AxisType axis, const QVector<double>& xModel)
       valuesF=crossSection(axis, iy);
       // This is the only difference with precedding function
       valuesF.resample(xModel);
-      for(int ix=0; ix<n; ix++) val[ix]=valuesF[ix].y();
+      for(int ix=0; ix<n; ix++) val[ix]=valuesF.y(ix);
       val+=n;
     }
     x=new double[n];
     delete [] _x;
     _x=x;
-    for(int ix=0; ix<n; ix++) x[ix]=valuesF[ix].x();
+    for(int ix=0; ix<n; ix++) x[ix]=valuesF.x(ix);
     _nx=n;
   } else {
     values=new double[_nx*n];
@@ -1240,13 +1262,13 @@ void IrregularGrid2DData::resample(AxisType axis, const QVector<double>& xModel)
       // This is the only difference with precedding function
       valuesF.resample(xModel);
       int offY=0;
-      for(int iy=0; iy<n; iy++, offY+=_nx) val[offY]=valuesF[iy].y();
+      for(int iy=0; iy<n; iy++, offY+=_nx) val[offY]=valuesF.y(iy);
       val++;
     }
     y=new double[n];
     delete [] _y;
     _y=y;
-    for(int iy=0; iy<n; iy++) y[iy]=valuesF[iy].x();
+    for(int iy=0; iy<n; iy++) y[iy]=valuesF.x(iy);
     _ny=n;
   }
   delete [] _values;
@@ -1269,13 +1291,13 @@ void IrregularGrid2DData::cut(AxisType axis, double min, double max, SamplingOpt
         values=new double[n*_ny];
         val=values;
       }
-      for(int ix=0; ix<n; ix++) val[ix]=valuesF[ix].y();
+      for(int ix=0; ix<n; ix++) val[ix]=valuesF.y(ix);
       val+=n;
     }
     x=new double[n];
     delete [] _x;
     _x=x;
-    for(int ix=0; ix<n; ix++) x[ix]=valuesF[ix].x();
+    for(int ix=0; ix<n; ix++) x[ix]=valuesF.x(ix);
     _nx=n;
   } else {
     for(int ix=0; ix<_nx; ix++) {
@@ -1287,13 +1309,13 @@ void IrregularGrid2DData::cut(AxisType axis, double min, double max, SamplingOpt
         val=values;
       }
       int offY=0;
-      for(int iy=0; iy<n; iy++, offY+=_nx) val[offY]=valuesF[iy].y();
+      for(int iy=0; iy<n; iy++, offY+=_nx) val[offY]=valuesF.y(iy);
       val++;
     }
     y=new double[n];
     delete [] _y;
     _y=y;
-    for(int iy=0; iy<n; iy++) y[iy]=valuesF[iy].x();
+    for(int iy=0; iy<n; iy++) y[iy]=valuesF.x(iy);
     _ny=n;
   }
   delete [] _values;
@@ -1367,11 +1389,11 @@ Curve<Point2D> IrregularGrid2DData::crossSection(const Curve<Point2D>& path, dou
   int n=path.count();
   Curve<Point2D> curve;
   for(int i=1; i<n; i++) {
-    seg.set(path.at(i-1), path.at(i));
+    seg.set(path.constAt(i-1), path.constAt(i));
     if(curve.count()==0) {
       curve.append(crossSection(seg, deltaX));
     } else {
-      curve.append(crossSection(seg, -curve.last().x()));
+      curve.append(crossSection(seg, -curve.lastX()));
     }
   }
   return curve;
@@ -1479,11 +1501,11 @@ Curve<Point2D> IrregularGrid2DData::crossSection(Segment2D seg, double deltaX) c
   int index;
   //printf("P1 %lg %lf\n",seg.p1().x,seg.p1().y);
   if(iX!=endX)
-    dX=seg.p1().distanceTo(interX.at(iX));
+    dX=seg.p1().distanceTo(interX.constAt(iX));
   else
     dX=std::numeric_limits<double>::infinity();
   if(iY!=endY)
-    dY=seg.p1().distanceTo(interY.at(iY));
+    dY=seg.p1().distanceTo(interY.constAt(iY));
   else
     dY=std::numeric_limits<double>::infinity();
   while(iX!=endX || iY!=endY) {
@@ -1529,7 +1551,7 @@ Curve<Point2D> IrregularGrid2DData::crossSection(Segment2D seg, double deltaX) c
       }
       iX+=incX;
       if(iX!=endX)
-        dX=seg.p1().distanceTo(interX.at(iX));
+        dX=seg.p1().distanceTo(interX.constAt(iX));
       else
         dX=std::numeric_limits<double>::infinity();
     } else {
@@ -1571,7 +1593,7 @@ Curve<Point2D> IrregularGrid2DData::crossSection(Segment2D seg, double deltaX) c
       }
       iY+=incY;
       if(iY!=endY)
-        dY=seg.p1().distanceTo(interY.at(iY));
+        dY=seg.p1().distanceTo(interY.constAt(iY));
       else
         dY=std::numeric_limits<double>::infinity();
     }
