@@ -281,7 +281,7 @@ namespace QGpCoreMath {
   }
 
   /*!
-    Rotates point by \a angle degrees clockwize around Z axis.
+    Rotates point by \a angle degrees clockwise around Z axis.
   */
   void Point2D::rotate(const Angle& a)
   {
@@ -448,7 +448,7 @@ namespace QGpCoreMath {
     Converts point from geographical coordinates (long and lat, WGS84) to rectangular coordinates in meters,
     using reference point \a ref (in geographical coordinates). Longitude is along X axis, latitude is along Y axis.
 
-    \a rotation is an angle to rotate north reference (clockwize, in degrees).
+    \a rotation is an angle to rotate north reference (clockwise, in degrees).
 
     Kindly provided by Matthias Ohrnberger.
   */
@@ -504,7 +504,7 @@ namespace QGpCoreMath {
     Converts point from rectangular coordinates in meters to geographical coordinates (lat and long, WGS84),
     using reference point \a ref (in geographical coordinates). Longitude is along X axis, latitude is along Y axis.
 
-    \a rotation is an angle to rotate north reference (clockwize in degrees).
+    \a rotation is an angle to rotate north reference (clockwise in degrees).
 
     Kindly provided by Matthias Ohrnberger.
   */
