@@ -548,20 +548,38 @@ namespace QGpCoreStat {
       SampleFilter filter;
       if(File::isDataLine(lastLine, _pattern)) {
         if(sample.read(lastLine, _xColumn, _yColumn, _noiseColumn, _powerColumn, _ringColumn, _validColumn)) {
+          if(_yFactor!=1.0) {
+            sample.setY(sample.y()*_yFactor);
+          }
           filter.append(sample);
         } else {
           ok=false;
         }
       }
-      while(!s.atEnd()) {
-        if(File::readLine(s, line, _pattern)) {
-          if(sample.read(line, _xColumn, _yColumn, _noiseColumn, _powerColumn, _ringColumn, _validColumn)) {
-            filter.append(sample);
-          } else {
-            ok=false;
-            break;
+      if(_yFactor==1.0) {
+        while(!s.atEnd()) {
+          if(File::readLine(s, line, _pattern)) {
+            if(sample.read(line, _xColumn, _yColumn, _noiseColumn, _powerColumn, _ringColumn, _validColumn)) {
+              filter.append(sample);
+            } else {
+              ok=false;
+              break;
+            }
+            progress.setValue(filter.count());
+          }
+        }
+      } else {
+        while(!s.atEnd()) {
+          if(File::readLine(s, line, _pattern)) {
+            if(sample.read(line, _xColumn, _yColumn, _noiseColumn, _powerColumn, _ringColumn, _validColumn)) {
+              sample.setY(sample.y()*_yFactor);
+              filter.append(sample);
+            } else {
+              ok=false;
+              break;
+            }
+            progress.setValue(filter.count());
           }
-          progress.setValue(filter.count());
         }
       }
       progress.end(filter.count());
@@ -677,7 +695,9 @@ namespace QGpCoreStat {
       static const QString msg=tr("X axis from %1 to %2 with %3 sample (step=as is)\n");
       App::log(msg.arg(_xValues.first())
                   .arg(_xValues.last())
-                  .arg(_xValues.count()));
+                  .arg(_xValues.count())
+                  .arg("asis")
+                  .arg("X"));
       if(!h->setXSampling(_xValues)) {
         delete h;
         return nullptr;
@@ -772,7 +792,7 @@ namespace QGpCoreStat {
         _validColumn=6;
       }
     } else if(line=="# time frequency polarization slowness azimuth valid") {
-      // Compatibility for results from September and October 2017
+      // Compatibility for results from September to October 2017
       // Option -ell* will ouput junk for these results
       slownessColumn=3;
       azimuthColumn=4;
