@@ -1455,14 +1455,24 @@ void DateTimeData::fromTime_t(time_t t)
   _fractions=0.0;
 }
 
+void DateTimeData::fromTime_t(double t)
+{
+  int tr=qFloor(t);
+  fromTime_t(static_cast<time_t>(tr));
+  // double precision is around 1.11e-16 (wikipedia)
+  // Current date in seconds from 1970 is around 1.5e9 (2020)
+  // Hence the smallest fraction that can represented with this format is 1e-6 s
+  _fractions=round((t-tr)*1e6)*1e-6;
+}
+
 void DateTimeData::fromTime_t(QString t)
 {
   QRegExp dec("[\\.,]");
   int decIndex=t.indexOf(dec, 0);
   if(decIndex==-1) {
-    fromTime_t(t.toInt());
+    fromTime_t(static_cast<time_t>(t.toInt()));
   } else {
-    fromTime_t(t.left(decIndex).toInt());
+    fromTime_t(static_cast<time_t>(t.left(decIndex).toInt()));
     _fractions=t.mid(decIndex).toDouble();
   }
 }
