@@ -29,11 +29,16 @@
 
 namespace QGpCoreTools {
 
-/*!
-  \class IncreaseStorage IncreaseStorage.h
-  \brief Brief description of class still missing
+  /*!
+    \class IncreaseStorage IncreaseStorage.h
+    \brief Brief description of class still missing
 
-  Full description of class still missing
-*/
+    Full description of class still missing
+  */
+
+  /*!
+    \fn IncreaseStorage(size_t defaultCapacity)
+    \a defaultCapacity is the initial capacity and the one set by clear().
+  */
 
 } // namespace QGpCoreTools
