@@ -492,17 +492,20 @@ Curve<Point2D> RefractionDippingModel::ray(const TiltNode& src, const TiltNode&
   int pathIndex=endLayer-1;
   if(pathIndex<0) pathIndex=0;
 
-  for(i=0;i<=endLayer;i++) {
+  for(i=0; i<=endLayer; i++) {
     x=srcPaths[pathIndex].abscissa(i);
-    rayPoints[i].setX(_xL+x*_cosangle[i] );
-    rayPoints[i].setY(_depthL[i]-x*_sinangle[i] );
+    Point2D& p=rayPoints.constXAt(i);
+    p.setX(_xL+x*_cosangle[i]);
+    p.setY(_depthL[i]-x*_sinangle[i]);
   }
 
-  for(i=0;i<=endLayer;i++) {
+  for(i=0; i<=endLayer; i++) {
     x=recPaths[pathIndex].abscissa(i);
-    rayPoints[lastpray-i].setX(_xL+x*_cosangle[i] );
-    rayPoints[lastpray-i].setY(_depthL[i]-x*_sinangle[i] );
+    Point2D& p=rayPoints.constXAt(lastpray-i);
+    p.setX(_xL+x*_cosangle[i]);
+    p.setY(_depthL[i]-x*_sinangle[i]);
   }
+  rayPoints.sort();
 
   return rayPoints;
 }
