@@ -142,12 +142,12 @@ QVariant ColumnTextItem::data(const QModelIndex &index, int role) const
       int row=index.row()-4;
       int sec=_parser->sectionIndexOf(row);
       if(_parser->sectionBeginRow(sec)==row) {
-        return QApplication::palette().mid().color();
+        return QGuiApplication::palette().mid().color();
       } else {
         return QVariant();
       }
     } else {
-      return QApplication::palette().button().color();
+      return QGuiApplication::palette().button().color();
     }
   case Qt::TextAlignmentRole:
     if(index.row()>=2) {
