@@ -1804,9 +1804,6 @@ namespace SciFigs {
     setTitle(tr("Frequency (Hz)"));
     setTitleInversedScale(tr("Period (s)"));
     setScaleType(Scale::Log);
-    setAutoTicks(false);
-    setMajorTicks(5);
-    setMinorTicks(1);
   }
 
 } // namespace SciFigs
