@@ -95,7 +95,7 @@ namespace SciFigs {
     // Init the color map
     ColorMap pal;
     pal.generateColorScale(20);
-    pal.setVLinear(0,1);
+    pal.setValues(0, 1, SamplingParameters::Linear);
     plot->setColorMap(pal);
 
     // Change properties of axis
