@@ -4,7 +4,7 @@
 // All headers of TapePositioningSystem library
 
 #include "Anchorage.h"
-#include "Cluster.h"
+#include "ClusterQuads.h"
 #include "Node.h"
 #include "TapePositioningSystemDLLExport.h"
 #include "Triangulator.h"
