@@ -63,7 +63,7 @@ static MainWindow * mainWindow;
 void openPageFiles(int argc, char ** argv)
 {
   double x=dx0, y=dy0, xPage=maxXPage, yPage=maxYPage;
-  QStringList fileList=CoreApplication::instance()->getFileList(argc, argv);
+  QStringList fileList=CoreApplication::getFileList(argc, argv);
   for(QStringList::iterator it=fileList.begin();it!=fileList.end();it++) {
     App::log(tr("Opening file %1\n").arg(*it) );
     mainWindow->sheet()->fileOpen( *it, x, y);
@@ -136,7 +136,7 @@ int main(int argc, char ** argv)
         mode=Text;
       } else if(arg=="-x-column") {
         Application::checkOptionArg(i, argc, argv);
-        int xci=atoi(argv[i]);
+        int xci=CoreApplication::toInt(i, i-1, argv);
         if(xci>=0) {
           xColumn=xci;
         }
@@ -169,7 +169,7 @@ int main(int argc, char ** argv)
           if(yci>=0) {
             yerrColumns.append(yci);
           } else {
-            App::log(tr("figue: bad column index '%1' for option '-yerr-columns'\n").arg(yc) );
+            App::log(tr("figue: bad column index '%1' for option '-yerr-columns'\n").arg(yc));
             return 2;
           }
         }
@@ -178,6 +178,16 @@ int main(int argc, char ** argv)
                       "Make sure that '-y-columns' appears before '-yerr-columns'."));
           return 2;
         }
+      } else if(arg=="-value-column") {
+        Application::checkOptionArg(i, argc, argv);
+        int yci=CoreApplication::toInt(i, i-1, argv);
+        if(yci>=0) {
+          yerrColumns.clear();
+          yerrColumns << yci;
+        } else {
+          App::log(tr("figue: bad column index '%1' for option '-yerr-columns'\n").arg(yci));
+          return 2;
+        }
       } else if(arg=="-name-columns") {
         Application::checkOptionArg(i, argc, argv);
         QString ycs=argv[i];
@@ -189,7 +199,7 @@ int main(int argc, char ** argv)
           if(yci>=0) {
             nameColumns.append(yci);
           } else {
-            App::log(tr("figue: bad column index '%1' for option '-name-columns'\n").arg(yc) );
+            App::log(tr("figue: bad column index '%1' for option '-name-columns'\n").arg(yc));
             return 2;
           }
         }
@@ -203,11 +213,11 @@ int main(int argc, char ** argv)
         makeupSheetFile=argv[i];
       } else if(arg=="-x") {
         Application::checkOptionArg(i, argc, argv);
-        xPlot=atof(argv[i]);
+        xPlot=CoreApplication::toDouble(i, i-1, argv);
         setXPlot=true;
       } else if(arg=="-y") {
         Application::checkOptionArg(i, argc, argv);
-        yPlot=atof(argv[i]);
+        yPlot=CoreApplication::toDouble(i, i-1, argv);
         setYPlot=true;
       } else if(arg=="-cpp") {
         Application::checkOptionArg(i, argc, argv);
@@ -333,6 +343,9 @@ int main(int argc, char ** argv)
     createCurveName(mainWindow->sheet());
     break;
   case Dots:
+    if(yerrColumns.isEmpty()) {
+      yerrColumns << yColumns.last()+1;
+    }
     createDots(mainWindow->sheet());
     break;
   case Circles:
@@ -445,6 +458,7 @@ ApplicationHelp * help()
                                      "Several values separated by comas to plot several curves with the same X values");
   h->addOption("-yerr-columns <INDEX>", "For options '-c', '-mc', and '-cn', define the YErr column index (default=none). "
                                         "The number of indexes provided must be the same as in '-y-columns'.");
+  h->addOption("-value-column <INDEX>", "For options 'd', define the value column index (default=yColumn+1).");
   h->addOption("-name-columns <INDEX>", "For options '-c', '-mc', and '-cn', define the name column index (default=none). "
                                         "The number of indexes provided must be the same as in '-y-columns'.");
   h->addGroup("Output","output");
@@ -947,27 +961,19 @@ void createDots(GraphicSheet * sheet)
       }
     } else {
       QString str(buf);
-      StringSection content(str);
-      dots.append(ValueDot());
-      ValueDot& d=dots.last();
-      const QChar * ptr=nullptr;
-      StringSection f;
-      f=content.nextField(ptr);
-      if(f.isValid()) {
-        d.setX(f.toDouble());
-      } else {
-        dots.erase(dots.end()-1);
-        break;
+      if(!str.isEmpty()) {
+        bool ok=true;
+        LineParser p(str);
+        ValueDot cp;
+        cp.setX(p.toDouble(xColumn, ok));
+        cp.setY(p.toDouble(yColumns.at(0), ok));
+        if(!yerrColumns.isEmpty()) {
+          cp.setValue(p.toDouble(yerrColumns.at(0), ok));
+        }
+        if(ok) {
+          dots.append(cp);
+        }
       }
-      f=content.nextField(ptr);
-      if(f.isValid()) {
-        d.setY(f.toDouble());
-      } else {
-        dots.erase(dots.end()-1);
-        break;
-      }
-      f=content.nextField(ptr);
-      if(f.isValid()) d.setValue(f.toDouble());
     }
   }
   CoreApplication::instance()->debugUserInterrupts(true);
@@ -1127,33 +1133,36 @@ void createManyCurves(GraphicSheet * sheet)
   CoreApplication::instance()->debugUserInterrupts(false);
   while(!sIn.atEnd()) {
     QString str=sIn.readLine();
-    if(str[0]=='\n' || str[0]=='#') {
-      if(curves.last().count()>0) {
-        if(curves.count()==curvesPerPlot) {
-          addManyCurvePlot(curves, plot, iPlot, sheet, &progress);
-          iPlot++;
-          curves.clear();
-          plot=createManyCurvesPlot(pal);
+    if(!str.isEmpty()) {   // Does nothing for an empty line
+      if(str[0]=='\n' || str[0]=='#') {
+        if(curves.last().count()>0) {
+          if(curves.count()==curvesPerPlot) {
+            qDebug() << "curvesPerPlot";
+            addManyCurvePlot(curves, plot, iPlot, sheet, &progress);
+            iPlot++;
+            curves.clear();
+            plot=createManyCurvesPlot(pal);
+          }
+          curves.append(ValueCurve());
+          progress.increaseValue(1);
         }
-        curves.append(ValueCurve());
-        progress.increaseValue(1);
+        int index=str.indexOf("value=");
+        if(index>=0) {
+          bool ok;
+          double v=str.mid(index+6).toDouble(&ok);
+          if(ok) curves.last().setValue(v);
+        }
+      } else {
+        StringSection content(str);
+        const QChar * ptr=nullptr;
+        StringSection f;
+        f=content.nextField(ptr);
+        double x, y;
+        if(f.isValid()) x=f.toDouble(); else break;
+        f=content.nextField(ptr);
+        if(f.isValid()) y=f.toDouble(); else break;
+        curves.last().append(Point2D(x,y));
       }
-      int index=str.indexOf("value=");
-      if(index>=0) {
-        bool ok;
-        double v=str.mid(index+6).toDouble(&ok);
-        if(ok) curves.last().setValue(v);
-      }
-    } else {
-      StringSection content(str);
-      const QChar * ptr=nullptr;
-      StringSection f;
-      f=content.nextField(ptr);
-      double x, y;
-      if(f.isValid()) x=f.toDouble(); else break;
-      f=content.nextField(ptr);
-      if(f.isValid()) y=f.toDouble(); else break;
-      curves.last().append(Point2D(x,y));
     }
   }
   CoreApplication::instance()->debugUserInterrupts(true);
