@@ -135,8 +135,6 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags f)
   }
 
   _progressBar=new ProgressBar(statusBar());
-  _progressBar->setMinimum(0);
-  _progressBar->setMaximum(100);
   _progressBar->setMaximumWidth(100);
   _progressBar->setMaximumHeight(20);
   _progressBar->setVisible(false);
@@ -210,7 +208,6 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags f)
   setAcceptDrops(true);
 
   _toolFactory=new ToolFactory(this);
-  _toolFactory->createToolActions();
   _toolFactory->addActions(_toolsMenu);
   // Get list of tool buttons before actions of tools
   QList<QToolButton *> toolButListBefore=findChildren<QToolButton *>();
@@ -621,12 +618,8 @@ void MainWindow::setProgressValue(int value)
 {
   TRACE;
   if(QThread::currentThread()==Application::instance()->mainThread()) {
-    if(value==_progressBar->maximum()) {
-      _progressHideTimer.start();
-    } else {
-      _progressHideTimer.stop();
-    }
     _progressBar->setValue(value);
+    delayedHideProgressBar();
   } else {
     emit asyncSetProgressValue(value);
   }
@@ -643,11 +636,22 @@ void MainWindow::increaseProgressValue(int value)
   TRACE;
   if(QThread::currentThread()==Application::instance()->mainThread()) {
     _progressBar->increaseValue(value);
+    delayedHideProgressBar();
   } else {
     emit asyncIncreaseProgressValue(value);
   }
 }
 
+void MainWindow::delayedHideProgressBar()
+{
+  TRACE;
+  if(_progressBar->atEnd()) {
+    _progressHideTimer.start();
+  } else {
+    _progressHideTimer.stop();
+  }
+}
+
 void MainWindow::hideProgressBar()
 {
   TRACE;
