@@ -397,7 +397,7 @@ FKParameters::ProcessType ActiveFKToolWidget::processType(int t) const
   TRACE;
   switch(t) {
   case 1:
-    return FKParameters::ActiveRTBF;
+    return FKParameters::ActiveCapon;
   default:
     break;
   }
