@@ -46,10 +46,7 @@ FKToolWidget::FKToolWidget(QWidget * parent) :
   connect(tool(), SIGNAL(waveNumberGridReady()), this, SLOT(setWaveNumberGrid()));
   connect(tool(), SIGNAL(finished()), this, SLOT(finish()));
 
-  freqSamp->setUnit(tr(" Hz"));
-  freqSamp->setPrecision(2);
-  freqSamp->setSingleStep(0.25);
-  freqSamp->setAdmissibleRange(1e-99, std::numeric_limits<double>::infinity());
+  freqSamp->setFrequency();
   winParam->removeFilterOption();
   winParam->setLength(WindowingParameters::FrequencyDependent, 50.0);
   maxPeakCount->setMaximum(INT_MAX);
@@ -135,7 +132,7 @@ void FKToolWidget::setWaveNumberGrid()
     gridStepEdit->blockSignals(false);
   }
   if(!_gridSizeTouched) {
-    k=FKParameters::gridSize(param->kmax());
+    k=param->gridSize();
     gridSizeEdit->blockSignals(true);
     if(k>0.0 && k<pow(10.0, -gridSizeEdit->decimals())) {
       gridSizeEdit->setDecimals(3-qFloor(log10(k)));
@@ -283,6 +280,7 @@ void FKToolWidget::start()
   // Get parameters from dialog box
   FKParameters * param=static_cast<FKParameters *>(parameters());
   if(!param) {
+    Message::warning(MSG_ID, tr("Starting"), tr("Error(s) found in parameters, check log."));
     return;
   }
   param->setOutputBaseName(outputFileName());
@@ -303,7 +301,7 @@ void FKToolWidget::finish()
 {
   unlockSubPool();
   FKParameters * param=static_cast<FKParameters *>(parameters());
-  tool()->results()->save(param->outputBaseName());
+  tool()->results()->save(param->outputBaseName(), tool()->log());
   delete param;
   setRunning(false);
 }
@@ -335,7 +333,7 @@ void FKToolWidget::getParameters(FKParameters& param) const
   param.setProcessType(processType(processTypeEdit->currentIndex()));
   param.setInversionMethod(inversionMethod(inversionMethodEdit->currentIndex()));
   // Store default parameters not accessible through GUI
-  param.setDefaultValues(param.processType(), param.inversionMethod(), tool()->array()->mode());
+  param.setDefaultValues(param.processType(), param.inversionMethod());
 
   timeLimits->getParameters(param.timeLimits());
   param.setSelectDurationFactor(0.01*selectMinimumDurationEdit->value());
@@ -422,7 +420,7 @@ FKParameters::ProcessType FKToolWidget::processType(int t) const
     case 2:
       return FKParameters::Conventional;
     //case 5:
-    //  return FKParameters::ActiveDirectSteering;
+    //  return FKParameters::ActiveDS;
     case 3:
       return FKParameters::ActiveRTBF;
     case 4:
@@ -445,7 +443,7 @@ FKParameters::ProcessType FKToolWidget::processType(int t) const
     break;
   }
   return FKParameters::RTBF;
-  //return FKParameters::DirectSteering;
+  //return FKParameters::RDS;
 }
 
 int FKToolWidget::processType(FKParameters::ProcessType t) const
@@ -454,15 +452,15 @@ int FKToolWidget::processType(FKParameters::ProcessType t) const
   switch (tool()->array()->components()) {
   case StationSignals::AllComponent:
     switch(t) {
-    case FKParameters::DirectSteering:
-    case FKParameters::DirectSteeringRefined:
+    case FKParameters::RDS:
+    case FKParameters::RDSRefined:
     case FKParameters::Omni:
     case FKParameters::PoggiRadial:
     case FKParameters::PoggiVertical:
       break;
-    case FKParameters::DirectSteeringRadial:
+    case FKParameters::RDSRadial:
       return 0;
-    case FKParameters::DirectSteeringVertical:
+    case FKParameters::RDSVertical:
       return 0;
     case FKParameters::RTBF:
       return 0;
@@ -470,7 +468,7 @@ int FKToolWidget::processType(FKParameters::ProcessType t) const
       return 1;
     case FKParameters::Conventional:
       return 2;
-    case FKParameters::ActiveDirectSteering:
+    case FKParameters::ActiveDS:
       return 3;
     case FKParameters::ActiveRTBF:
       return 3;
@@ -480,19 +478,19 @@ int FKToolWidget::processType(FKParameters::ProcessType t) const
     break;
   case StationSignals::VerticalComponent:
     switch(t) {
-    case FKParameters::DirectSteering:
-    case FKParameters::DirectSteeringRefined:
+    case FKParameters::RDS:
+    case FKParameters::RDSRefined:
     case FKParameters::Omni:
     case FKParameters::PoggiRadial:
     case FKParameters::PoggiVertical:
-    case FKParameters::DirectSteeringRadial:
-    case FKParameters::DirectSteeringVertical:
+    case FKParameters::RDSRadial:
+    case FKParameters::RDSVertical:
     case FKParameters::RTBF:
     case FKParameters::RTBFRadial:
       break;
     case FKParameters::Conventional:
       return 1;
-    case FKParameters::ActiveDirectSteering:
+    case FKParameters::ActiveDS:
       return 2;
     case FKParameters::ActiveRTBF:
       return 2;
@@ -616,11 +614,11 @@ void FKToolWidget::on_processTypeEdit_currentIndexChanged(int index)
   bool threeComp=tool()->array()->components()==StationSignals::AllComponent;
   FKParameters::ProcessType t=processType(index);
   switch(t) {
-  case FKParameters::DirectSteering:
-  case FKParameters::DirectSteeringRefined:
+  case FKParameters::RDS:
+  case FKParameters::RDSRefined:
   case FKParameters::Omni:
-  case FKParameters::DirectSteeringRadial:
-  case FKParameters::DirectSteeringVertical:
+  case FKParameters::RDSRadial:
+  case FKParameters::RDSVertical:
     dampingFactorEdit->setEnabled(true);
     srcRecDistance->setEnabled(false);
     rotateStepCountEdit->setEnabled(false);
@@ -638,7 +636,7 @@ void FKToolWidget::on_processTypeEdit_currentIndexChanged(int index)
     rotateStepCountEdit->setEnabled(threeComp);
     srcRecDistance->setEnabled(false);
     break;
-  case FKParameters::ActiveDirectSteering:
+  case FKParameters::ActiveDS:
   case FKParameters::ActiveRTBF:
     dampingFactorEdit->setEnabled(true);
     rotateStepCountEdit->setEnabled(false);
@@ -662,7 +660,7 @@ void FKToolWidget::on_processTypeEdit_currentIndexChanged(int index)
     if(!_gridSizeTouched) {
       param.setGridSize(0.0);
     }
-    param.setDefaultValues(param.processType(), param.inversionMethod(), tool()->array()->mode());
+    param.setDefaultValues(param.processType(), param.inversionMethod());
     processTypeEdit->blockSignals(true);
     setParameters(&param);
     processTypeEdit->blockSignals(false);
