@@ -28,6 +28,8 @@
 
 #include "GeopsyFigs.h"
 #include "geopsyfigsVersion.h"
+#include "geopsyfigsInstallPath.h"
+#include "SciFigsVersion.h"
 
 // Force DLL dependency (required under Windows)
 #ifdef Q_OS_WIN
@@ -35,6 +37,11 @@
 TimeWindow fake(DateTime::minimumTime, DateTime::maximumTime);
 #endif
 
+const char * GeopsyFigs::interfaceVersion() const
+{
+  return SCIFIGS_VERSION;
+}
+
 PACKAGE_INFO("geopsyfigs", GEOPSYFIGS)
 
 #if(QT_VERSION < QT_VERSION_CHECK(5, 0, 0))
