@@ -341,28 +341,28 @@ bool CoordReader::setOptions(int& argc, char ** argv)
         _mode=FromKML;
       } else if(arg=="-reference") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _geoReference.setX(atof(argv[i]));
+        _geoReference.setX(CoreApplication::toDouble(i, i-1, argv));
         CoreApplication::checkOptionArg(i, argc, argv);
-        _geoReference.setY(atof(argv[i]));
+        _geoReference.setY(CoreApplication::toDouble(i, i-2, argv));
       } else if(arg=="-translate") {
         _mode=Translate;
         CoreApplication::checkOptionArg(i, argc, argv);
-        _translateVector.setX(atof(argv[i]));
+        _translateVector.setX(CoreApplication::toDouble(i, i-1, argv));
         CoreApplication::checkOptionArg(i, argc, argv);
-        _translateVector.setY(atof(argv[i]));
+        _translateVector.setY(CoreApplication::toDouble(i, i-2, argv));
       } else if(arg=="-rotate") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _mode=Rotate;
-        // Input is counted clockwize while Angle uses math angles
-        _angle.setDegrees(-atof(argv[i]));
+        // Input is counted clockwise while Angle uses math angles
+        _angle.setDegrees(-CoreApplication::toDouble(i, i-1, argv));
       } else if(arg=="-average") {
         _mode=Average;
       } else if(arg=="-localize") {
         _mode=Localize;
         CoreApplication::checkOptionArg(i, argc, argv);
-        _cartReference.setX(atof(argv[i]));
+        _cartReference.setX(CoreApplication::toDouble(i, i-1, argv));
         CoreApplication::checkOptionArg(i, argc, argv);
-        _cartReference.setY(atof(argv[i]));
+        _cartReference.setY(CoreApplication::toDouble(i, i-2, argv));
       } else if(arg=="-lookup") {
         _mode=Lookup;
         CoreApplication::checkOptionArg(i, argc, argv);
@@ -378,12 +378,12 @@ bool CoordReader::setOptions(int& argc, char ** argv)
       } else if(arg=="-circle") {
         _mode=GenerateCircle;
         CoreApplication::checkOptionArg(i, argc, argv);
-        double radius=atof(argv[i]);
+        double radius=CoreApplication::toDouble(i, i-1, argv);
         CoreApplication::checkOptionArg(i, argc, argv);
-        int n=atoi(argv[i]);
+        int n=CoreApplication::toInt(i, i-2, argv);
         int i0=0;
         if(CoreApplication::checkOptionArg(i, argc, argv, false)) {
-          i0=atoi(argv[i]);
+          i0=CoreApplication::toInt(i, i-3, argv);
         }
         QTextStream sOut(stdout);
         Angle a;
@@ -403,7 +403,7 @@ bool CoordReader::setOptions(int& argc, char ** argv)
         _mode=ArrayLimits;
       } else if(arg=="-stddev") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _distanceStddev=atof(argv[i]);
+        _distanceStddev=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-origin") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _origin=argv[i];
@@ -418,13 +418,13 @@ bool CoordReader::setOptions(int& argc, char ** argv)
         coordFile=argv[i];
       } else if(arg=="-prior-precision") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _priorPrecision=atof(argv[i]);
+        _priorPrecision=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-posterior-precision") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _posteriorPrecision=atof(argv[i]);
+        _posteriorPrecision=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-cluster-count") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _clusterCount=atoi(argv[i]);
+        _clusterCount=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-no-progress") {
         delete _progress;
         _progress=nullptr;
@@ -433,11 +433,11 @@ bool CoordReader::setOptions(int& argc, char ** argv)
       } else if(arg=="-mag-declination") {
         _mode=MagDeclination;
         CoreApplication::checkOptionArg(i, argc, argv);
-        int year=atoi(argv[i]);
+        int year=CoreApplication::toInt(i, i-1, argv);
         CoreApplication::checkOptionArg(i, argc, argv);
-        int month=atoi(argv[i]);
+        int month=CoreApplication::toInt(i, i-2, argv);
         CoreApplication::checkOptionArg(i, argc, argv);
-        int day=atoi(argv[i]);
+        int day=CoreApplication::toInt(i, i-3, argv);
         _date=QDate(year, month, day);
       } else if(arg=="-out") {
         CoreApplication::checkOptionArg(i, argc, argv);
