@@ -130,28 +130,28 @@ int main(int argc, char ** argv)
       } else if(arg=="-akaike") {
         CoreApplication::checkOptionArg(i, argc, argv);
         misfitType=Akaike;
-        akaikeDof=atoi(argv[i]);
+        akaikeDof=CoreApplication::toInt(i, i-1, argv);
         forceMisfitType=true;
       } else if(arg=="-akaike-few") {
         CoreApplication::checkOptionArg(i, argc, argv);
         misfitType=AkaikeFewSamples;
-        akaikeDof=atoi(argv[i]);
+        akaikeDof=CoreApplication::toInt(i, i-1, argv);
         forceMisfitType=true;
       } else if(arg=="-o") {
         CoreApplication::checkOptionArg(i, argc, argv);
         reportFile=argv[i];
       } else if(arg=="-m-in") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        maxInputMisfit=atof(argv[i])*(1.0+1e-15);
+        maxInputMisfit=CoreApplication::toDouble(i, i-1, argv)*(1.0+1e-15);
       } else if(arg=="-m-out") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        maxOutputMisfit=atof(argv[i])*(1.0+1e-15);
+        maxOutputMisfit=CoreApplication::toDouble(i, i-1, argv)*(1.0+1e-15);
       } else if(arg=="-best-out") {
         CoreApplication::checkOptionArg(i, argc, argv);
         bestOutputCount=atoi(argv[i] );
       } else if(arg=="-n-out") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        maxOutputCount=atoi(argv[i]);
+        maxOutputCount=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-all") {
         CoreApplication::checkOptionArg(i, argc, argv);
         targets->dispersionTarget().setSelected(true);
@@ -160,7 +160,7 @@ int main(int argc, char ** argv)
         targets->ellipticityPeakTarget().setSelected(true);
         targets->refractionVpTarget().setSelected(true);
         targets->refractionVsTarget().setSelected(true);
-        double m=atof(argv[i]);
+        double m=CoreApplication::toDouble(i, i-1, argv);
         targets->dispersionTarget().setMinimumMisfit(m);
         targets->autocorrTarget().setMinimumMisfit(m);
         targets->ellipticityCurveTarget().setMinimumMisfit(m);
@@ -170,31 +170,31 @@ int main(int argc, char ** argv)
       } else if(arg=="-disp") {
         CoreApplication::checkOptionArg(i, argc, argv);
         targets->dispersionTarget().setSelected(true);
-        targets->dispersionTarget().setMinimumMisfit(atof(argv[i]));
+        targets->dispersionTarget().setMinimumMisfit(CoreApplication::toDouble(i, i-1, argv));
       } else if(arg=="-autocorr") {
         CoreApplication::checkOptionArg(i, argc, argv);
         targets->autocorrTarget().setSelected(true);
-        targets->autocorrTarget().setMinimumMisfit(atof(argv[i]));
+        targets->autocorrTarget().setMinimumMisfit(CoreApplication::toDouble(i, i-1, argv));
       } else if(arg=="-ellcurve") {
         CoreApplication::checkOptionArg(i, argc, argv);
         targets->ellipticityCurveTarget().setSelected(true);
-        targets->ellipticityCurveTarget().setMinimumMisfit(atof(argv[i]));
+        targets->ellipticityCurveTarget().setMinimumMisfit(CoreApplication::toDouble(i, i-1, argv));
       } else if(arg=="-ellpeak") {
         CoreApplication::checkOptionArg(i, argc, argv);
         targets->ellipticityPeakTarget().setSelected(true);
-        targets->ellipticityPeakTarget().setMinimumMisfit(atof(argv[i]));
+        targets->ellipticityPeakTarget().setMinimumMisfit(CoreApplication::toDouble(i, i-1, argv));
       } else if(arg=="-refravp") {
         CoreApplication::checkOptionArg(i, argc, argv);
         targets->refractionVpTarget().setSelected(true);
-        targets->refractionVpTarget().setMinimumMisfit(atof(argv[i]));
+        targets->refractionVpTarget().setMinimumMisfit(CoreApplication::toDouble(i, i-1, argv));
       } else if(arg=="-refravs") {
         CoreApplication::checkOptionArg(i, argc, argv);
         targets->refractionVsTarget().setSelected(true);
-        targets->refractionVsTarget().setMinimumMisfit(atof(argv[i]));
+        targets->refractionVsTarget().setMinimumMisfit(CoreApplication::toDouble(i, i-1, argv));
       } else if(arg=="-mt") {
         CoreApplication::checkOptionArg(i, argc, argv);
         targets->magnetoTelluricTarget().setSelected(true);
-        targets->magnetoTelluricTarget().setMinimumMisfit(atof(argv[i]));
+        targets->magnetoTelluricTarget().setMinimumMisfit(CoreApplication::toDouble(i, i-1, argv));
       } else if(arg=="-f") {
         reportAction=ReportWriter::Overwrite;
       } else if(arg=="-resume") {
