@@ -31,6 +31,7 @@
 #include <QGpCompatibility.h>
 #include <QGpCoreTools.h>
 #include <QGpCoreWave.h>
+
 #include "gpdcreportVersion.h"
 #include "gpdcreportInstallPath.h"
 
@@ -113,18 +114,18 @@ int main(int argc, char ** argv)
     if(arg[0]=='-') {
       if(arg=="-n") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        nModels=atoi(argv[i]);
+        nModels=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-m") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        maxMisfit=atof(argv[i])*(1.0+1e-15);
+        maxMisfit=CoreApplication::toDouble(i, i-1, argv)*(1.0+1e-15);
       } else if(arg=="-best") {
         randomize=false;
         CoreApplication::checkOptionArg(i, argc, argv);
-        bestModelCount=atoi(argv[i]);
+        bestModelCount=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-rand") {
         randomize=true;
         CoreApplication::checkOptionArg(i, argc, argv);
-        bestModelCount=atoi(argv[i]);
+        bestModelCount=CoreApplication::toInt(i, i-1, argv);
 #if __GNUC__ < 6 && ! defined(__clang__)
 #error
         App::log(tr("gpdcreport: old compiler, randomize option not available.\n") );
@@ -189,50 +190,50 @@ int main(int argc, char ** argv)
       } else if(arg=="-pR") {
         outputType=DispersionRayleighPhase;
         CoreApplication::checkOptionArg(i, argc, argv);
-        modeIndex=atoi(argv[i]);
+        modeIndex=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-gR") {
         outputType=DispersionRayleighGroup;
         CoreApplication::checkOptionArg(i, argc, argv);
-        modeIndex=atoi(argv[i]);
+        modeIndex=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-pL") {
         outputType=DispersionLovePhase;
         CoreApplication::checkOptionArg(i, argc, argv);
-        modeIndex=atoi(argv[i]);
+        modeIndex=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-gL") {
         outputType=DispersionLoveGroup;
         CoreApplication::checkOptionArg(i, argc, argv);
-        modeIndex=atoi(argv[i]);
+        modeIndex=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-se") {
         outputType=SignedEllipticity;
         CoreApplication::checkOptionArg(i, argc, argv);
-        modeIndex=atoi(argv[i]);
+        modeIndex=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-ae") {
         outputType=AbsoluteEllipticity;
         CoreApplication::checkOptionArg(i, argc, argv);
-        modeIndex=atoi(argv[i]);
+        modeIndex=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-aV") {
         outputType=AutocorrVertical;
         CoreApplication::checkOptionArg(i, argc, argv);
-        modeIndex=atoi(argv[i]);
+        modeIndex=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-aR") {
         outputType=AutocorrRadial;
         CoreApplication::checkOptionArg(i, argc, argv);
-        modeIndex=atoi(argv[i]);
+        modeIndex=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-aT") {
         outputType=AutocorrTransverse;
         CoreApplication::checkOptionArg(i, argc, argv);
-        modeIndex=atoi(argv[i]);
+        modeIndex=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-ring") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        ringIndexes.append(atoi(argv[i]));
+        ringIndexes.append(CoreApplication::toInt(i, i-1, argv));
       } else if(arg=="-refraVp") {
         outputType=RefractionVp;
         CoreApplication::checkOptionArg(i, argc, argv);
-        sourceIndex=atoi(argv[i]);
+        sourceIndex=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-refraVs") {
         outputType=RefractionVs;
         CoreApplication::checkOptionArg(i, argc, argv);
-        sourceIndex=atoi(argv[i]);
+        sourceIndex=CoreApplication::toInt(i, i-1, argv);
       } else {
         App::log(tr("gpdcreport: bad option %1, see --help\n").arg(argv[i]) );
         return cleanReturn(2);
@@ -314,7 +315,6 @@ int main(int argc, char ** argv)
       report->addReference();
       reports << report;
       if(!report->open()) {
-        App::log(tr("gpdcreport: cannot open file %1 for reading.\n").arg(argv[i]) );
         return cleanReturn(2);
       }
       report->synchronize();
