@@ -118,7 +118,7 @@ void Simulator::init(const QVector<Point2D>& array, double sampling)
   _gridLayer->setObjectName("wave number map");
   ColorMap& map=_gridLayer->beginColorMapChange();
   map.generateColorScale(20, ColorPalette::McNames, true);
-  map.setVLinear(0.0, 1.0);
+  map.setValues(0.0, 1.0, SamplingParameters::Linear);
   _gridLayer->endColorMapChange();
 
   _maxLayer=new NameLineLayer(waveNumMap);
