@@ -99,20 +99,27 @@ HistogramWidget::HistogramWidget(QWidget * parent)
 
   a=new QAction(tr("&Mean"), this);
   curveBut->menu()->addAction(a);
-  connect(a, SIGNAL(triggered()), this, SLOT(addMeanCurve()) );
+  connect(a, SIGNAL(triggered()), this, SLOT(addMeanCurve()));
   a=new QAction(tr("Medi&an"), this);
   curveBut->menu()->addAction(a);
-  connect(a, SIGNAL(triggered()), this, SLOT(addMedianCurve()) );
+  connect(a, SIGNAL(triggered()), this, SLOT(addMedianCurve()));
   a=new QAction(tr("Mo&de"), this);
   curveBut->menu()->addAction(a);
-  connect(a, SIGNAL(triggered()), this, SLOT(addModeCurve()) );
+  connect(a, SIGNAL(triggered()), this, SLOT(addModeCurve()));
   a=new QAction(tr("&Gaussian mixture point"), this);
   curveBut->menu()->addAction(a);
-  connect(a, SIGNAL(triggered()), this, SLOT(addGaussianMixturePoint()) );
-  a=new QAction(tr("&Gaussian mixture curve"), this);
+  connect(a, SIGNAL(triggered()), this, SLOT(addGaussianMixturePoint()));
+  a=new QAction(tr("Gaussian mixture &curve"), this);
   curveBut->menu()->addAction(a);
-  connect(a, SIGNAL(triggered()), this, SLOT(addGaussianMixtureCurve()) );
+  connect(a, SIGNAL(triggered()), this, SLOT(addGaussianMixtureCurve()));
+  a=new QAction(tr("&Pick"), this);
+  curveBut->menu()->addAction(a);
+  connect(a, SIGNAL(triggered()), this, SLOT(pick()));
+  a=new QAction(tr("&Pick and split"), this);
+  curveBut->menu()->addAction(a);
+  connect(a, SIGNAL(triggered()), this, SLOT(pickAndSplit()));
 
+  // Density plot
   w=new AxisWindow(this);
   sheet->sheet()->addObject(w);
   w->setPrintLeft(0.5);
@@ -126,16 +133,20 @@ HistogramWidget::HistogramWidget(QWidget * parent)
   _densityBandLayer->setOpacity(0.6);
   _densityBandLayer->addBand(0, 0, Brush(QColor( 205, 242, 255)));
   connect(_densityBandLayer, SIGNAL(bandPicked(double, double)), this, SLOT(setBand( double, double)));
+  _densityHistogramLayer=new HistogramLayer(w);
+  _densityHistogramLayer->setObjectName("DensityHistogram");
+  _densityHistogramLayer->setReferenceLine(new PlotLine2D);
+  _densityHistogramLayer->addLine(Pen(Qt::blue), Symbol(Symbol::NoSymbol, 1.0, Pen(Qt::NoPen), QColor(152, 152, 255)));
   _densityCurveLayer=new LineLayer(w);
-  _densityCurveLayer->setObjectName( "Density" );
+  _densityCurveLayer->setObjectName("DensityCurve");
   _densityCurveLayer->setReferenceLine(new PlotLine2D);
-  _densityCurveLayer->setReferencePen(Pen( Qt::blue));
-  _densityCurveLayer->addLine(Pen(Qt::blue), Symbol());
+  _densityCurveLayer->setReferencePen(Pen(Qt::blue));
   _densityCurveLayer->addLine(Pen(Qt::red), Symbol());
   _densityCurveLayer->addLine(Pen(Qt::green), Symbol());
   w->yAxis()->setNumberType('e');
   w->yAxis()->setNumberPrecision(1);
   w->yAxis()->setTitle("Probability density");
+  w->yAxis()->setZoomEnabled(false);
   sheet->sheet()->showObject(w);
 
   w=new AxisWindow(this);
@@ -162,6 +173,7 @@ HistogramWidget::HistogramWidget(QWidget * parent)
   w->yAxis()->setNumberType('f');
   w->yAxis()->setNumberPrecision(1);
   w->yAxis()->setTitle("Cumulative probability");
+  w->yAxis()->setZoomEnabled(false);
   sheet->sheet()->showObject(w);
 
   Settings::getWidget(this, "Histogram");
@@ -204,8 +216,8 @@ void HistogramWidget::setHistogram(Histogram2D * hist)
   if(!_hist->yTitle().isNull()) {
     _histogramLayer->graph()->yAxis()->setTitle(_hist->yTitle());
     _histogramLayer->graph()->yAxis()->setTitleInversedScale(_hist->yTitleInversedScale());
-    _densityCurveLayer->graph()->xAxis()->setTitle(_hist->yTitle());
-    _densityCurveLayer->graph()->xAxis()->setTitleInversedScale(_hist->yTitleInversedScale());
+    _densityHistogramLayer->graph()->xAxis()->setTitle(_hist->yTitle());
+    _densityHistogramLayer->graph()->xAxis()->setTitleInversedScale(_hist->yTitleInversedScale());
     _cumulativeCurveLayer->graph()->xAxis()->setTitle(_hist->yTitle());
     _cumulativeCurveLayer->graph()->xAxis()->setTitleInversedScale(_hist->yTitleInversedScale());
   }
@@ -223,7 +235,7 @@ void HistogramWidget::setHistogram(Histogram2D * hist)
     break;
   }
   _histogramLayer->graph()->yAxis()->setScaleType(hist->yScaleType());
-  _densityCurveLayer->graph()->xAxis()->setScaleType(hist->yScaleType());
+  _densityHistogramLayer->graph()->xAxis()->setScaleType(hist->yScaleType());
   _cumulativeCurveLayer->graph()->xAxis()->setScaleType(hist->yScaleType());
 
   _gaussianMixtureDensities.setHistogram(_hist);
@@ -236,7 +248,6 @@ void HistogramWidget::setHistogram(Histogram2D * hist)
 void HistogramWidget::setLimits()
 {
   TRACE;
-
   GraphContent * gc=_histogramLayer->graphContent();
   AxisWindow * w=gc->graph();
 
@@ -248,10 +259,10 @@ void HistogramWidget::setLimits()
   ColorMap pal=_histogramLayer->colorMap();
   double m=_histogramLayer->grid().maximumValue();
   if(_normalize) {
-    pal.setVLog(0.0001*m, m);
+    pal.setValues(0.0001*m, m, SamplingParameters::Log);
   } else {
     pal.generateColorScale(qRound(m+1.0), ColorPalette::McNamesClip, true);
-    pal.setVLinear(0.5, m-0.5);
+    pal.setValues(0.5, m-0.5, SamplingParameters::Linear);
   }
   _histogramLayer->setColorMap(pal);
 }
@@ -266,109 +277,71 @@ void HistogramWidget::on_xScroll_valueChanged()
     xEdit->setText(QString("%1 %2").arg(_hist->x(ix)).arg(_xUnit));
   }
 
-  PlotLine2D * gridDensityLine=static_cast<PlotLine2D *>(_densityCurveLayer->line(0));
-  PlotLine2D * gaussDensityLine=static_cast<PlotLine2D *>(_densityCurveLayer->line(1));
-  PlotLine2D * gaussMixDensityLine=static_cast<PlotLine2D *>(_densityCurveLayer->line(2));
-  PlotLine2D * gridCumulativeLine=static_cast<PlotLine2D *>(_cumulativeCurveLayer->line(0));
+  PlotLine2D * gridDensity=static_cast<PlotLine2D *>(_densityHistogramLayer->line(0));
+  PlotLine2D * gaussDensityLine=static_cast<PlotLine2D *>(_densityCurveLayer->line(0));
+  PlotLine2D * gaussMixDensityLine=static_cast<PlotLine2D *>(_densityCurveLayer->line(1));
+  PlotLine2D * gridCumulative=static_cast<PlotLine2D *>(_cumulativeCurveLayer->line(0));
   PlotLine2D * gaussCumulativeLine=static_cast<PlotLine2D *>(_cumulativeCurveLayer->line(1));
   PlotLine2D * gaussMixCumulativeLine=static_cast<PlotLine2D *>(_cumulativeCurveLayer->line(2));
 
   _densityCurveLayer->lockDelayPainting();
+  _densityHistogramLayer->lockDelayPainting();
   _cumulativeCurveLayer->lockDelayPainting();
 
-  IrregularGrid2D grid=*_hist;
-  int n=grid.y().count();
-  // If option -no-normalize is used make sure grid is normalized
-  grid.normalizeArea(YAxis);
-  // Keep a copy of the real world y axis cell widths (necessary to convert to cumulative function)
-  QVector<double> binWidth(n);
-  for(int i=0; i<n; i++) {
-    binWidth[i]=grid.height(i);
-  }
-  // Median computed on the real world values
-  double med=grid.median(YAxis, ix);
+  Histogram densityCurve=_hist->histogramAt(ix);
+  gridDensity->setCurve(densityCurve);
+  gridCumulative->setCurve(densityCurve.cumulative());
+
+  // Compute average directly on samples (exact)
+  RealStatisticalValue meanStat=_hist->mean(ix);
+  RealStatisticalValue medStat=densityCurve.median();
+  RealStatisticalValue modStat=densityCurve.mode();
+
+  double mean, stddev;
   if(_hist->ySampling() & LogScale) {
-    grid.log10(YAxis);
-    med=log10(med);
+    mean=::log(meanStat.mean());
+    stddev=::log(meanStat.stddev());
   } else if(_hist->ySampling() & InversedScale) {
-    grid.inverse(YAxis);
-    med=1.0/med;
+    mean=1.0/meanStat.mean();
+    stddev=1.0/meanStat.stddev();
+  } else {
+    mean=meanStat.mean();
+    stddev=meanStat.stddev();
   }
 
-  Curve<Point2D> curve=grid.crossSection(YAxis, ix);
-  if(_hist->ySampling() & LogScale) {
-    curve.xExp10();
-  } else if(_hist->ySampling() & InversedScale) {
-    curve.xInverse();
-  }
+  int nSimulated=10*densityCurve.count();
+  densityCurve.resample(nSimulated, densityCurve.firstX(),
+                        densityCurve.lastX(), _hist->ySampling());
+  gaussDensityLine->setCurve(densityCurve);
+  gaussCumulativeLine->setCurve(densityCurve);
+  gaussMixDensityLine->setCurve(densityCurve);
+  gaussMixCumulativeLine->setCurve(densityCurve);
 
-  gridDensityLine->setCurve(curve);
-  gaussDensityLine->setCurve(curve);
-  gaussMixDensityLine->setCurve(curve);
-  gridCumulativeLine->setCurve(curve);
-  gaussCumulativeLine->setCurve(curve);
-  gaussMixCumulativeLine->setCurve(curve);
-  ASSERT(n==gridDensityLine->curve().count());
-
-  double mean=grid.mean(YAxis, ix);
-  double stddev=sqrt(grid.variance(YAxis, ix));
-  double meddev=sqrt(grid.variance(YAxis, ix, med));
-  double mod=grid.mode(YAxis, ix);
-  double moddev=sqrt(grid.variance(YAxis, ix, mod));
-  double modFitdev=grid.varianceFit(YAxis, ix, mod);
-
-  double sum=0;
-  for(int i=0; i<n; i++) {
-    Point2D& p=gridCumulativeLine->curve()[i];
-    sum+=p.y()*binWidth.at(i);
-    p.setY(sum);
-  }
-
-  GaussDistribution gd(med, meddev);
-  for(int i=0; i<n; i++) {
-    Point2D& p=gaussDensityLine->curve()[i];
+  GaussDistribution gd(mean, stddev);
+  Densities mixd=_gaussianMixtureDensities.densities(ix);
+  for(int i=nSimulated-1; i>=0; i--) {
+    Point2D& gaussDens=gaussDensityLine->curve().constXAt(i);
+    Point2D& gaussCum=gaussCumulativeLine->curve().constXAt(i);
+    Point2D& mixDens=gaussMixDensityLine->curve().constXAt(i);
+    Point2D& mixCum=gaussMixCumulativeLine->curve().constXAt(i);
+    double x=gaussDens.x();
     if(_hist->ySampling() & LogScale) {
-      // 1/p, source: https://en.wikipedia.org/wiki/Log-normal_distribution
-      p.setY(gd.rho(log10(p.x()))/p.x());
+      gaussDens.setY(gd.rho(::log(x)));
+      gaussCum.setY(gd.part(log(x)));
+      mixDens.setY(mixd.density(log(x)));
+      mixCum.setY(mixd.cumulativeDensity(log(x)));
     } else if(_hist->ySampling() & InversedScale) {
-      p.setY(gd.rho(1.0/p.x()));
+      gaussDens.setY(gd.rho(1.0/x));
+      gaussCum.setY(gd.part(1.0/x));
+      mixDens.setY(mixd.density(1.0/x));
+      mixCum.setY(mixd.cumulativeDensity(1.0/x));
     } else {
-      p.setY(gd.rho(p.x()));
+      gaussDens.setY(gd.rho(x));
+      gaussCum.setY(gd.part(x));
+      mixDens.setY(mixd.density(x));
+      mixCum.setY(mixd.cumulativeDensity(x));
     }
   }
-  for(int i=0; i<n; i++) {
-    Point2D& p=gaussCumulativeLine->curve()[i];
-    if(_hist->ySampling() & LogScale) {
-      p.setY(gd.part(log10(p.x())));
-    } else if(_hist->ySampling() & InversedScale) {
-      p.setY(gd.part(1.0/p.x()));
-    } else {
-      p.setY(gd.part(p.x()));
-    }
-  }
-
-  Densities d=_gaussianMixtureDensities.densities(ix);
-  for(int i=0; i<n; i++) {
-    Point2D& p=gaussMixDensityLine->curve()[i];
-    if(_hist->ySampling() & LogScale) {
-      p.setY(d.density(log10(p.x())));
-    } else if(_hist->ySampling() & InversedScale) {
-      p.setY(d.density(1.0/p.x()));
-    } else {
-      p.setY(d.density(p.x()));
-    }
-  }
-  for(int i=0; i<n; i++) {
-    Point2D& p=gaussMixCumulativeLine->curve()[i];
-    if(_hist->ySampling() & LogScale) {
-      p.setY(d.cumulativeDensity(log10(p.x())));
-    } else if(_hist->ySampling() & InversedScale) {
-      p.setY(d.cumulativeDensity(1.0/p.x()));
-    } else {
-      p.setY(d.cumulativeDensity(p.x()));
-    }
-  }
-
 
   const ParallelBand& b=_densityBandLayer->band(0);
   saveBand(_lastX, b.minimum(), b.maximum());
@@ -377,44 +350,30 @@ void HistogramWidget::on_xScroll_valueChanged()
     setBand(p.x(), p.y());
   } else {
     if(_hist->ySampling() & LogScale) {
-      setBand(log10(med-meddev), log10(med+meddev));
+      setBand(log(mean-stddev), log(mean+stddev));
     } else if(_hist->ySampling() & InversedScale) {
-      setBand(1.0/(med-meddev), 1.0/(med+meddev));
+      setBand(1.0/(mean-stddev), 1.0/(mean+stddev));
     } else {
-      setBand(med-meddev, med+meddev);
+      setBand(mean-stddev, mean+stddev);
     }
   }
   _lastX=ix;
 
   _densityCurveLayer->unlock();
+  _densityHistogramLayer->unlock();
   _cumulativeCurveLayer->unlock();
 
-  if(_hist->ySampling() & LogScale) {
-    mean=pow(10.0, mean);
-    stddev=pow(10.0, stddev);
-    med=pow(10.0, med);
-    meddev=pow(10.0, meddev);
-    mod=pow(10.0, mod);
-    moddev=pow(10.0, moddev);
-    modFitdev=pow(10.0, modFitdev);
-  } else if(_hist->ySampling() & InversedScale) {
-    mean=1.0/mean;
-    stddev=1.0/stddev;
-    med=1.0/med;
-    meddev=1.0/meddev;
-    mod=1.0/mod;
-    moddev=1.0/moddev;
-    modFitdev=1.0/modFitdev;
-  }
-
   statValuesLabel->setText(tr("Mean %2%1 - Stddev %3%1\n"
                               "Median %4%1 - Meddev %5%1\n"
                               "Mode %6%1 - Moddev %7%1\n"
                               "Cumulative median misfit %10")
                             .arg(_yUnit.isEmpty() ? _yUnit : " " + _yUnit)
-                            .arg(mean, 0, 'g', 3).arg(stddev, 0, 'g', 3)
-                            .arg(med, 0, 'g', 3).arg(meddev, 0, 'g', 3)
-                            .arg(mod, 0, 'g', 3).arg(moddev, 0, 'g', 3)
+                            .arg(meanStat.mean(), 0, 'g', 6)
+                            .arg(meanStat.stddev(), 0, 'g', 6)
+                            .arg(medStat.mean(), 0, 'g', 6)
+                            .arg(medStat.stddev(), 0, 'g', 6)
+                            .arg(modStat.mean(), 0, 'g', 6)
+                            .arg(modStat.stddev(), 0, 'g', 6)
                             .arg(_hist->misfit(_hist->x(ix), gd)));
 
   Rect r;
@@ -490,7 +449,7 @@ Point2D HistogramWidget::restoreBand(int ix)
   }
   Curve<RealStatisticalPoint>& lowCurve=static_cast<RealStatisticalProxy *>(lowProp->proxy())->curve();
   Curve<RealStatisticalPoint>& highCurve=static_cast<RealStatisticalProxy *>(highProp->proxy())->curve();
-  return Point2D(lowCurve.at(x).y(), highCurve.at(x).y());
+  return Point2D(lowCurve.interpole(x).y(), highCurve.interpole(x).y());
 }
 
 void HistogramWidget::setBand(double min, double max)
@@ -574,7 +533,7 @@ void HistogramWidget::on_undoAllBut_clicked()
 void HistogramWidget::shortcutEvents(Shortcuts s)
 {
   TRACE;
-  GraphContent * gc=_densityCurveLayer->graphContent();
+  GraphContent * gc=_densityHistogramLayer->graphContent();
   const ParallelBand& b=_densityBandLayer->band(0);
   int ix=xScroll->value();
 
@@ -713,6 +672,30 @@ void HistogramWidget::addGaussianMixture(int index)
   on_xScroll_valueChanged();
 }
 
+void HistogramWidget::pick()
+{
+  TRACE;
+  addCurve(Curve<RealStatisticalPoint>());
+  CurveProperties * prop=curves->curves().last();
+  RealStatisticalLine * line=static_cast<RealStatisticalLine *>(prop->line());
+  line->setPen(Pen(Qt::NoPen));
+  line->setSymbol(Symbol(Symbol::Circle, 1.2, Qt::SolidLine, Qt::SolidPattern));
+  prop->beginCurveChange();
+  line->setCurve(_hist->pickAll(4, 0.1, 0.2));
+  prop->endCurveChange();
+}
+
+void HistogramWidget::pickAndSplit()
+{
+  TRACE;
+  Curve<RealStatisticalPoint> pc=_hist->pickAll(4, 0.1, 0.2);
+  QList<Curve<RealStatisticalPoint>> curves=pc.split(1.2, LogScale, 0.1, 5);
+  int n=curves.count();
+  for(int i=0; i<n; i++) {
+    addCurve(curves.at(i));
+  }
+}
+
 bool HistogramWidget::loadCurves(const QStringList& fileNames, const QString& formatFileName)
 {
   TRACE;
