@@ -42,35 +42,36 @@ Reader::Reader()
   _azimuthTitle=tr("Azimuth (degrees)");
   _powerTitle=tr("Power");
 
-  _dispersionColumn=3;
-  _ellipticityColumn=5;
-  _noiseColumn=6;
-  _azimuthColumn=4;
-  _powerColumn=7;
-
   _dispersionSampling.setMinimum(1.0/3500.0);
-  _dispersionSampling.setMaximum(1.0/100.0);
-  _dispersionSampling.setType(SamplingParameters::Log);
-  _dispersionSampling.setCount(200);
+  _dispersionSampling.setMaximum(1.0/50.0);
+  _dispersionSampling.setScaleType(SamplingParameters::Log);
+  _dispersionSampling.setStep(1.015);
   _ellipticitySampling.setMinimum(-90.0);
   _ellipticitySampling.setMaximum(90.0);
-  _ellipticitySampling.setCount(200);
+  _ellipticitySampling.setStep(1.0);
   _noiseSampling.setMinimum(0.01);
   _noiseSampling.setMaximum(10);
-  _noiseSampling.setType(SamplingParameters::Log);
-  _noiseSampling.setCount(50);
+  _noiseSampling.setScaleType(SamplingParameters::Log);
+  _noiseSampling.setStep(1.05);
   _azimuthSampling.setMaximum(360.0);
-  _azimuthSampling.setCount(72);
+  _azimuthSampling.setStep(5.0);
   _powerSampling.setMaximum(1.0);
   _powerSampling.setCount(200);
-  _powerSampling.setType(SamplingParameters::Log);
+  _powerSampling.setScaleType(SamplingParameters::Log);
 
   memset(_manual.flags, 0, sizeof(int)*5);
   _samples=nullptr;
   _relativeRange=0.2;
-  _nullEll=true;
+  _undefinedEll=true;
+  _splitEllSign=false;
   _noiseDev=std::numeric_limits<double>::infinity();
   _slownessDev=0.0;
+  _minV=0.0;
+  _maxV=std::numeric_limits<double>::infinity();
+  _minK=0.0;
+  _maxK=std::numeric_limits<double>::infinity();
+  _arrayKmin=0.0;
+  _arrayKmax=0.0;
 }
 
 Reader::~Reader()
@@ -78,6 +79,17 @@ Reader::~Reader()
   delete _samples;
 }
 
+void Reader::setRayleigh()
+{
+  _showPlots=Dispersion;
+  _showPlots|=Ellipticity;
+  _showPlots|=Noise;
+  _showPlots|=Azimuth;
+  _showPlots|=Power;
+  _undefinedEll=false;
+  _splitEllSign=true;
+}
+
 bool Reader::setOptions(int& argc, char ** argv)
 {
   TRACE;
@@ -87,13 +99,7 @@ bool Reader::setOptions(int& argc, char ** argv)
     QByteArray arg=argv[i];
     if(arg[0]=='-') {
       if(arg=="-R") {
-        setPattern("R");
-        _polarization="Rayleigh";
-        _nullEll=false;
-      } else if(arg=="-L") {
-        setPattern("L");
-        _polarization="Love";
-        _nullEll=true;
+        setRayleigh();
       } else if(arg=="-show-disp") {
         _showPlots|=Dispersion;
       } else if(arg=="-show-ell") {
@@ -105,66 +111,68 @@ bool Reader::setOptions(int& argc, char ** argv)
       } else if(arg=="-show-pow") {
         _showPlots|=Power;
       // --------------- DISPERSION
-      } else if(arg=="-disp-column" || arg=="-y-column") {
-        CoreApplication::checkOptionArg(i, argc, argv);
-        _dispersionColumn=atoi(argv[i]);
-        _manual.dispersion.column=true;
       } else if(arg=="-disp-sampling" || arg=="-y-sampling") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        if(strcmp(argv[i],"inversed")==0) {
-          _dispersionSampling.setInversed(true);
-        } else if(strcmp(argv[i],"log")==0) {
-          _dispersionSampling.setType(SamplingParameters::Log);
-        } else {
-          _dispersionSampling.setType(SamplingParameters::Linear);
+        if(!_dispersionSampling.setScaleType(argv[i])) {
+          App::log(tr("Error setting dispersion sampling\n"));
+          return false;
         }
         _manual.dispersion.gridScaleType=true;
       } else if(arg=="-disp-min" || arg=="-y-min") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _dispersionSampling.setMinimum(atof(argv[i]));
+        _dispersionSampling.setMinimum(CoreApplication::toDouble(i, i-1, argv));
         _manual.dispersion.minimum=true;
       } else if(arg=="-disp-max" || arg=="-y-max") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _dispersionSampling.setMaximum(atof(argv[i]));
+        _dispersionSampling.setMaximum(CoreApplication::toDouble(i, i-1, argv));
         _manual.dispersion.maximum=true;
       } else if(arg=="-disp-count" || arg=="-y-count") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _dispersionSampling.setCount(atoi(argv[i]));
-        _manual.dispersion.count=true;
+        _dispersionSampling.setCount(CoreApplication::toInt(i, i-1, argv));
+        _manual.dispersion.stepCount=true;
         if(_dispersionSampling.count()<=0) {
           App::log(tr("negative or null number of dispersion samples (option -disp-count)\n") );
           return false;
         }
+      } else if(arg=="-disp-step" || arg=="-y-step") {
+        CoreApplication::checkOptionArg(i, argc, argv);
+        _dispersionSampling.setStep(CoreApplication::toDouble(i, i-1, argv));
+        _manual.dispersion.stepCount=true;
+        if(_dispersionSampling.step()<=0) {
+          App::log(tr("negative or null step for dispersion samples (option -disp-step)\n") );
+          return false;
+        }
       } else if(arg=="-disp-title" || arg=="-y-title") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _dispersionTitle=argv[i];
       // --------------- ELLIPTICITY
-      } else if(arg=="-ell-column") {
-        CoreApplication::checkOptionArg(i, argc, argv);
-        _ellipticityColumn=atoi(argv[i]);
-        _manual.ellipticity.column=true;
       } else if(arg=="-ell-sampling") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        if(strcmp(argv[i],"inversed")==0) {
-          _ellipticitySampling.setInversed(true);
-        } else if(strcmp(argv[i],"log")==0) {
-          _ellipticitySampling.setType(SamplingParameters::Log);
-        } else {
-          _ellipticitySampling.setType(SamplingParameters::Linear);
+        if(!_ellipticitySampling.setScaleType(argv[i])) {
+          App::log(tr("Error setting ellipticity sampling\n"));
+          return false;
         }
         _manual.ellipticity.gridScaleType=true;
       } else if(arg=="-ell-min") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _ellipticitySampling.setMinimum(atof(argv[i]));
+        _ellipticitySampling.setMinimum(CoreApplication::toDouble(i, i-1, argv));
         _manual.ellipticity.minimum=true;
       } else if(arg=="-ell-max") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _ellipticitySampling.setMaximum(atof(argv[i]));
+        _ellipticitySampling.setMaximum(CoreApplication::toDouble(i, i-1, argv));
         _manual.ellipticity.maximum=true;
       } else if(arg=="-ell-count") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _ellipticitySampling.setCount(atoi(argv[i]));
-        _manual.ellipticity.count=true;
+        _ellipticitySampling.setCount(CoreApplication::toInt(i, i-1, argv));
+        _manual.ellipticity.stepCount=true;
+        if(_ellipticitySampling.count()<=0) {
+          App::log(tr("negative or null number of ellipticity samples (option -ell-count)\n") );
+          return false;
+        }
+      } else if(arg=="-ell-step") {
+        CoreApplication::checkOptionArg(i, argc, argv);
+        _ellipticitySampling.setStep(CoreApplication::toDouble(i, i-1, argv));
+        _manual.ellipticity.stepCount=true;
         if(_ellipticitySampling.count()<=0) {
           App::log(tr("negative or null number of ellipticity samples (option -ell-count)\n") );
           return false;
@@ -173,32 +181,33 @@ bool Reader::setOptions(int& argc, char ** argv)
         CoreApplication::checkOptionArg(i, argc, argv);
         _ellipticityTitle=argv[i];
       // --------------- NOISE
-      } else if(arg=="-noise-column") {
-        CoreApplication::checkOptionArg(i, argc, argv);
-        _noiseColumn=atoi(argv[i]);
-        _manual.noise.column=true;
       } else if(arg=="-noise-sampling") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        if(strcmp(argv[i],"inversed")==0) {
-          _noiseSampling.setInversed(true);
-        } else if(strcmp(argv[i],"log")==0) {
-          _noiseSampling.setType(SamplingParameters::Log);
-        } else {
-          _noiseSampling.setType(SamplingParameters::Linear);
+        if(!_noiseSampling.setScaleType(argv[i])) {
+          App::log(tr("Error setting noise sampling\n"));
+          return false;
         }
         _manual.noise.gridScaleType=true;
       } else if(arg=="-noise-min") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _noiseSampling.setMinimum(atof(argv[i]));
+        _noiseSampling.setMinimum(CoreApplication::toDouble(i, i-1, argv));
         _manual.noise.minimum=true;
       } else if(arg=="-noise-max") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _noiseSampling.setMaximum(atof(argv[i]));
+        _noiseSampling.setMaximum(CoreApplication::toDouble(i, i-1, argv));
         _manual.noise.maximum=true;
       } else if(arg=="-noise-count") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _noiseSampling.setCount(atoi(argv[i]));
-        _manual.noise.count=true;
+        _noiseSampling.setCount(CoreApplication::toInt(i, i-1, argv));
+        _manual.noise.stepCount=true;
+        if(_noiseSampling.count()<=0) {
+          App::log(tr("negative or null number of noise samples (option -noise-count)\n") );
+          return false;
+        }
+      } else if(arg=="-noise-step") {
+        CoreApplication::checkOptionArg(i, argc, argv);
+        _noiseSampling.setStep(CoreApplication::toDouble(i, i-1, argv));
+        _manual.noise.stepCount=true;
         if(_noiseSampling.count()<=0) {
           App::log(tr("negative or null number of noise samples (option -noise-count)\n") );
           return false;
@@ -207,32 +216,33 @@ bool Reader::setOptions(int& argc, char ** argv)
         CoreApplication::checkOptionArg(i, argc, argv);
         _noiseTitle=argv[i];
       // --------------- AZIMUTH
-      } else if(arg=="-az-column") {
-        CoreApplication::checkOptionArg(i, argc, argv);
-        _azimuthColumn=atoi(argv[i]);
-        _manual.azimuth.column=true;
       } else if(arg=="-az-sampling") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        if(strcmp(argv[i],"inversed")==0) {
-          _azimuthSampling.setInversed(true);
-        } else if(strcmp(argv[i],"log")==0) {
-          _azimuthSampling.setType(SamplingParameters::Log);
-        } else {
-          _azimuthSampling.setType(SamplingParameters::Linear);
+        if(!_azimuthSampling.setScaleType(argv[i])) {
+          App::log(tr("Error setting azimuth sampling\n"));
+          return false;
         }
         _manual.azimuth.gridScaleType=true;
       } else if(arg=="-az-min") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _azimuthSampling.setMinimum(atof(argv[i]));
+        _azimuthSampling.setMinimum(CoreApplication::toDouble(i, i-1, argv));
         _manual.azimuth.minimum=true;
       } else if(arg=="-az-max") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _azimuthSampling.setMaximum(atof(argv[i]));
+        _azimuthSampling.setMaximum(CoreApplication::toDouble(i, i-1, argv));
         _manual.azimuth.maximum=true;
       } else if(arg=="-az-count") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _azimuthSampling.setCount(atoi(argv[i]));
-        _manual.azimuth.count=true;
+        _azimuthSampling.setCount(CoreApplication::toInt(i, i-1, argv));
+        _manual.azimuth.stepCount=true;
+        if(_azimuthSampling.count()<=0) {
+          App::log(tr("negative or null number of azimuth samples (option -az-count)\n") );
+          return false;
+        }
+      } else if(arg=="-az-step") {
+        CoreApplication::checkOptionArg(i, argc, argv);
+        _azimuthSampling.setStep(CoreApplication::toDouble(i, i-1, argv));
+        _manual.azimuth.stepCount=true;
         if(_azimuthSampling.count()<=0) {
           App::log(tr("negative or null number of azimuth samples (option -az-count)\n") );
           return false;
@@ -241,32 +251,33 @@ bool Reader::setOptions(int& argc, char ** argv)
         CoreApplication::checkOptionArg(i, argc, argv);
         _azimuthTitle=argv[i];
       // --------------- POWER
-      } else if(arg=="-pow-column") {
-        CoreApplication::checkOptionArg(i, argc, argv);
-        _powerColumn=atoi(argv[i]);
-        _manual.power.column=true;
       } else if(arg=="-pow-sampling") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        if(strcmp(argv[i],"inversed")==0) {
-          _powerSampling.setInversed(true);
-        } else if(strcmp(argv[i],"log")==0) {
-          _powerSampling.setType(SamplingParameters::Log);
-        } else {
-          _powerSampling.setType(SamplingParameters::Linear);
+        if(!_powerSampling.setScaleType(argv[i])) {
+          App::log(tr("Error setting power sampling\n"));
+          return false;
         }
         _manual.power.gridScaleType=true;
       } else if(arg=="-pow-min") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _powerSampling.setMinimum(atof(argv[i]));
+        _powerSampling.setMinimum(CoreApplication::toDouble(i, i-1, argv));
         _manual.power.minimum=true;
       } else if(arg=="-pow-max") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _powerSampling.setMaximum(atof(argv[i]));
+        _powerSampling.setMaximum(CoreApplication::toDouble(i, i-1, argv));
         _manual.power.maximum=true;
       } else if(arg=="-pow-count") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _powerSampling.setCount(atoi(argv[i]));
-        _manual.power.count=true;
+        _powerSampling.setCount(CoreApplication::toInt(i, i-1, argv));
+        _manual.power.stepCount=true;
+        if(_powerSampling.count()<=0) {
+          App::log(tr("negative or null number of power samples (option -pow-count)\n") );
+          return false;
+        }
+      } else if(arg=="-pow-step") {
+        CoreApplication::checkOptionArg(i, argc, argv);
+        _powerSampling.setStep(CoreApplication::toDouble(i, i-1, argv));
+        _manual.power.stepCount=true;
         if(_powerSampling.count()<=0) {
           App::log(tr("negative or null number of power samples (option -pow-count)\n") );
           return false;
@@ -276,20 +287,34 @@ bool Reader::setOptions(int& argc, char ** argv)
         _powerTitle=argv[i];
       } else if(arg=="-curve-rel-range") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _relativeRange=atof(argv[i]);
+        _relativeRange=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-ref-disp-layer") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _referenceDispersionLayer=argv[i];
       } else if(arg=="-ref-ell-layer") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _referenceEllipticityLayer=argv[i];
-      } else if(arg=="-null-ell") {
-        _nullEll=true;
+      } else if(arg=="-undef-ell") {
+        _undefinedEll=true;
+      } else if(arg=="-no-split-ell-sign") {
+        _splitEllSign=false;
       } else if(arg=="-low-noise") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _noiseDev=atof(argv[i]);
+        _noiseDev=CoreApplication::toDouble(i, i-1, argv);
         CoreApplication::checkOptionArg(i, argc, argv);
-        _slownessDev=atof(argv[i]);
+        _slownessDev=CoreApplication::toDouble(i, i-2, argv);
+      } else if(arg=="-v-min") {
+        CoreApplication::checkOptionArg(i, argc, argv);
+        _minV=CoreApplication::toDouble(i, i-1, argv);
+      } else if(arg=="-v-max") {
+        CoreApplication::checkOptionArg(i, argc, argv);
+        _maxV=CoreApplication::toDouble(i, i-1, argv);
+      } else if(arg=="-k-min") {
+        CoreApplication::checkOptionArg(i, argc, argv);
+        _minK=CoreApplication::toDouble(i, i-1, argv);
+      } else if(arg=="-k-max") {
+        CoreApplication::checkOptionArg(i, argc, argv);
+        _maxK=CoreApplication::toDouble(i, i-1, argv);
       } else {
         argv[j++]=argv[i];
       }
@@ -303,10 +328,6 @@ bool Reader::setOptions(int& argc, char ** argv)
   }
   if(_showPlots==0) {
     _showPlots=Dispersion;
-    if(_polarization=="Rayleigh") {
-      _showPlots|=Ellipticity;
-      _showPlots|=Noise;
-    }
     _showPlots|=Azimuth;
     _showPlots|=Power;
   }
@@ -316,57 +337,91 @@ bool Reader::setOptions(int& argc, char ** argv)
 void Reader::helpY(ApplicationHelp * h)
 {
   h->addGroup("Dispersion Values", "disp");
-  h->addOption("-disp-column", "Column index containing dispersion values (0 is the first)." );
   h->addOption("-disp-sampling <SAMPLING>","Defines the slowness sampling type:\n"
                                         "  linear     regular sampling\n"
                                         "  log        regular sampling on a log scale\n"
                                         "  inversed   regular sampling on an inversed");
   h->addOption("-disp-min <MIN>","Minimum of range for slownness axis (default=deduced from data)");
   h->addOption("-disp-max <MAX>","Maximum of range for slowness axis (default=deduced from data)");
+  h->addOption("-disp-step <STEP>","Step between samples along slowness");
   h->addOption("-disp-count <COUNT>","Number of samples along slowness");
   h->addOption("-disp-title <TITLE>","Title for slowness axis");
   h->addGroup("Ellipticity Values", "ell");
-  h->addOption("-ell-column", "Column index containing ellipticity values (0 is the first)." );
   h->addOption("-ell-sampling <SAMPLING>","Defines the ellipticity sampling type:\n"
                                         "  linear     regular sampling\n"
                                         "  log        regular sampling on a log scale\n"
                                         "  inversed   regular sampling on an inversed");
   h->addOption("-ell-min <MIN>","Minimum of range for slownness axis (default=deduced from data)");
   h->addOption("-ell-max <MAX>","Maximum of range for ellipticity axis (default=deduced from data)");
+  h->addOption("-ell-step <STEP>","Step between samples along ellipticity");
   h->addOption("-ell-count <COUNT>","Number of samples along ellipticity");
   h->addOption("-ell-title <TITLE>","Title for ellipticity axis");
   h->addGroup("Noise Values", "ell");
-  h->addOption("-noise-column", "Column index containing noise values (0 is the first)." );
   h->addOption("-noise-sampling <SAMPLING>","Defines the noise sampling type:\n"
                                         "  linear     regular sampling\n"
                                         "  log        regular sampling on a log scale\n"
                                         "  inversed   regular sampling on an inversed");
   h->addOption("-noise-min <MIN>","Minimum of range for slownness axis (default=deduced from data)");
   h->addOption("-noise-max <MAX>","Maximum of range for noise axis (default=deduced from data)");
+  h->addOption("-noise-step <STEP>","Step between samples along noise");
   h->addOption("-noise-count <COUNT>","Number of samples along noise");
   h->addOption("-noise-title <TITLE>","Title for noise axis");
   h->addGroup("Azimuth Values", "az");
-  h->addOption("-az-column", "Column index containing azimuth values (0 is the first)." );
   h->addOption("-az-sampling <SAMPLING>","Defines the azimuth sampling type:\n"
                                         "  linear     regular sampling\n"
                                         "  log        regular sampling on a log scale\n"
                                         "  inversed   regular sampling on an inversed");
   h->addOption("-az-min <MIN>","Minimum of range for slownness axis (default=deduced from data)");
   h->addOption("-az-max <MAX>","Maximum of range for azimuth axis (default=deduced from data)");
+  h->addOption("-az-step <STEP>","Step between samples along azimuth");
   h->addOption("-az-count <COUNT>","Number of samples along azimuth");
   h->addOption("-az-title <TITLE>","Title for azimuth axis");
   h->addGroup("Power Values", "pow");
-  h->addOption("-pow-column", "Column index containing power values (0 is the first)." );
   h->addOption("-pow-sampling <SAMPLING>","Defines the power sampling type:\n"
                                         "  linear     regular sampling\n"
                                         "  log        regular sampling on a log scale\n"
                                         "  inversed   regular sampling on an inversed");
   h->addOption("-pow-min <MIN>","Minimum of range for slownness axis (default=deduced from data)");
   h->addOption("-pow-max <MAX>","Maximum of range for power axis (default=deduced from data)");
+  h->addOption("-pow-step <STEP>","Step between samples along power");
   h->addOption("-pow-count <COUNT>","Number of samples along power");
   h->addOption("-pow-title <TITLE>","Title for power axis");
 }
 
+/*!
+  Grab some processing parameter values.
+*/
+bool Reader::parse(QTextStream& s)
+{
+  TRACE;
+  QString line;
+  bool maxFormatHeader=false;
+  do {
+    line=s.readLine();
+    if(!maxFormatHeader && line.startsWith("# MAX FORMAT RELEASE")) {
+      maxFormatHeader=true;
+    }
+    if(line.startsWith("K_MIN")) {
+      _arrayKmin=line.section("=", 1, -1).toDouble();
+    } else if(line.startsWith("K_MAX")) {
+      _arrayKmax=line.section("=", 1, -1).toDouble();
+    } else if(line.startsWith("PROCESS_TYPE")) {
+      if(line.section("=", 1, -1)=="RTBF") {
+        setRayleigh();
+      }
+    } else if(line.startsWith("# abs_time frequency slowness azimuth ")) {
+      //if(!configureFKMaxFile(line)) {
+      //  return false;
+      //}
+      maxFormatHeader=false;
+    }
+  } while (maxFormatHeader || line.startsWith("#"));
+
+  // Effectively load samples, last line is provided because we may reading
+  // stdin where seek() is not supported.
+  return loadSamples(s, line);
+}
+
 /*!
 
 */
@@ -378,7 +433,7 @@ bool Reader::loadSamples(QTextStream& s, const QString& lastLine)
   bool ok=true;
   Sample sample;
   QString line;
-  if(File::isDataLine(lastLine, pattern())) {
+  if(File::isDataLine(lastLine)) {
     if(sample.read(lastLine)) {
       _samples->append(sample);
     } else {
@@ -386,7 +441,7 @@ bool Reader::loadSamples(QTextStream& s, const QString& lastLine)
     }
   }
   while(!s.atEnd()) {
-    if(File::readLine(s, line, pattern())) {
+    if(File::readLine(s, line)) {
       if(sample.read(line)) {
         _samples->append(sample);
       } else {
@@ -415,135 +470,134 @@ bool Reader::setDefaultLimits()
     x.append(it->frequency());
   }
   setDefaultXLimits(x);
-  int ny;
 
-  ny=_dispersionSampling.count();
+  double m;
   if(!_manual.dispersion.minimum) {
-    it=_samples->begin();
-    _dispersionSampling.setMinimum(it->slowness());
-    for(it++; it!=_samples->end(); it++) {
-      if(it->slowness()<_dispersionSampling.minimum()) {
-        _dispersionSampling.setMinimum(it->slowness());
+    m=std::numeric_limits<double>::infinity();
+    for(it=_samples->begin(); it!=_samples->end(); it++) {
+      if(it->slowness()<m) {
+        m=it->slowness();
       }
     }
+    _dispersionSampling.setMinimum(m);
   }
   if(!_manual.dispersion.maximum) {
-    it=_samples->begin();
-    _dispersionSampling.setMaximum(it->slowness());
-    for(it++; it!=_samples->end(); it++) {
-      if(it->slowness()>_dispersionSampling.maximum()) {
-        _dispersionSampling.setMaximum(it->slowness());
+    m=0.0;
+    for(it=_samples->begin(); it!=_samples->end(); it++) {
+      if(it->slowness()>m) {
+        m=it->slowness();
       }
     }
-  }
-  _dispersionSampling.setCount(ny);
-
-  if(_dispersionSampling.type()==SamplingParameters::Log || _dispersionSampling.isInversed()) {
-    if(_dispersionSampling.minimum()<=0.0 || _dispersionSampling.maximum()<=0.0) {
-      App::log(tr("null or negative values not allowed for log and inversed scales (dispersion axis)\n") );
-      return false;
-    }
+    _dispersionSampling.setMaximum(m);
   }
 
-  ny=_ellipticitySampling.count();
+  if(!_dispersionSampling.isValidRange(true)) {
+    App::log(tr("Error while checking dispersion axis\n"));
+    return false;
+  }
+  _dispersionSampling.includeLimits();
+
   if(!_manual.ellipticity.minimum) {
-    it=_samples->begin();
-    _ellipticitySampling.setMinimum(it->ellipticity());
-    for(it++; it!=_samples->end(); it++) {
-      if(it->ellipticity()<_ellipticitySampling.minimum()) {
-        _ellipticitySampling.setMinimum(it->ellipticity());
+    m=std::numeric_limits<double>::infinity();
+    for(it=_samples->begin(); it!=_samples->end(); it++) {
+      if(it->ellipticity()<m) {
+        m=it->ellipticity();
       }
     }
+    _ellipticitySampling.setMinimum(m);
   }
   if(!_manual.ellipticity.maximum) {
-    it=_samples->begin();
-    _ellipticitySampling.setMaximum(it->ellipticity());
-    for(it++; it!=_samples->end(); it++) {
-      if(it->ellipticity()>_ellipticitySampling.maximum()) {
-        _ellipticitySampling.setMaximum(it->ellipticity());
+    m=-std::numeric_limits<double>::infinity();
+    for(it=_samples->begin(); it!=_samples->end(); it++) {
+      if(it->ellipticity()>m) {
+        m=it->ellipticity();
       }
     }
-  }
-  _ellipticitySampling.setCount(ny);
-
-  if(_ellipticitySampling.type()==SamplingParameters::Log || _ellipticitySampling.isInversed()) {
-    if(_ellipticitySampling.minimum()<=0.0 || _ellipticitySampling.maximum()<=0.0) {
-      App::log(tr("null or negative values not allowed for log and inversed scales (ellipticity axis)\n") );
-      return false;
-    }
+    _ellipticitySampling.setMaximum(m);
   }
 
-  ny=_noiseSampling.count();
+  if(!_ellipticitySampling.isValidRange(true)) {
+    App::log(tr("Error while checking ellipticity axis\n"));
+    return false;
+  }
+  _ellipticitySampling.includeLimits();
+
   if(!_manual.noise.minimum) {
-    it=_samples->begin();
-    _noiseSampling.setMinimum(it->noise());
-    for(it++; it!=_samples->end(); it++) {
-      if(it->noise()<_noiseSampling.minimum()) {
-        _noiseSampling.setMinimum(it->noise());
+    m=std::numeric_limits<double>::infinity();
+    for(it=_samples->begin(); it!=_samples->end(); it++) {
+      if(it->noise()<m) {
+        m=it->noise();
       }
     }
+    _noiseSampling.setMinimum(m);
   }
   if(!_manual.noise.maximum) {
-    _noiseSampling.setMaximum(50.0);
-  }
-  _noiseSampling.setCount(ny);
-
-  if(_noiseSampling.type()==SamplingParameters::Log || _noiseSampling.isInversed()) {
-    if(_noiseSampling.minimum()<=0.0 || _noiseSampling.maximum()<=0.0) {
-      App::log(tr("null or negative values not allowed for log and inversed scales (noise axis)\n") );
-      _noiseSampling.setType(SamplingParameters::Linear);
-      _noiseSampling.setInversed(false);
+    if(_noiseSampling.minimum()<50.0) {
+      _noiseSampling.setMaximum(50.0);
+    } else {
+       _noiseSampling.setRange(_noiseSampling.minimum(), _noiseSampling.minimum());
     }
   }
 
-  ny=_azimuthSampling.count();
+  if(!_noiseSampling.isValidRange(true)) {
+    App::log(tr("Back to linear scale for noise axis\n"));
+    _noiseSampling.setScaleType(SamplingParameters::Linear);
+  }
+  _noiseSampling.includeLimits();
+
   if(!_manual.azimuth.minimum) {
-    it=_samples->begin();
-    _azimuthSampling.setMinimum(it->azimuth());
-    for(it++; it!=_samples->end(); it++) {
-      if(it->azimuth()<_azimuthSampling.minimum()) {
-        _azimuthSampling.setMinimum(it->azimuth());
+    m=360.0;
+    for(it=_samples->begin(); it!=_samples->end(); it++) {
+      if(it->azimuth()<m) {
+        m=it->azimuth();
       }
     }
+    _azimuthSampling.setMinimum(m);
   }
   if(!_manual.azimuth.maximum) {
-    it=_samples->begin();
-    _azimuthSampling.setMaximum(it->azimuth());
-    for(it++; it!=_samples->end(); it++) {
-      if(it->azimuth()>_azimuthSampling.maximum()) {
-        _azimuthSampling.setMaximum(it->azimuth());
+    m=0.0;
+    for(it=_samples->begin(); it!=_samples->end(); it++) {
+      if(it->azimuth()>m) {
+        m=it->azimuth();
       }
     }
-  }
-  _azimuthSampling.setCount(ny);
-
-  if(_azimuthSampling.type()==SamplingParameters::Log || _azimuthSampling.isInversed()) {
-    if(_azimuthSampling.minimum()<=0.0 || _azimuthSampling.maximum()<=0.0) {
-      App::log(tr("null or negative values not allowed for log and inversed scales (azimuth axis)\n") );
-      return false;
-    }
+    _azimuthSampling.setMaximum(m);
   }
 
-  ny=_powerSampling.count();
+  if(!_azimuthSampling.isValidRange(true)) {
+    App::log(tr("Error while checking azimuth axis\n"));
+    return false;
+  }
+  _azimuthSampling.includeLimits();
+
   if(!_manual.power.maximum) {
-    it=_samples->begin();
-    _powerSampling.setMaximum(it->power());
-    for(it++; it!=_samples->end(); it++) {
-      if(it->power()>_powerSampling.maximum()) {
-        _powerSampling.setMaximum(it->power());
+    m=0.0;
+    for(it=_samples->begin(); it!=_samples->end(); it++) {
+      if(it->power()>m) {
+        m=it->power();
       }
     }
+    _powerSampling.setMaximum(m);
   }
   if(!_manual.power.minimum) {
     _powerSampling.setMinimum(_powerSampling.maximum()*0.001);
   }
-  _powerSampling.setCount(ny);
 
-  if(_powerSampling.type()==SamplingParameters::Log || _powerSampling.isInversed()) {
-    if(_powerSampling.minimum()<=0.0 || _powerSampling.maximum()<=0.0) {
-      App::log(tr("null or negative values not allowed for log and inversed scales (power axis)\n") );
-      return false;
-    }
+  if(!_powerSampling.isValidRange(true)) {
+    App::log(tr("Error while checking power axis\n"));
+    return false;
   }
+  _powerSampling.includeLimits();
+
   return true;
 }
+
+Histogram2D * Reader::histogram(const SamplingParameters& ySampling) const
+{
+  TRACE;
+  Histogram2D * h=HistogramReader::histogram(ySampling);
+  if(h) {
+    h->setXTitle(tr("Frequency (Hz)"));
+  }
+  return h;
+}
