@@ -273,7 +273,7 @@ void HistogramWidget::setLimits()
   }
   _statWidget->gridChanged();
   ColorMap& map=_gridLayer->beginColorMapChange();
-  map.setVLinear(0, _gridLayer->grid().maximumValue()*0.5);
+  map.setValues(0, _gridLayer->grid().maximumValue()*0.5, SamplingParameters::Linear);
  _gridLayer->endColorMapChange();
   g->deepUpdate();
 }
@@ -341,7 +341,7 @@ void HistogramWidget::gridFilterCurve(int curveIndex, double sigmaFactor)
     for(int i=grid.nx()-1; i>=0; i--) {
       double x=grid.x(i);
       if(curve.isInsideRange(x)) {
-        const RealStatisticalPoint& p=curve.at(x);
+        const RealStatisticalPoint& p=curve.constAt(x);
         double dy=p.stddev()*sigmaFactor;
         _maxList->unselectValue(x, p.mean()-dy, p.mean()+dy);
       }
@@ -355,8 +355,17 @@ void HistogramWidget::on_gridSmooth_clicked()
 {
   TRACE;
   IrregularGrid2D grid=_gridLayer->grid();
+
+  // TODO make it configurable
+  SmoothingParameters smoothParam;
+  smoothParam.setMethod(SmoothingParameters::Function);
+  smoothParam.setWidthType(SmoothingParameters::Log);
+  smoothParam.setWidth(0.2);
+  smoothParam.setScaleType(SamplingParameters::Linear);
+  smoothParam.windowFunction().setShape(WindowFunctionParameters::Cosine);
+
   for(int i=grid.nx()-1; i>=0; i--) {
-    grid.smooth(YAxis, i);
+    grid.smooth(YAxis, i, smoothParam);
   }
   setGrid(grid);
 }
@@ -397,13 +406,13 @@ void HistogramWidget::normalizeGrid(IrregularGrid2D& grid)
   TRACE;
   bool logScale=isLogScale();
   if(logScale) {
-    grid.log10(YAxis);
+    grid.log(YAxis);
   }
   for(int i=grid.nx() - 1;i >= 0;i-- ) {
     grid.normalizeArea(YAxis, i);
   }
   if(logScale) {
-    grid.pow10(YAxis);
+    grid.exp(YAxis);
   }
 }
 
@@ -468,11 +477,11 @@ void HistogramWidget::addMedianCurve()
   bool logScale=isLogScale();
   IrregularGrid2D grid=_gridLayer->grid();
   if(logScale) {
-    grid.log10(YAxis);
+    grid.log(YAxis);
   }
   Curve<RealStatisticalPoint> c(grid.nx());
   for(int iFreq=0;iFreq < grid.nx();iFreq++ ) {
-    RealStatisticalPoint& p=c[iFreq];
+    RealStatisticalPoint& p=c.constXAt(iFreq); // Not true but sort afterwards
     p.setX(grid.x(iFreq));
     double a=grid.median(YAxis, iFreq);
     if(logScale) {
@@ -497,11 +506,11 @@ void HistogramWidget::addModeCurve()
   bool logScale=isLogScale();
   IrregularGrid2D grid=_gridLayer->grid();
   if(logScale) {
-    grid.log10(YAxis);
+    grid.log(YAxis);
   }
   Curve<RealStatisticalPoint> c(grid.nx());
   for(int iFreq=0;iFreq < grid.nx();iFreq++ ) {
-    RealStatisticalPoint& p=c[iFreq];
+    RealStatisticalPoint& p=c.at(iFreq);
     p.setX(grid.x(iFreq) );
     double a=grid.mode(YAxis,iFreq);
     if(logScale) {
@@ -528,7 +537,7 @@ void HistogramWidget::addMeanCurve()
   SamplingOption scale=isLogScale() ? LogScale : LinearScale;
   Curve<RealStatisticalPoint> c(grid.nx());
   for(int iFreq=0;iFreq < grid.nx();iFreq++ ) {
-    RealStatisticalPoint& p=c[iFreq];
+    RealStatisticalPoint& p=c.at(iFreq);
     p.setX(grid.x(iFreq));
     p=_maxList->meanValue(p.x(), scale);
   }
