@@ -237,21 +237,21 @@ void SpacSelector::inverseAt()
                                   // if iFreq does not exist for that curve
   AutocorrDispersion eng;
   if(index>-1) {
-    FactoryPoint p=curve.at(index);
+    FactoryPoint p=curve.constAt(index);
     p.setStddev(0.0);
     plot->line(2)->clear();
     static_cast<ModalLine *>(plot->line(2))->curve().append(p);
     // Inversion itself
     eng.setRing(_autocorr->ring(iRing));
-    double omega=2*M_PI*autocorrFactory.x()->at(iFreq);
-    QVector<double> solutions=eng.dispersion(p, omega*smin, omega*smax);
+    double omega=2.0*M_PI*autocorrFactory.x()->at(iFreq);
+    QVector<double> solutions=eng.dispersion(omega, p.y(), omega*smin, omega*smax);
     // Plot solutions
     LayerLocker ll(_limitsSolutionsLayer);
     Curve<Point2D>& invRes=limitsSolutionsCurve(0);
     invRes.clear();
     for(int i=0; i<solutions.count() && i<_maxSolutionCount; i++) {
       //printf("sol @ ring %i, %lf Hz %lf\n", iRing, autocorrFactory.x()->at(iFreq), 1/solutions.at(i));
-      invRes.append(Point2D( p.x(), solutions.at(i)) );
+      invRes.append(Point2D(p.x(), solutions.at(i)));
     }
     _gridLayer->graph()->graphContent()->deepUpdate();
     w->graphContent()->deepUpdate();
@@ -272,8 +272,8 @@ void SpacSelector::setK()
   Curve<Point2D>& resol=limitsSolutionsCurve(1);
   Curve<Point2D>& aliasing=limitsSolutionsCurve(2);
   for(int i=0;i<resol.count();i++) {
-    resol[i].setY(invWLmin/resol[i].x());
-    aliasing[i].setY(invWLmax/aliasing[i].x());
+    resol.setY(i, invWLmin/resol.x(i));
+    aliasing.setY(i, invWLmax/aliasing.x(i));
   }
   _gridLayer->graph()->graphContent()->deepUpdate();
 }
@@ -294,27 +294,26 @@ void SpacSelector::inverse()
   double smax=1.0/options->vminEdit->value();
   int nCells=options->nCells->value();
   // Get grids ready
-  IrregularGrid2D grid(omegaCount,nCells);
-  for(int i=0;i<omegaCount;i++)
+  IrregularGrid2D grid(omegaCount, nCells);
+  for(int i=0; i<omegaCount; i++) {
     grid.setX(i, autocorrFactory.x()->at(i));
-  grid.setLinear(YAxis,smin,smax);
+  }
+  grid.setLinear(YAxis, smin, smax);
   grid.init(0.0);
   IrregularGrid2D ringGrid;
   AutocorrDispersion eng;
   // Uncoupled inversion of all points
   for(int ir=0;ir<ringCount;ir++) {
     eng.setRing(_autocorr->ring(ir));
-    ringGrid.init(omegaCount,nCells,0.0);
+    ringGrid.init(omegaCount, nCells, 0.0);
     QList<ModalCurve> ringCurves=_autocorr->ringCurves(ir);
     ModalCurve& curve=ringCurves.first();
-    curve.sort(); // to remove in 3.4
     for(int iw=0; iw<omegaCount; iw++) {
       double x=autocorrFactory.x()->at(iw);
-      FactoryPoint p=curve.at(x);
+      FactoryPoint p=curve.interpole(x);
       if(p.isValid() && fabs(p.y())>0.025) {
-        double f=autocorrFactory.x()->at(iw);
-        double omega=2*M_PI*f;
-        QVector<double> solutions=eng.dispersion(p, omega*smin, omega*smax);
+        double omega=2.0*M_PI*x;
+        QVector<double> solutions=eng.dispersion(omega, p.y(), omega*smin, omega*smax);
         for(int i=0; i<solutions.count() && i<_maxSolutionCount; i++) {
           double s=solutions.at(i);
           //printf("%lf %lf\n", f, s);
@@ -345,7 +344,7 @@ void SpacSelector::setLimits()
   ColorMap pal=_gridLayer->colorMap();
   pal.generateColorScale(maxCount+2, ColorPalette::Hsv);
   pal.setColor(0, Qt::white);
-  pal.setVLinear(0.5,maxCount+0.5);
+  pal.setValues(0.5, maxCount+0.5, SamplingParameters::Linear);
   pal.setWhiteTransparent(true);
   _gridLayer->setColorMap(pal);
   _gridLayer->graph()->yAxis()->setRange(smin*0.95,smax*1.05);
@@ -407,12 +406,12 @@ void SpacSelector::selectSamples()
       if(nLimits<2) {
         ModalCurve& f=static_cast<ModalLine *>(curves->currentLayer()->line(i))->curve();
         // Extrapole to cover the whole frequency range
-        if(f.at(0).x()>freq[0]) {
-          f.append(FactoryPoint(freq[0], f.at(0).y()));
+        if(f.firstX()>freq[0]) {
+          f.append(FactoryPoint(freq[0], f.firstY()));
           f.sort();
         }
-        if(f.last().x()<freq.last()) {
-          f.append(FactoryPoint( freq.last(), f.last().y()) );
+        if(f.lastX()<freq.last()) {
+          f.append(FactoryPoint(freq.last(), f.lastY()));
           f.sort();
         }
         limits[nLimits]=f;
@@ -441,10 +440,10 @@ void SpacSelector::selectSamples()
   // Make sure that minS is always less than maxS
   int iw;
   for(iw=0;iw<omegaCount;iw++) {
-    if(minS[iw].y()>maxS[iw].y()) {
-      double tmp=maxS[iw].y();
-      maxS[iw].setY(minS[iw].y());
-      minS[iw].setY(tmp);
+    if(minS.y(iw)>maxS.y(iw)) {
+      double tmp=maxS.y(iw);
+      maxS.setY(iw, minS.y(iw));
+      minS.setY(iw, tmp);
     }
   }
   if(_selAutocorr) delete _selAutocorr;
@@ -464,16 +463,16 @@ void SpacSelector::selectSamples()
     for(int iw=0;iw<omegaCount;iw++) {
       int index=curve.indexOf(iw);
       if(index>-1) {
-        const FactoryPoint& p=curve.at(index);
+        const FactoryPoint& p=curve.constAt(index);
         double omega=2*M_PI*autocorrFactory.x()->at(iw);
-        QVector<double> solutions=eng.dispersion(p, omega*smin, omega*smax);
+        QVector<double> solutions=eng.dispersion(omega, p.y(), omega*smin, omega*smax);
         for(int i=0; i<solutions.count() && i<_maxSolutionCount; i++) {
           double s=solutions.at(i);
           //printf("sol @ ring %i, %lf Hz %lf\n", ir, autocorrFactory.x()->at(iw), 1/s);
-          if(s>minS[iw].y() && s<maxS[iw].y() &&
+          if(s>minS.y(iw) && s<maxS.y(iw) &&
               s>smin && s<smax &&
               omega*s>kmin && omega*s<kmax) {
-            selCurve[index].setValid(true);
+            selCurve.setValid(index, true);
             break;
           }
         }
@@ -488,9 +487,9 @@ void SpacSelector::selectSamples()
     log=log.arg(kmin).arg(kmax).arg(1.0/smax).arg(1.0/smin).arg(nCells);
     QString str("%1 Hz\t%2 m/s\t%3 m/s\n");
     for(int iw=0;iw<omegaCount;iw++)
-      log+=str.arg(freq[iw]).arg(minS[iw].y()).arg(maxS[iw].y());
+      log+=str.arg(freq[iw]).arg(minS.y(iw)).arg(maxS.y(iw));
     selCurve.addLog(log);
-    _selAutocorr->addCurve(selCurve);
+    _selAutocorr->add(selCurve);
     w->graphContent()->deepUpdate();
   }
 }
@@ -526,12 +525,25 @@ void SpacSelector::autoPick()
     return;
   }
 
+
+  // Smoothing parameters
+  SmoothingParameters smoothParam;
+  smoothParam.setMethod(SmoothingParameters::Function);
+  smoothParam.setWidthType(SmoothingParameters::Log);
+  smoothParam.setWidth(0.2);
+  smoothParam.setScaleType(SamplingParameters::Linear);
+  smoothParam.windowFunction().setShape(WindowFunctionParameters::Cosine);
+
   IrregularGrid2D grid=_gridLayer->grid();
-  grid.smooth(YAxis);
-  grid.smooth(YAxis);
-  Curve<Point2D> cs=grid.crossSection(YAxis, 0);
-  double iyMax=cs.maximumY(0);
-  curve=grid.followMaximumX<ModalCurve, FactoryPoint>(0, grid.nx()-1, iyMax);
+  grid.smooth(YAxis, smoothParam);
+  curve.resize(grid.nx());
+  for(int i=0; i<grid.nx(); i++) {
+    FactoryPoint& p=curve.at(i);
+    p.setX(grid.x(i));
+    int m;
+    grid.maximumValue(YAxis, i, m);
+    p.setY(grid.y(m));
+  }
   invalidateRangeK(curve);
   curves->addLine(line);
 }
@@ -573,10 +585,10 @@ void SpacSelector::adjustCurve(CurveProperties * properties, double min, double
   ModalProxy * proxy=static_cast<ModalProxy *>(properties->proxy());
   ModalCurve& c=proxy->curve();
   if(min<=0.0) {
-    min=c.at(c.minimumX()).x();
+    min=c.x(c.minimumX());
   }
   if(max<=0.0) {
-    max=c.at(c.maximumX()).x();
+    max=c.x(c.maximumX());
   }
   properties->beginCurveChange();
   adjustCurve(c, min, max, options);
@@ -598,9 +610,17 @@ void SpacSelector::adjustCurve(ModalCurve& curve, double min, double max, Sampli
   if(options & InversedScale) {
     curve.xInverse();
   }
+
+  // Smoothing parameters
+  SmoothingParameters smoothParam;
+  smoothParam.setMethod(SmoothingParameters::Function);
+  smoothParam.setWidthType(SmoothingParameters::Log);
+  smoothParam.setWidth(0.1);
+  smoothParam.setScaleType(SamplingParameters::Log);
+  smoothParam.windowFunction().setShape(WindowFunctionParameters::Cosine);
+
   IrregularGrid2D grid=_gridLayer->grid();
-  grid.smooth(YAxis);
-  grid.smooth(YAxis);
+  grid.smooth(YAxis, smoothParam);
   grid.followMaximumX<ModalCurve, FactoryPoint>(curve, min, max);
   // Re-Transforms axis according options (log and inv)
   if(options & InversedScale) {
